// ----------------------------------------------------------------------------
//
// Copyright (C) 2012 Adobe Systems Incorporated
// All Rights Reserved.
//
// NOTICE:  All information contained  herein is,  and remains the property of
// Adobe Systems Incorporated and its suppliers, if any.  The intellectual and
// technical  concepts  contained  herein  are  proprietary  to  Adobe Systems
// Incorporated  and  its suppliers  and may  be covered  by U.S.  and Foreign
// Patents, patents in process, and are protected by trade secret or copyright
// law.  Dissemination of this information or reproduction of this material is
// strictly forbidden  unless prior  written permission is obtained from Adobe
// Systems Incorporated.
//
// ----------------------------------------------------------------------------

// include this for the fallback shaders, and Helper fragment
//[Include] name=TestShaders.glsl

///////////////////////////////////////

[Source] name=AceVS
	
[Param]
	in vec4 position;
	in vec2 uv0;
	
	out vec4 positiont;
	out vec2 uv0;
	
[Code]
	// simple pass-through
	outputs.positiont = inputs.position;
	outputs.uv0       = inputs.uv0;

///////////////////////////////////////

[Fragment] name=AceBase

[Param]
	uniform sampler2D myTexture;

	in vec2 uv0;
	
	out vec4 target0;
	
[Code]
	outputs.target0 = texture2D(myTexture, inputs.uv0);
	

///////////////////////////////////////

[Source] name=ACENOPTransformPS

[Function]
	//void ACENOP23 (inout vec4 color)
	//{
	//}
	
[Ref] name=AceBase
	
[Code]

	
///////////////////////////////////////

[Source] name=ACEOptimizedCMYKtoCMYKPS

[Param]
	uniform sampler3D ACEOptCMYKtoCMYK12Table;
	
[Function]
	void ACEOptCMYKtoCMYK12 (inout vec4 color)
	{
		float val = color.a * 15.0;
		float idx = floor (val);
		float frc = val - idx;
		vec2 rg = (vec2(0.5,0.5) + 15.0 * color.rg) / 16.0;
		float b0 = (0.5 + 15.0 * color.b + 16.0 * idx) / 256.0;
		float b1 = b0 + 16.0 / 256.0;
		vec4 k0 = vec4 (rg, b0, 0.0);
		vec4 k1 = vec4 (rg, b1, 0.0);
		k0 = texture3D (ACEOptCMYKtoCMYK12Table, k0.rgb);
		k1 = texture3D (ACEOptCMYKtoCMYK12Table, k1.rgb);
		color = mix (k0, k1, frc); // mods alpha to 0
	}

[Ref] name=AceBase

[Code]

	ACEOptCMYKtoCMYK12 (outputs.target0);

///////////////////////////////////////

[Source] name=ACEOptimizedCMYKtoRGBPS

[Param]
	uniform sampler3D ACEOptCMYKtoRGB10Table;
	
[Function]
	void ACEOptCMYKtoRGB10 (inout vec4 color)
	{
		float val = color.a * 8.0;
		float idx = floor (val);
		float frc = val - idx;
		vec2 rg = (vec2(0.5,0.5) + 8.0 * color.rg) / 16.0;
		float b0 = (0.5 + 8.0 * color.b + 9.0 * idx) / 128.0;
		float b1 = b0 + 9.0 / 128.0;
		vec3 k0 = vec3 (rg, b0);
		vec3 k1 = vec3 (rg, b1);
		k0 = texture3D (ACEOptCMYKtoRGB10Table, k0).rgb;
		k1 = texture3D (ACEOptCMYKtoRGB10Table, k1).rgb;
		color.rgb = mix (k0, k1, frc);
	}
	
[Ref] name=AceBase

[Code]
	ACEOptCMYKtoRGB10 (outputs.target0);
	
///////////////////////////////////////

[Source] name=ACEOptimizedCMYKtoXYZPS

[Param]

uniform sampler1D ACEOptCMYKtoXYZ8Curves;
uniform sampler3D ACEOptCMYKtoXYZ8Table;

[Function]

void ACEOptCMYKtoXYZ8 (inout vec4 color)
{
	color = (vec4(0.5,0.5,0.5,0.5) + 255.0 * color) / 256.0;
	
	color.r = texture1D (ACEOptCMYKtoXYZ8Curves, color.r).r;
	color.g = texture1D (ACEOptCMYKtoXYZ8Curves, color.g).g;
	color.b = texture1D (ACEOptCMYKtoXYZ8Curves, color.b).b;
	color.a = texture1D (ACEOptCMYKtoXYZ8Curves, color.a).a;
	
	float val = color.a * 8.0;
	float idx = floor (val);
	float frc = val - idx;
	vec2 rg = (vec2(0.5,0.5) + 8.0 * color.rg) / 16.0;
	float b0 = (0.5 + 8.0 * color.b + 9.0 * idx) / 128.0;
	float b1 = b0 + 9.0 / 128.0;
	vec3 k0 = vec3 (rg, b0);
	vec3 k1 = vec3 (rg, b1);
	
	k0 = texture3D (ACEOptCMYKtoXYZ8Table, k0.rgb).rgb;
	k1 = texture3D (ACEOptCMYKtoXYZ8Table, k1.rgb).rgb;
	
	color.rgb = mix (k0, k1, frc);
	color.r = (color.r * 100.0 + 16.0) / 116.0;
	color.g = (255.0 * color.g - 128.0) / 500.0;
	color.b = (255.0 * color.b - 128.0) / 200.0;
	
	vec3 xyz;
	xyz.r = color.r + color.g;
	xyz.g = color.r;
	xyz.b = color.r - color.b;
	
	float brk = 6.0 / 29.0;
	float scale = 108.0 / 841.0;
	float offset = 4.0 / 29.0;
	
	if (xyz.r >= brk)
		color.r = pow (xyz.r, 3.0);
	else
		color.r = scale * (xyz.r - offset);
	if (xyz.g >= brk)
		color.g = pow (xyz.g, 3.0);
	else
		color.g = scale * (xyz.g - offset);
	if (xyz.b >= brk)
		color.b = pow (xyz.b, 3.0);
	else
		color.b = scale * (xyz.b - offset);
}

[Ref] name=AceBase

[Code]

	ACEOptCMYKtoXYZ8 (outputs.target0);

///////////////////////////////////////

[Source] name=ACEOptimizedGamutCMYKPS

[Param]
uniform sampler3D ACEOptCMYKGamut22Table;

[Function]

void ACEOptCMYKGamut22 (inout vec4 color)
{
	float val = color.a * 15.0;
	float idx = floor (val);
	float frc = val - idx;
	vec2 rg = (vec2(0.5,0.5) + 15.0 * color.rg) / 16.0;
	float b0 = (0.5 + 15.0 * color.b + 16.0 * idx) / 256.0;
	float b1 = b0 + 16.0 / 256.0;
	vec4 k0 = vec4 (rg, b0, 0.0);
	vec4 k1 = vec4 (rg, b1, 0.0);
	k0 = texture3D (ACEOptCMYKGamut22Table, k0.rgb);
	k1 = texture3D (ACEOptCMYKGamut22Table, k1.rgb);
	color = mix (k0, k1, frc);
	color = (color.r >= 128.0 / 255.0) ? vec4 (1,1,1,1) : vec4 (0,0,0,0);
}

[Ref] name=AceBase

[Code]
	ACEOptCMYKGamut22 (outputs.target0);

///////////////////////////////////////

[Source] name=ACEOptimizedGamutRGBPS

[Param]
uniform sampler3D ACEOptRGBGamut21Table;

[Function]
void ACEOptRGBGamut21 (inout vec4 color)
{
	color.rgb = saturate (color.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 31.0 * color.rgb) / 32.0;
	color.rgb = texture3D (ACEOptRGBGamut21Table, color.rgb).rgb;
	color.rgb = (color.r >= 128.0 / 255.0) ? vec3 (1,1,1) : vec3 (0,0,0);
}

[Ref] name=AceBase

[Code]
	ACEOptRGBGamut21 (outputs.target0);

///////////////////////////////////////

[Source] name=ACEOptimizedGrayTo32PS

[Param]
uniform sampler1D ACEOptGrayToRGB19Curves;

[Function]

void ACEOptGrayToRGB19 (inout vec4 color)
{
	color.rgb = saturate (color.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 255.0 * color.rgb) / 256.0;
	color.rgb = texture1D (ACEOptGrayToRGB19Curves, color.r).rgb;
}

[Ref] name=AceBase

[Code]
	ACEOptGrayToRGB19 (outputs.target0);

	
	
///////////////////////////////////////

[Source] name=ACEOptimizedGrayToXYPS

[Param]
uniform sampler1D ACEOptGrayToXYZ13Curves;

[Function]

void ACEOptGrayToXYZ13 (inout vec4 color)
{
	color.rgb = saturate (color.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 255.0 * color.rgb) / 256.0;
	color.rgb = texture1D (ACEOptGrayToXYZ13Curves, color.r).rgb;
}

[Ref] name=AceBase

[Code]
	ACEOptGrayToXYZ13 (outputs.target0);

///////////////////////////////////////

[Source] name=ACEOptimizedLabGamutPS

[Param]
uniform sampler3D ACEOptLabGamut20Table;

[Function]

void ACEOptLabGamut20 (inout vec4 color)
{
	color.rgb = saturate (color.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 32.0 * color.rgb) / 64.0;
	color.rgb = texture3D (ACEOptLabGamut20Table, color.rgb).rgb;
	color.rgb = (color.r >= 128.0 / 255.0) ? vec3 (1,1,1) : vec3 (0,0,0);
}

[Ref] name=AceBase

[Code]
	ACEOptLabGamut20 (outputs.target0);

///////////////////////////////////////

[Source] name=ACEOptimizedLabToRGBPS

[Param]
uniform sampler3D ACEOptLabToRGB2Table;
uniform sampler1D ACEOptLabToRGB2Curves;

[Function]
void ACEOptLabToRGB2 (inout vec4 color)
{
	color.rgb = saturate (color.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 24.0 * color.rgb) / 32.0;
	color.rgb = texture3D (ACEOptLabToRGB2Table, color.rgb).rgb;
	color.rgb = (vec3(0.5,0.5,0.5) + 255.0 * color.rgb) / 256.0;
	color.r = texture1D (ACEOptLabToRGB2Curves, color.r).r;
	color.g = texture1D (ACEOptLabToRGB2Curves, color.g).g;
	color.b = texture1D (ACEOptLabToRGB2Curves, color.b).b;
}

[Ref] name=AceBase

[Code]
	ACEOptLabToRGB2 (outputs.target0);
	
///////////////////////////////////////

[Source] name=ACEOptimizedMatrixRGBToRGB_1DPS

[Param]
uniform sampler1D ACEOptMatrixRGBtoRGB17InCurves;
uniform mat3 ACEOptMatrixRGBtoRGB17Matrix;
uniform sampler1D ACEOptMatrixRGBtoRGB17OutCurves;

[Function]
void ACEOptMatrixRGBtoRGB17 (inout vec4 color)
{
	color.rgb = saturate (color.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 255.0 * color.rgb) / 256.0;
	color.r = texture1D (ACEOptMatrixRGBtoRGB17InCurves, color.r).r;
	color.g = texture1D (ACEOptMatrixRGBtoRGB17InCurves, color.g).g;
	color.b = texture1D (ACEOptMatrixRGBtoRGB17InCurves, color.b).b;
	color.rgb = mul(ACEOptMatrixRGBtoRGB17Matrix, color.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 2047.0 * color.rgb) / 2048.0;
	color.r = texture1D (ACEOptMatrixRGBtoRGB17OutCurves, color.r).r;
	color.g = texture1D (ACEOptMatrixRGBtoRGB17OutCurves, color.g).g;
	color.b = texture1D (ACEOptMatrixRGBtoRGB17OutCurves, color.b).b;
}

[Ref] name=AceBase

[Code]
	ACEOptMatrixRGBtoRGB17 (outputs.target0);

	
	
///////////////////////////////////////

[Source] name=ACEOptimizedMatrixRGBToRGB_2DPS

[Param]

uniform sampler2D ACEOptMatrixRGBtoRGB17InCurves;
uniform mat3 ACEOptMatrixRGBtoRGB17Matrix;
uniform sampler2D ACEOptMatrixRGBtoRGB17OutCurves;

[Function]

void ACEOptMatrixRGBtoRGB17 (inout vec4 color)
{
	color.rgb = saturate (color.rgb);
	float texture_width = 16.0, texture_height = 16.0;
	vec3 y1 = floor((255.0 * color.rgb) / texture_width);
	vec3 x1 = floor(255.0 * color.rgb) - y1 * texture_width;
	y1 = (vec3(0.5,0.5,0.5) + y1) / texture_height;
	x1 = (vec3(0.5,0.5,0.5) + x1) / texture_width;
	color.r = texture2D (ACEOptMatrixRGBtoRGB17InCurves, vec2(x1.r, y1.r)).r;
	color.g = texture2D (ACEOptMatrixRGBtoRGB17InCurves, vec2(x1.g, y1.g)).g;
	color.b = texture2D (ACEOptMatrixRGBtoRGB17InCurves, vec2(x1.b, y1.b)).b;
	color.rgb = mul(ACEOptMatrixRGBtoRGB17Matrix, color.rgb);
	texture_width = 64.0; texture_height = 32.0;
	vec3 y2 = floor((2047.0 * color.rgb) / texture_width);
	vec3 x2 = floor(2047.0 * color.rgb) - y2 * texture_width;
	y2 = (vec3(0.5,0.5,0.5) + y2) / texture_height;
	x2 = (vec3(0.5,0.5,0.5) + x2) / texture_width;
	color.r = texture2D (ACEOptMatrixRGBtoRGB17OutCurves, vec2(x2.r, y2.r)).r;
	color.g = texture2D (ACEOptMatrixRGBtoRGB17OutCurves, vec2(x2.g, y2.g)).g;
	color.b = texture2D (ACEOptMatrixRGBtoRGB17OutCurves, vec2(x2.b, y2.b)).b;
}

[Ref] name=AceBase

[Code]
	ACEOptMatrixRGBtoRGB17 (outputs.target0);
	
///////////////////////////////////////

[Source] name=ACEOptimizedMatrixRGBToXYZ_1DPS

[Param]
uniform sampler1D ACEOptMatrixRGBtoXYZ16Curves;
uniform mat3 ACEOptMatrixRGBtoXYZ16Matrix;

[Function]
void ACEOptMatrixRGBtoXYZ16 (inout vec4 color)
{
	color.rgb = (vec3(0.5,0.5,0.5) + 255.0 * color.rgb) / 256.0;
	color.r = texture1D (ACEOptMatrixRGBtoXYZ16Curves, color.r).r;
	color.g = texture1D (ACEOptMatrixRGBtoXYZ16Curves, color.g).g;
	color.b = texture1D (ACEOptMatrixRGBtoXYZ16Curves, color.b).b;
	color.rgb = mul(ACEOptMatrixRGBtoXYZ16Matrix, color.rgb);
}

[Ref] name=AceBase

[Code]
	ACEOptMatrixRGBtoXYZ16 (outputs.target0);

	
	
	
///////////////////////////////////////

[Source] name=ACEOptimizedMatrixRGBToXYZ_2DPS

[Param]
uniform sampler2D ACEOptMatrixRGBtoXYZ16Curves;
uniform mat3 ACEOptMatrixRGBtoXYZ16Matrix;

[Function]
void ACEOptMatrixRGBtoXYZ16 (inout vec4 color)
{
	float texture_width = 16.0, texture_height = 16.0;
	vec3 y = floor((255.0 * color.rgb) / texture_width);
	vec3 x = floor(255.0 * color.rgb) - y * texture_width;
	y = (vec3(0.5,0.5,0.5) + y) / texture_height;
	x = (vec3(0.5,0.5,0.5) + x) / texture_width;
	color.r = texture2D (ACEOptMatrixRGBtoXYZ16Curves, vec2(x.r, y.r)).r;
	color.g = texture2D (ACEOptMatrixRGBtoXYZ16Curves, vec2(x.g, y.g)).g;
	color.b = texture2D (ACEOptMatrixRGBtoXYZ16Curves, vec2(x.b, y.b)).b;
	color.rgb = mul(ACEOptMatrixRGBtoXYZ16Matrix, color.rgb);
}

[Ref] name=AceBase

[Code]
	ACEOptMatrixRGBtoXYZ16 (outputs.target0);

///////////////////////////////////////

[Source] name=ACEOptimizedRGBToCMYKPS

[Param]
uniform sampler3D ACEOptRGBtoCMYK11Table;

[Function]
void ACEOptRGBtoCMYK11 (inout vec4 color)
{
	color.rgb = (vec3(0.5,0.5,0.5) + 24.0 * color.rgb) / 32.0;
	color = texture3D (ACEOptRGBtoCMYK11Table, color.rgb);
	color = 1.0 - color;
}

[Ref] name=AceBase

[Code]
	ACEOptRGBtoCMYK11 (outputs.target0);

///////////////////////////////////////

[Source] name=ACEOptimizedRGBToGrayPS

[Param]

uniform sampler1D ACEOptRGBtoGray6Curves;
uniform sampler3D ACEOptRGBtoGray6Table;

[Function]

void ACEOptRGBtoGray6 (inout vec4 color)
{
	color.rgb = saturate (color.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 255.0 * color.rgb) / 256.0;
	color.r = texture1D (ACEOptRGBtoGray6Curves, color.r).r;
	color.g = texture1D (ACEOptRGBtoGray6Curves, color.g).g;
	color.b = texture1D (ACEOptRGBtoGray6Curves, color.b).b;
	color.rgb = (vec3(0.5,0.5,0.5) + 15.0 * color.rgb) / 16.0;
	color.rgb = texture3D (ACEOptRGBtoGray6Table, color.rgb).rgb;
}

[Ref] name=AceBase

[Code]
	ACEOptRGBtoGray6 (outputs.target0);	

///////////////////////////////////////

[Source] name=ACEOptimizedRGBToLabPS

[Param]
uniform sampler1D ACEOptRGBtoLab3Curves;
uniform sampler3D ACEOptRGBtoLab3Table;

[Function]

void ACEOptRGBtoLab3 (inout vec4 color)
{
	color.rgb = saturate (color.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 255.0 * color.rgb) / 256.0;
	color.r = texture1D (ACEOptRGBtoLab3Curves, color.r).r;
	color.g = texture1D (ACEOptRGBtoLab3Curves, color.g).g;
	color.b = texture1D (ACEOptRGBtoLab3Curves, color.b).b;
	color.rgb = (vec3(0.5,0.5,0.5) + 24.0 * color.rgb) / 32.0;
	color.rgb = texture3D (ACEOptRGBtoLab3Table, color.rgb).rgb;
}

[Ref] name=AceBase

[Code]
	ACEOptRGBtoLab3 (outputs.target0);
	
///////////////////////////////////////

[Source] name=ACEOptimizedRGBToRGBPS

[Param]

uniform sampler1D ACEOptRGBtoRGB1Curves;
uniform sampler3D ACEOptRGBtoRGB1Table;

[Function]

void ACEOptRGBtoRGB1 (inout vec4 color)
{
	color.rgb = saturate (color.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 255.0 * color.rgb) / 256.0;
	color.r = texture1D (ACEOptRGBtoRGB1Curves, color.r).r;
	color.g = texture1D (ACEOptRGBtoRGB1Curves, color.g).g;
	color.b = texture1D (ACEOptRGBtoRGB1Curves, color.b).b;
	color.rgb = (vec3(0.5,0.5,0.5) + 15.0 * color.rgb) / 16.0;
	color.rgb = texture3D (ACEOptRGBtoRGB1Table, color.rgb).rgb;
}

[Ref] name=AceBase

[Code]
	ACEOptRGBtoRGB1 (outputs.target0);

///////////////////////////////////////

[Source] name=ACEOptimizedRGBToRGBOverRangePS

[Param]

uniform sampler1D ACEOptMatrixRGBtoRGBOverRange14Curves;
uniform sampler3D ACEOptMatrixRGBtoRGBOverRange14Table;

[Function]

void ACEOptMatrixRGBtoRGBOverRange14 (inout vec4 color)
{
	color.rgb = clamp (color.rgb, vec3(-1.0,-1.0,-1.0), vec3(6.0,6.0,6.0)) + 1.0;
	vec4 idx = floor (color);
	vec4 frc = color - idx;
	vec4 color0 = (idx + 0.5) / 8.0;
	vec4 color1 = (idx + 1.5) / 8.0;
	color0.r = texture1D (ACEOptMatrixRGBtoRGBOverRange14Curves, color0.r).r;
	color0.g = texture1D (ACEOptMatrixRGBtoRGBOverRange14Curves, color0.g).g;
	color0.b = texture1D (ACEOptMatrixRGBtoRGBOverRange14Curves, color0.b).b;
	color1.r = texture1D (ACEOptMatrixRGBtoRGBOverRange14Curves, color1.r).r;
	color1.g = texture1D (ACEOptMatrixRGBtoRGBOverRange14Curves, color1.g).g;
	color1.b = texture1D (ACEOptMatrixRGBtoRGBOverRange14Curves, color1.b).b;
	color.rgb = mix (color0.rgb, color1.rgb, frc.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 31.0 * color.rgb) / 32.0;
	color.rgb = texture3D (ACEOptMatrixRGBtoRGBOverRange14Table, color.rgb).rgb;
}

[Ref] name=AceBase

[Code]
	ACEOptMatrixRGBtoRGBOverRange14 (outputs.target0);
	
///////////////////////////////////////

[Source] name=ACEOptimizedRGBToXYZPS

[Param]

uniform sampler1D ACEOptMatrixRGBtoXYZ5Curves;
uniform sampler3D ACEOptMatrixRGBtoXYZ5Table;

[Function]

void ACEOptMatrixRGBtoXYZ5 (inout vec4 color)
{
	color.rgb = (vec3(0.5,0.5,0.5) + 255.0 * color.rgb) / 256.0;
	color.r = texture1D (ACEOptMatrixRGBtoXYZ5Curves, color.r).r;
	color.g = texture1D (ACEOptMatrixRGBtoXYZ5Curves, color.g).g;
	color.b = texture1D (ACEOptMatrixRGBtoXYZ5Curves, color.b).b;
	color.rgb = (vec3(0.5,0.5,0.5) + 24.0 * color.rgb) / 32.0;
	color.rgb = texture3D (ACEOptMatrixRGBtoXYZ5Table, color.rgb).rgb;
	color.r = (color.r * 100.0 + 16.0) / 116.0;
	color.g = (255.0 * color.g - 128.0) / 500.0;
	color.b = (255.0 * color.b - 128.0) / 200.0;
	vec3 xyz;
	xyz.r = color.r + color.g;
	xyz.g = color.r;
	xyz.b = color.r - color.b;
	float brk = 6.0 / 29.0;
	float scale = 108.0 / 841.0;
	float offset = 4.0 / 29.0;
	if (xyz.r >= brk)
		color.r = pow (xyz.r, 3.0);
	else
		color.r = scale * (xyz.r - offset);
	if (xyz.g >= brk)
		color.g = pow (xyz.g, 3.0);
	else
		color.g = scale * (xyz.g - offset);
	if (xyz.b >= brk)
		color.b = pow (xyz.b, 3.0);
	else
		color.b = scale * (xyz.b - offset);
}

[Ref] name=AceBase

[Code]
	ACEOptMatrixRGBtoXYZ5 (outputs.target0);

	
///////////////////////////////////////

[Source] name=ACEOptimizedXYZToCMYKPS

[Param]
uniform sampler3D ACEOptXYZtoCMYK9Table;

[Function]
void ACEOptXYZtoCMYK9 (inout vec4 color)
{
	float brk = 216.0 / 24389.0;
	float scale = 24389.0 / 27.0;
	if (color.r >= brk)
		color.r = 116.0 * pow (color.r, 1.0 / 3.0) - 16.0;
	else
		color.r = color.r * scale;
	if (color.g >= brk)
		color.g = 116.0 * pow (color.g, 1.0 / 3.0) - 16.0;
	else
		color.g = color.g * scale;
	if (color.b >= brk)
		color.b = 116.0 * pow (color.b, 1.0 / 3.0) - 16.0;
	else
		color.b = color.b * scale;
	vec3 xyz;
	xyz.r = color.g / 100.0;
	xyz.g = ((500.0 / 116.0) * (color.r - color.g) + 128.0) / 255.0;
	xyz.b = ((200.0 / 116.0) * (color.g - color.b) + 128.0) / 255.0;
	color.rgb = saturate (xyz.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 24.0 * color.rgb) / 32.0;
	color = texture3D (ACEOptXYZtoCMYK9Table, color.rgb);
	color = 1.0 - color;
}

[Ref] name=AceBase

[Code]
	ACEOptXYZtoCMYK9 (outputs.target0);

	
	
	
///////////////////////////////////////

[Source] name=ACEOptimizedXYZToMatrixRGB_1DPS

[Param]
uniform mat3 ACEOptXYZtoMatrixRGB15Matrix;
uniform sampler1D ACEOptXYZtoMatrixRGB15Curves;

[Function]

void ACEOptXYZtoMatrixRGB15 (inout vec4 color)
{
	color.rgb = mul(ACEOptXYZtoMatrixRGB15Matrix, color.rgb);
	color.rgb = saturate (color.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 2047.0 * color.rgb) / 2048.0;
	color.r = texture1D (ACEOptXYZtoMatrixRGB15Curves, color.r).r;
	color.g = texture1D (ACEOptXYZtoMatrixRGB15Curves, color.g).g;
	color.b = texture1D (ACEOptXYZtoMatrixRGB15Curves, color.b).b;
}

[Ref] name=AceBase

[Code]
	ACEOptXYZtoMatrixRGB15 (outputs.target0);
	
///////////////////////////////////////

[Source] name=ACEOptimizedXYZToMatrixRGB_2DPS

[Param]

uniform mat3 ACEOptXYZtoMatrixRGB15Matrix;
uniform sampler2D ACEOptXYZtoMatrixRGB15Curves;

[Function]

void ACEOptXYZtoMatrixRGB15 (inout vec4 color)
{
	color.rgb = mul(ACEOptXYZtoMatrixRGB15Matrix, color.rgb);
	color.rgb = saturate (color.rgb);
	float texture_width = 64.0, texture_height = 32.0;
	vec3 y = floor((2047.0 * color.rgb) / texture_width);
	vec3 x = floor(2047.0 * color.rgb) - y * texture_width;
	y = (vec3(0.5,0.5,0.5) + y) / texture_height;
	x = (vec3(0.5,0.5,0.5) + x) / texture_width;
	color.r = texture2D (ACEOptXYZtoMatrixRGB15Curves, vec2(x.r, y.r)).r;
	color.g = texture2D (ACEOptXYZtoMatrixRGB15Curves, vec2(x.g, y.g)).g;
	color.b = texture2D (ACEOptXYZtoMatrixRGB15Curves, vec2(x.b, y.b)).b;
}

[Ref] name=AceBase

[Code]
	ACEOptXYZtoMatrixRGB15 (outputs.target0);
	


///////////////////////////////////////

[Source] name=ACEOptimizedXYZToRGBPS

[Param]
uniform sampler3D ACEOptXYZtoMatrixRGB4Table;
uniform sampler1D ACEOptXYZtoMatrixRGB4Curves;

[Function]
void ACEOptXYZtoMatrixRGB4 (inout vec4 color)
{
	float brk = 216.0 / 24389.0;
	float scale = 24389.0 / 27.0;
	if (color.r >= brk)
		color.r = 116.0 * pow (color.r, 1.0 / 3.0) - 16.0;
	else
		color.r = color.r * scale;
	if (color.g >= brk)
		color.g = 116.0 * pow (color.g, 1.0 / 3.0) - 16.0;
	else
		color.g = color.g * scale;
	if (color.b >= brk)
		color.b = 116.0 * pow (color.b, 1.0 / 3.0) - 16.0;
	else
		color.b = color.b * scale;
		
	vec3 xyz;
	xyz.r = color.g / 100.0;
	xyz.g = ((500.0 / 116.0) * (color.r - color.g) + 128.0) / 255.0;
	xyz.b = ((200.0 / 116.0) * (color.g - color.b) + 128.0) / 255.0;
	
	color.rgb = saturate (xyz.rgb);
	color.rgb = (vec3(0.5,0.5,0.5) + 24.0 * color.rgb) / 32.0;
	color.rgb = texture3D (ACEOptXYZtoMatrixRGB4Table, color.rgb).rgb;
	color.rgb = (vec3(0.5,0.5,0.5) + 255.0 * color.rgb) / 256.0;
	
	color.r = texture1D (ACEOptXYZtoMatrixRGB4Curves, color.r).r;
	color.g = texture1D (ACEOptXYZtoMatrixRGB4Curves, color.g).g;
	color.b = texture1D (ACEOptXYZtoMatrixRGB4Curves, color.b).b;
}

[Ref] name=AceBase

[Code]
	ACEOptXYZtoMatrixRGB4 (outputs.target0);

	
	
///////////////////////////////////////

[Source] name=ACEStandardLabToXYZPS

[Function]

void ACEStdLabToXYZ7 (inout vec4 color)
{
	color.rgb = saturate (color.rgb);
	color.r = (color.r * 100.0 + 16.0) / 116.0;
	color.g = (255.0 * color.g - 128.0) / 500.0;
	color.b = (255.0 * color.b - 128.0) / 200.0;
	
	vec3 xyz;
	xyz.r = color.r + color.g;
	xyz.g = color.r;
	xyz.b = color.r - color.b;
	
	float brk = 6.0 / 29.0;
	float scale = 108.0 / 841.0;
	float offset = 4.0 / 29.0;
	
	if (xyz.r >= brk)
		color.r = pow (xyz.r, 3.0);
	else
		color.r = scale * (xyz.r - offset);
	if (xyz.g >= brk)
		color.g = pow (xyz.g, 3.0);
	else
		color.g = scale * (xyz.g - offset);
	if (xyz.b >= brk)
		color.b = pow (xyz.b, 3.0);
	else
		color.b = scale * (xyz.b - offset);
}

[Ref] name=AceBase

[Code]
	ACEStdLabToXYZ7 (outputs.target0);

//////////////////


[Material]  name=ACENOPTransform vs=AceVS ps=ACENOPTransformPS

[Material]  name=ACEOptimizedCMYKtoCMYK vs=AceVS ps=ACEOptimizedCMYKtoCMYKPS

[Material]  name=ACEOptimizedCMYKtoRGB vs=AceVS ps=ACEOptimizedCMYKtoRGBPS

[Material]  name=ACEOptimizedCMYKtoXYZ vs=AceVS ps=ACEOptimizedCMYKtoXYZPS

[Material]  name=ACEOptimizedGamutCMYK vs=AceVS ps=ACEOptimizedGamutCMYKPS

[Material]  name=ACEOptimizedGamutRGB vs=AceVS ps=ACEOptimizedGamutRGBPS

[Material]  name=ACEOptimizedGrayTo32 vs=AceVS ps=ACEOptimizedGrayTo32PS

[Material]  name=ACEOptimizedGrayToXY vs=AceVS ps=ACEOptimizedGrayToXYPS
	
[Material]  name=ACEOptimizedLabGamut vs=AceVS ps=ACEOptimizedLabGamutPS
	
[Material]  name=ACEOptimizedLabToRGB vs=AceVS ps=ACEOptimizedLabToRGBPS
	
[Material]  name=ACEOptimizedMatrixRGBToRGB_1D vs=AceVS ps=ACEOptimizedMatrixRGBToRGB_1DPS

[Material]  name=ACEOptimizedMatrixRGBToRGB_2D vs=AceVS ps=ACEOptimizedMatrixRGBToRGB_2DPS

[Material]  name=ACEOptimizedMatrixRGBToXYZ_1D vs=AceVS ps=ACEOptimizedMatrixRGBToXYZ_1DPS

[Material]  name=ACEOptimizedMatrixRGBToXYZ_2D vs=AceVS ps=ACEOptimizedMatrixRGBToXYZ_2DPS
	
[Material]  name=ACEOptimizedRGBToCMYK vs=AceVS ps=ACEOptimizedRGBToCMYKPS

[Material]  name=ACEOptimizedRGBToGray vs=AceVS ps=ACEOptimizedRGBToGrayPS
	
[Material]  name=ACEOptimizedRGBToLab vs=AceVS ps=ACEOptimizedRGBToLabPS

[Material]  name=ACEOptimizedRGBToRGB vs=AceVS ps=ACEOptimizedRGBToRGBPS

[Material]  name=ACEOptimizedRGBToRGBOverRange vs=AceVS ps=ACEOptimizedRGBToRGBOverRangePS
	
[Material]  name=ACEOptimizedRGBToXYZ vs=AceVS ps=ACEOptimizedRGBToXYZPS

[Material]  name=ACEOptimizedXYZToCMYK vs=AceVS ps=ACEOptimizedXYZToCMYKPS

[Material]  name=ACEOptimizedXYZToMatrixRGB_1D vs=AceVS ps=ACEOptimizedXYZToMatrixRGB_1DPS

[Material]  name=ACEOptimizedXYZToMatrixRGB_2D vs=AceVS ps=ACEOptimizedXYZToMatrixRGB_2DPS

[Material]  name=ACEOptimizedXYZToRGB vs=AceVS ps=ACEOptimizedXYZToRGBPS

[Material]  name=ACEStandardLabToXYZ vs=AceVS ps=ACEStandardLabToXYZPS

