// ----------------------------------------------------------------------------
//
// Copyright (C) 2012 Adobe Systems Incorporated
// All Rights Reserved.
//
// NOTICE:  All information contained  herein is,  and remains the property of
// Adobe Systems Incorporated and its suppliers, if any.  The intellectual and
// technical  concepts  contained  herein  are  proprietary  to  Adobe Systems
// Incorporated  and  its suppliers  and may  be covered  by U.S.  and Foreign
// Patents, patents in process, and are protected by trade secret or copyright
// law.  Dissemination of this information or reproduction of this material is
// strictly forbidden  unless prior  written permission is obtained from Adobe
// Systems Incorporated.
//
// ----------------------------------------------------------------------------

// include this for the fallback shaders, and Header
//[Include] name=TestShaders.glsl

//////////////////////////////////////////////////////////

[Source] name=ACRPipeVS

[Param]

    uniform mat4 modelToClipTfm; // uClipToModelTfm;

    in vec3 position;
    in vec4 color;
    in vec2 uv0;

    out vec4 positiont;
    out vec4 color;
    out vec2 uv0;

[Code]

    // This just processes standard vertices with pos, tex, color
    outputs.positiont = mul(vec4(inputs.position,1.0), modelToClipTfm);
    outputs.uv0   = inputs.uv0;
    outputs.color = inputs.color; // unused in vs

//////////////////////////////////////////////////////////

[Source] name=ACRPipePS

[Param]

// this is for texture2DArray, not needed in gl3
//#extension GL_EXT_gpu_shader4 : require

/*****************************************************************************/

// Relative image coordinates in [0,1].

in vec2 uv0;
out vec4 target0;


/******************************************************************************/

// Experimental GPU pipe based on OpenGL Shading Language (GLSL). Similar to the
// reference Camera Raw 6.x pipe (Process Version 2010), but has some simplifications
// and approximations.

/*****************************************************************************/

[Header]

#define kCombineStage3Denoise 0

const vec3 kWeights = vec3 (0.25, 0.5, 0.25);

// Visualization mode.
// TODO: these should be different shaders (spliced)

const int kVisNormal	= 0;				 // Normal rendering.
const int kVisClarity	= 1;				 // Show local contrast mask.
const int kVisFillLight = 2;				 // Show fill light mask.
const int kVisGrain		= 3;				 // Show grain buffer.

// consts not preserved by params parser yet, so break them out

// Texture coordinates for each row:
const float kBigTableRow_ToneCurve       = 0.5 / 6.0;
const float kBigTableRow_FillLightCurve  = 1.5 / 6.0;
const float kBigTableRow_ProfileWarp0    = 2.5 / 6.0;
const float kBigTableRow_ProfileWarp1    = 3.5 / 6.0;
const float kBigTableRow_ProfileWarp2    = 4.5 / 6.0;
const float kBigTableRow_ProfileVignette = 5.5 / 6.0;

// First and last texture coordinates for each column:

const float kBigTableCol_First =    0.5 / 4096.0;
const float kBigTableCol_Last  = 4095.5 / 4096.0;

const int kMaxGradients = 16;
const float kLocalExposureScale = 4.0;

/*****************************************************************************/

[Param]

// Uniform variables passed in from the host application. These are effectively
// constants for all pixels.

/*****************************************************************************/


uniform int uVisMode; // TODO: variant


/*****************************************************************************/

// Pixel dimensions.

uniform vec2 uDefaultCropSize;				 // Default crop area, level 0, in pixels.

uniform vec2 uStage3Size;					 // Full stage 3 image size, level 0, in pixels.

// Scale & offset to facilitate a transformation from stage 3 image-relative
// coordinates to default crop area-relative coordinates.

uniform vec2 uToDefaultCropScale;
uniform vec2 uToDefaultCropOffset;

// Ratio of source image pixel size (at pyramid level 0) to output image pixel size
// (e.g., if zoomed out to 25%, then uRenderScale will be 0.25).

uniform float uRenderScale;

/*****************************************************************************/

// Stage 3 image.

#if kCombineStage3Denoise
//uniform sampler2DArray uStage3DenoiseImage;
#else
uniform sampler2D uStage3Image;
#endif

/*****************************************************************************/

// White balance and color matrix. Note that the matrix uABC2RGB will be pre-swizzled
// via the indices implied by uWBSelectorMatrix.

uniform vec3 uWhiteBalance;

uniform mat3 uWBSelectorMatrix;

uniform mat3 uABC2RGB;						 // Camera RGB -> RIMM matrix.

uniform float uWhiteBalanceRecover;

uniform float uWhiteBalanceColorLimit;

/*****************************************************************************/

// Exposure & blacks.

uniform vec3 uRampSlope;
uniform vec3 uRampBlack;
uniform vec3 uRampRadius;
uniform vec3 uRampScale;

/*****************************************************************************/

// Saturation and vibrance.

uniform mat3 uSaturationMatrix;

uniform float uVibrance;

/*****************************************************************************/

// Big table texture: 6 rows x 4096 columns (i.e., 4096 entries per table).
//
// row 0: tone curve
// row 1: fill light curve
// row 2: profile warp curve, plane 0
// row 3: profile warp curve, plane 1
// row 4: profile warp curve, plane 2
// row 5: profile vignette curve

uniform sampler2D uBigTableTex;

/*****************************************************************************/

// Fill light.

uniform bool uHasFillLight; // TODO: variant

uniform sampler2D uFillLightMaskTex;

/*****************************************************************************/

// Clarity.

uniform bool uHasLocalContrast; // TODO: variant
uniform float uClarityAmount;
uniform sampler2D uLocalContrastMask;

/*****************************************************************************/

// Denoise.

uniform bool uHasDenoise; // TODO: variant
 
uniform float uLumaDenoiseAmount;

#if !kCombineStage3Denoise
uniform sampler2D uDenoiseMask;
#endif

uniform mat3 uDenoiseRGBtoYCC;
uniform mat3 uDenoiseYCCtoRGB;

uniform float uDenoiseCurveIntegratedScale;
uniform float uDenoiseCurveIntegratedOffset;
uniform float uDenoiseCurveNoiseScale;
uniform float uDenoiseCurveNoiseOffset;

/*****************************************************************************/

// HSL and gray mixer.

uniform bool uHasHSL; // TODO: variant
uniform bool uConvertToGray; // TODO: variant

// 4xN texture, where N is the result of HSLTunerTableCount (), which is assumed to
// be equal to the result of GrayMixerTableCount (). The first 3 rows are the HSL
// tables. The fourth row is the gray mixer table.

uniform sampler2D uHSLGrayTex;

uniform vec2 uHSLGrayCoords;

uniform vec4 uHSLGrayRows;

uniform float uToneContrastSplit;

/*****************************************************************************/

// Split tone.

uniform bool uHasSplitTone; // TODO: variant

uniform float uSplitToneBalanceMapAlpha;
uniform vec3  uSplitToneShadowFactor;
uniform vec3  uSplitToneDeltaFactor;

/*****************************************************************************/

// Post-crop vignette.

uniform bool uHasPostCropVignetteHighlightPriority; // TODO: variant
uniform bool uHasPostCropVignetteColorPriority; // TODO: variant
uniform bool uHasPostCropVignettePaintOverlay; // TODO: variant

uniform float uPostCropVignetteAmount;
uniform float uPostCropVignetteMidpoint;
uniform float uPostCropVignetteFeather;
uniform float uPostCropVignetteRoundness;
uniform float uPostCropVignetteHighlightContrast;

// For non-negative roundness.

uniform float uPostCropVignetteEllipseA;
uniform float uPostCropVignetteEllipseB;
uniform float uPostCropVignetteEllipseC;
uniform float uPostCropVignetteEllipseD;
uniform float uPostCropVignetteEllipseE;

// For negative roundness.

uniform mat3 uPostCropVignetteCropTransform;
uniform float uPostCropVignetteCropAspectRatio;

uniform bool uBlendStyleLensVignette; // TODO: variant

/*****************************************************************************/

// Grain.

uniform bool uHasGrain; // TODO: variant
uniform float uGrainAmount;

#if 0 // qOpenGLES // get sampler count down, might be able to chop out code only, and let this dead strip

//uniform sampler2D uGrainTex;

#endif

/*****************************************************************************/

// Profile-based distortion and lateral CA.

uniform bool uHasProfileWarp; // TODO: variant

uniform vec2 uProfileWarpCenter;
uniform vec2 uProfileWarpToTable;
uniform vec2 uProfileWarpFromTable;

/*****************************************************************************/

// Profile-based vignette. Use LUT for now.

uniform bool uHasLensProfileVignette; // TODO: variant

uniform vec2 uVignetteProfileScale;
uniform vec2 uVignetteProfileOffset;

/*****************************************************************************/

// Manual lens vignette.

uniform float uLensVignetteAmount;
uniform float uLensVignetteMidpoint;

/*****************************************************************************/

// Manual lateral CA.

uniform vec3 uCAScale;

/*****************************************************************************/

// Manual distortion.

uniform float uManualDistortB;

// Normalization scale factors to map from relative coordinates to a scale
// where the radius from the center to the corner is 1.0. Accounts correctly
// for the image's aspect ratio (e.g., 3:2).

uniform vec2 uNormalizeDistance;

/*****************************************************************************/

// Perspective.

uniform mat3 uPerspectiveMatrix;			 // Backward perspective matrix.

/*****************************************************************************/

// Simple monitor profile.

//uniform mat3  uRIMM2Monitor;				 // RIMM -> Monitor RGB matrix.
//uniform float uMonitorGamma;				 // Monitor gamma.

/*****************************************************************************/

// Shadow & highlight warnings.

uniform bool uShowShadowWarning; // TDOO: variant
uniform bool uShowHighlightWarning; // TODO: variant

/*****************************************************************************/

// Color tables.

uniform bool uHasHueSatMap; // TODO: variant
uniform bool uHasLookTable; // TODO: variant

uniform sampler3D uHueSatMap;
uniform sampler3D uLookTable;

uniform vec3 uHueSatMapCoordsFirst;
uniform vec3 uHueSatMapCoordsLast;

uniform vec3 uLookTableCoordsFirst;
uniform vec3 uLookTableCoordsLast;

/*****************************************************************************/

// Gradients.

// TODO: this is 16 * 9 = 144 constants, consider a texture for this ?
//  can pack the 8 float arrays into 2 vec4
 

uniform bool uHasGradients; // TODO: variant

uniform int uNumGradients;					 // <= kMaxGradients

uniform vec4 uGradients[kMaxGradients];

uniform vec4 uGradientsWeights[kMaxGradients*2];

//uniform float uLocalExposureGradientWeights [kMaxGradients];
//uniform float uLocalClarityGradientWeights [kMaxGradients];
//uniform float uLocalSaturationGradientWeights [kMaxGradients];
//uniform float uLocalContrastGradientWeights [kMaxGradients];
//uniform float uLocalLuminanceGradientWeights [kMaxGradients];
//uniform float uLocalBlueYellowGradientWeights [kMaxGradients];
//uniform float uLocalGreenMagentaGradientWeights [kMaxGradients];
//uniform float uLocalNoiseWeights [kMaxGradients];



// Add other channels:

// ...

[Function]

/*****************************************************************************/

// It is assumed that the inputs pt is in relative coordinates, relative to the full
// stage 3 image area (i.e., the origin is the top-left of the full stage 3 image).
// This routine transforms the coordinates to be relative to the default crop area,
// which is often slightly smaller than the stage 3 image area.

vec2 ToDefaultCrop (vec2 pt)
	{

	return (pt * uToDefaultCropScale) + uToDefaultCropOffset;

	}

/*****************************************************************************/

float NormalizedSquaredDistanceFromCenter (vec2 pt)
	{

	vec2 diff = (pt - vec2 (0.5, 0.5)) * uNormalizeDistance;

	return dot (diff, diff);

	}

/*****************************************************************************/

float NormalizedSquaredPinnedDistanceFromCenter (vec2 pt)
	{

	return min (NormalizedSquaredDistanceFromCenter (pt),
				1.0);
	
	}

/*****************************************************************************/

vec3 Clip01 (vec3 rgb)
	{
    // TODO: use saturate
    
	return clamp (rgb,
				  vec3 (0.0, 0.0, 0.0),
				  vec3 (1.0, 1.0, 1.0));

	}

/*****************************************************************************/

// Maps (-inf,+inf) to (-1,+1).

float MapInfRangeToSignedNormRange (float x)
	{
	
	if (x < 0.0)
		{
		
		return pow (2.0, x) - 1.0;
		
		}

	else if (x > 0.0)
		{
		
		return 1.0 - pow (2.0, -x);
		
		}

	return 0.0;
	
	}

/*****************************************************************************/

vec4 BigTableLookup (float index,
					 float rowOffset)
	{

	index = mix (kBigTableCol_First,
				 kBigTableCol_Last,
				 index); 
	
	return texture2D (uBigTableTex, 
					  vec2 (index, rowOffset));
	
	}

/*****************************************************************************/

#if 1

float sRGB_Evaluate (float x)
	{

	if (x <= 0.0031308)
		return x * 12.92;
		
	else
		return 1.055 * pow (x, 1.0 / 2.4) - 0.055;
	
	}

vec3 sRGB_Evaluate (vec3 x)
	{

	return vec3 (sRGB_Evaluate (x.r),
				 sRGB_Evaluate (x.g),
				 sRGB_Evaluate (x.b));

	}

float sRGB_EvaluateInverse (float y)
	{

	if (y <= 0.0031308 * 12.92)
		return y * (1.0 / 12.92);
		
	else
		return pow ((y + 0.055) * (1.0 / 1.055), 2.4);
	
	}

vec3 sRGB_EvaluateInverse (vec3 x)
	{

	return vec3 (sRGB_EvaluateInverse (x.r),
				 sRGB_EvaluateInverse (x.g),
				 sRGB_EvaluateInverse (x.b));

	}

#else 

// TODO: Vectorize the source above, 1 vec pow vs. 3

#define sRGB(x) \
    if (x <= 0.0031308) \
		return x * 12.92; \
	else \
		return 1.055 * pow (x, 1.0 / 2.4) - 0.055; 

 #define sRGBInverse(x) \
    if (y <= 0.0031308 * 12.92) \
		return y * (1.0 / 12.92); \
	else \
		return pow ((y + 0.055) * (1.0 / 1.055), 2.4);
	
	
float sRGB_Evaluate (float x)
	{
    sRGB(x)
	}

vec3 sRGB_Evaluate (vec3 x)
	{
    sRGB(x)
	}

float sRGB_EvaluateInverse (float y)
	{
	sRGBInverse(y)
	}

vec3 sRGB_EvaluateInverse (vec3 y)
	{
	sRGBInverse(y)
	}

#endif

/*****************************************************************************/

// Based on DNG_RGBtoHSV (see dng_utils.h).

vec3 RGBtoHSV (vec3 src)
	{

	float h, s, v;
	
	v = max (src.r, max (src.g, src.b));

	float gap = v - min (src.r, min (src.g, src.b));
	
	if (gap > 0.0)
		{

		if (src.r == v)
			{
			
			h = (src.g - src.b) / gap;
			
			if (h < 0.0)
				{
				h += 6.0;
				}
				
			}
			
		else if (src.g == v) 
			{
			h = 2.0 + (src.b - src.r) / gap;
			}
			
		else
			{
			h = 4.0 + (src.r - src.g) / gap;
			}
			
		s = gap / v;
		
		}
		
	else
		{
		h = 0.0;
		s = 0.0;
		}

	return vec3 (h, s, v);
	
	}

/*****************************************************************************/

// Based on DNG_HSVtoRGB (see dng_utils.h).

vec3 HSVtoRGB (vec3 hsv)
	{

	float r, g, b;

	float h = hsv.x;
	float s = hsv.y;
	float v = hsv.z;

	if (s > 0.0)
		{
		
		if (h < 0.0)
			h += 6.0;
			
		if (h >= 6.0)
			h -= 6.0;
			
		float i = floor (h);

		float f = h - i;
		
		float p = v * (1.0 - s);

		float q = v * (1.0 - s * f);

		float t = (v * (1.0 - s * (1.0 - f)));

		if (i == 0.0)
			{
			r = v; g = t; b = p;
			}

		else if (i == 1.0)
			{
			r = q; g = v; b = p;
			}

		else if (i == 2.0)
			{
			r = p; g = v; b = t;
			}
		
		else if (i == 3.0)
			{
			r = p; g = q; b = v;
			}

		else if (i == 4.0)
			{
			r = t; g = p; b = v;
			}

		else
			{
			r = v; g = p; b = q;
			}
		
		}
		
	else
		{
		r = v;
		g = v;
		b = v;
		}

	return vec3 (r, g, b);
	
	}

/*****************************************************************************/

float DoProfileLensVignette (vec2 pt)
	{

	// Map pre-warp relative coordinates to squared radius. The uVignetteProfileScale
	// and uVignetteProfileOffset constants account for the fact that the optical
	// center may not correspond to the image center.

	vec2 dxdy = pt * uVignetteProfileScale + uVignetteProfileOffset;

	float rr = dot (dxdy, dxdy);

	// Let the hardware deal with restricting the useful range to [0, 1], i.e., via
	// the clamp-to-edge behavior.

	return BigTableLookup (rr, kBigTableRow_ProfileVignette).x;

	}

/*****************************************************************************/

float DoManualLensVignette (float rr,
							float amount,
							float midpoint,
							float amountScale)
	{

	// Vignette math borrowed from cr_vignette.cpp.
	
	float power = 1.0 + 4.0 * midpoint;

	float scale = pow (2.0, abs (amount) * amountScale) - 1.0;

	float y = 1.0 + scale * pow (rr, power);

	if (amount < 0.0)
		{

		y = 1.0 / y;

		}

	return y;

	}

/*****************************************************************************/

float DoManualLensVignette (float rr)
	{

	return DoManualLensVignette (rr,
								 uLensVignetteAmount,
								 uLensVignetteMidpoint,
								 2.0);

	}

/*****************************************************************************/

vec3 DoLensVignette (vec3 image, 
					 vec2 pt)
	{

	float rr = NormalizedSquaredPinnedDistanceFromCenter (pt);

	float scale = DoManualLensVignette (rr);

	if (uHasLensProfileVignette)
		{
		scale *= DoProfileLensVignette (pt);
		}

	scale = max (scale, 0.0);

	return image * scale;
	
	}

/*****************************************************************************/

vec2 DoPerspective (vec2 pt)
	{

	// Copied from cr_perspective.cpp.
	
	const float kPerspectiveMaxScale64 = 5.0;
	const float kPerspectiveMinScale64 = 1.0 / kPerspectiveMaxScale64;

	vec3 xyh = uPerspectiveMatrix * vec3 (pt, 1.0);

	xyh.z = clamp (xyh.z, 
				   kPerspectiveMinScale64,
				   kPerspectiveMaxScale64);

	float s = 1.0 / xyh.z;

	xyh *= s;

	pt = xyh.xy;

	return pt;
	
	}

/*****************************************************************************/

vec2 DoDistort (vec2 pt,
				float radiusScale)
	{

	// Subtract center, scale the deltas, and add the center back.

	const vec2 center = vec2 (0.5, 0.5);

	return center + (radiusScale * (pt - center));

	}

/*****************************************************************************/

vec2 DoManualDistort (vec2 pt)
	{

	// Manual distortion is computed relative to the default crop area (not the full
	// stage 3 image area).

	vec2 ptDefaultCrop = ToDefaultCrop (pt);

	float rr = NormalizedSquaredPinnedDistanceFromCenter (ptDefaultCrop);

	float scale = (1.0 - uManualDistortB) + uManualDistortB * rr;

	pt = DoDistort (pt, 
					scale);

	return pt;
	
	}

/*****************************************************************************/

vec2 DoProfileDistort (vec2 pt,
					   float rowOffset)
	{

	// Profile-based distortion is computed relative to the full stage 3 image area
	// (not the default crop area).

	vec2 diff = (pt - uProfileWarpCenter) * uProfileWarpToTable;

	float rr = dot (diff, diff);

	float r = sqrt (rr);

	vec2 ratio = BigTableLookup (r, rowOffset).xx;

	diff = diff * ratio;

	return uProfileWarpCenter + (diff * uProfileWarpFromTable);

	}

/*****************************************************************************/

// Note that 'pt' is an inout parameter. On inputs it is the destination (ideal,
// corrected) coordinate in the range [0,1]. On output it is the source (real,
// uncorrected) coordinate.

vec3 GetStage3 (vec2 pt,
				out vec2 ptR,
				out vec2 ptG,
				out vec2 ptB)
	{

	// Work backwards from dst (ideal, corrected) to src (real, uncorrected).

	// Perspective.

	pt = DoPerspective (pt);

	// Manual distortion.

	pt = DoManualDistort (pt);

	// Profile-based warp.

	ptR = pt;
	ptG = pt;
	ptB = pt;

	if (uHasProfileWarp)
		{

		ptR = DoProfileDistort (pt, kBigTableRow_ProfileWarp0);
		ptG = DoProfileDistort (pt, kBigTableRow_ProfileWarp1);
		ptB = DoProfileDistort (pt, kBigTableRow_ProfileWarp2);

		// BULLSHIT: Effectively disable lateral CA adjustment in the shader.

		ptR = mix (ptR, ptG, vec2 (0.99999));
		ptB = mix (ptB, ptG, vec2 (0.99999));

		}

	// Manual lateral CA.

	/*
	ptR = DoDistort (ptR, uCAScale.r);
	ptG = DoDistort (ptG, uCAScale.g);
	ptB = DoDistort (ptB, uCAScale.b);
	*/

	// BULLSHIT: Effectively disable lateral CA adjustment in the shader.

	ptR += 0.0000001* DoDistort (ptR, uCAScale.r);
	ptG += 0.0000001* DoDistort (ptG, uCAScale.g);
	ptB += 0.0000001* DoDistort (ptB, uCAScale.b);

	// Get image.

#if kCombineStage3Denoise
	vec3 stage3 = vec3 (texture2DArray (uStage3DenoiseImage, vec3 (ptR, 0)).r,
						texture2DArray (uStage3DenoiseImage, vec3 (ptG, 0)).g,
						texture2DArray (uStage3DenoiseImage, vec3 (ptB, 0)).b);
#else
	vec3 stage3 = vec3 (texture2D (uStage3Image, ptR).r,
						texture2D (uStage3Image, ptG).g,
						texture2D (uStage3Image, ptB).b);
#endif
	return stage3;

	}

/*****************************************************************************/

// Ramp stage (exposure & blacks). Instead of using a 16-bit table per reference
// implementation, just do the math directly.

vec3 DoRamp (vec3 x)
	{

	vec3 y = x - (uRampBlack - uRampRadius);

	vec3 result = uRampScale * y * y;

	vec3 result2 = min ((x - uRampBlack) * uRampSlope, 
						vec3 (1.0));

	vec3 result1 = vec3 (0.0);

	result = mix (result,
				  result2,
				  vec3 (greaterThanEqual (x,
										  uRampBlack + uRampRadius)));

	result = mix (result,
				  result1, 
				  vec3 (lessThanEqual (x,
									   uRampBlack - uRampRadius)));

	return result;
	
	}

/*****************************************************************************/

vec3 DoSaturation (vec3 rgb)
	{
	
	vec3 temp = mul(uSaturationMatrix, rgb);

	temp = Clip01 (temp);

	return temp;
	
	}

/*****************************************************************************/

vec3 DoRGBtoMMH (vec3 rgb)
	{

	float r = rgb.r;
	float g = rgb.g;
	float b = rgb.b;

	// Mirror 0: Force g >= b

	bool mirror0 = (b > g);

	if (mirror0)
		{

		float temp = g;

		g = b;
		b = temp;

		}

	// Mirror 1: Force r >= b

	bool mirror1 = (b > r);

	if (mirror1)
		{

		float temp = r;

		r = b;
		b = temp;

		}

	// Mirror 2: Force r >= g

	bool mirror2 = (g > r);

	if (mirror2)
		{

		float temp = g;

		g = r;
		r = temp;

		}

	float hue;

	if (r == b)
		{
		hue = 0.0;
		}
	else
		{
		hue = (g - b) / (r - b);
		}

	if (mirror2)
		{
		hue = 2.0 - hue;
		}

	if (mirror1)
		{
		hue = 4.0 - hue;
		}

	if (mirror0)
		{
		hue = 6.0 - hue;
		}

	return vec3 (b, r, hue);
	
	}

/*****************************************************************************/

vec3 DoMMHtoRGB (vec3 mmh)
	{

	float hue = mmh.z;

	float r = mmh.y;
	float b = mmh.x;

	// We might have a hue angle that has been bumped up once for reds being low and
	// again for being off the end.

	if (hue > 6.0)
		{
		hue -= 6.0;
		}

	if (hue > 6.0)
		{
		hue -= 6.0;
		}

	bool mirror0 = (hue > 3.0);

	if (mirror0)
		{
		hue = 6.0 - hue;
		}

	bool mirror1 = (hue > 2.0);

	if (mirror1)
		{
		hue = 4.0 - hue;
		}

	bool mirror2 = (hue > 1.0);

	if (mirror2)
		{
		hue = 2.0 - hue;
		}

	float g = b + (r - b) * hue;

	if (mirror2)
		{

		float temp = g;

		g = r;
		r = temp;

		}

	if (mirror1)
		{

		float temp = r;

		r = b;
		b = temp;

		}

	if (mirror0)
		{

		float temp = g;

		g = b;
		b = temp;

		}

	return vec3 (r, g, b);

	}

/*****************************************************************************/

// this only makes sense to run at the tail of the ACR shader if there is only one monitor
//   if more than one, then need to apply this to the target as it's copied to the backbuffer for each monitor
//   best not to bake this into the saved image.  I want to keep ACE shader uploads separate from the ACR shader.
//   that means that we run through the image at the end with the ACE shader (ping-pong or to backbuffer).
//  I need the image cached to not have monitor calibration (possibly 16f?).
 
//vec3 DoColorConvert(vec3 image)
//    {
//    image = mul(image, uRIMM2Monitor); // switch to pre-mul
//    image = Clip01 (image);
//    image = pow (image, vec3 (uMonitorGamma));
//    }

vec3 DoColorConvert(vec3 image)
    {
    return image;
    }

/*****************************************************************************/

// Skin protection works on hues shifted by one sextant and stored using a 0..6
// scale. We recognize a min and max value for a central range together with
// falloffs on the man and max sides.

const float kMinSkinHue = -60.0;
const float kMinCentralSkinHue = 0.0;											
const float kMaxCentralSkinHue = 30.0;										
const float kMaxSkinHue = 45.0;												

const float kMinScaledSkinHue		 = ( kMinSkinHue		+ 60.0 ) / 60.0;	
const float kMinScaledCentralSkinHue = ( kMinCentralSkinHue + 60.0 ) / 60.0;	
const float kMaxScaledCentralSkinHue = ( kMaxCentralSkinHue + 60.0 ) / 60.0;	
const float kMaxScaledSkinHue		 = ( kMaxSkinHue		+ 60.0 ) / 60.0;	

const float kMinSideFalloff = 1.0 / (kMinScaledCentralSkinHue - kMinScaledSkinHue); 
const float kMaxSideFalloff = 1.0 / (kMaxScaledSkinHue - kMaxScaledCentralSkinHue); 
	
/*****************************************************************************/

vec3 DoIncreaseVibrance (vec3 minMaxHue,
						 float vibranceShift)
	{
	
	// We use different gains for saturation in skin tones and elsewhere.

	const float kMaxSaturationGain = 6.0;
	const float kMaxSkinSaturationGain = 1.75;
	
	const float kStandardFactor = 1.0 - ( 1.0 / kMaxSaturationGain	   );
	const float kSkinFactor		= 1.0 - ( 1.0 / kMaxSkinSaturationGain );
	const float kSkinProtectionSlope = kSkinFactor - kStandardFactor;

	float minValue = minMaxHue.x;
	float maxValue = minMaxHue.y;

	// We measure a value akin to saturation by taking the delta between maxValue and minValue

	float gap  = maxValue - minValue;

	if (gap == 0.0)
		{
		return minMaxHue;
		}

	float saturation = gap / maxValue;

	float shiftFactor = vibranceShift;

	// We protect skin tones by masking on low saturation colors in a particular
	// hue range.

	float hue = minMaxHue.z;

	// Begin CALC_SKIN_PROTECTION

	float shiftedHue = hue + 1.0;

	if (6.0 <= shiftedHue)
		shiftedHue -= 6.0;

	float minSide = clamp (1.0 - (kMinScaledCentralSkinHue - shiftedHue) * kMinSideFalloff, 
						   0.0,
						   1.0);	

	float maxSide = clamp (1.0 - (shiftedHue - kMaxScaledCentralSkinHue) * kMaxSideFalloff, 
						   0.0,
						   1.0);

	float protection = min (minSide, 
							maxSide);

	protection *= (1.0 - saturation * saturation);							

	// End CALC_SKIN_PROTECTION

	// Masked by protection, multiply shift factor with itself. This means that
	// in skin tones, not only is vibrance weaker in its total effect, it kicks
	// in more slowly as well.

	shiftFactor = shiftFactor * (shiftFactor * protection + 1.0 - protection);

	float saturationFactor = kStandardFactor + kSkinProtectionSlope * protection;
		// Up to a 6x gain in saturation. Flesh tones are much restricted
		// in what they can get.

	saturationFactor *= shiftFactor;
		// The shift factor is the processed parameter value.

	saturationFactor *= 1.0 - minValue;
		// Limit the effect in bright highlights because we don't want
		// to fight with issues from highlight recovery.

	// We don't want to apply the effect in the deepest shadows because
	// most of the information there is just noise.

	float shadowRolloff = min ( 1.0, 16.0 * maxValue );
	shadowRolloff *= 2.0 - shadowRolloff;
		// Smooth it out by doing a self-screen.

	saturationFactor *= shadowRolloff;

	// We will mask the brightness change based on the saturation so that only
	// saturation values in the middle of the range will see any brightness
	// shift.

	float saturationMask = saturation * (1.0 - saturation);

	saturationMask *= 2.0 - saturationMask;

	float brightnessFactor = shiftFactor * shadowRolloff * saturationMask;

	saturation = saturation / (1.0 - saturationFactor * (1.0 - saturation));

	maxValue *= 1.0 + 0.25 * brightnessFactor * (1.0 - maxValue); // self screen

	minValue = maxValue - saturation * maxValue;

	return vec3 (minValue, 
				 maxValue,
				 hue);
		
	}

/*****************************************************************************/

vec3 DoDecreaseVibrance (vec3 minMaxHue,
						 float vibranceShift)
	{

	float minValue = minMaxHue.x;
	float maxValue = minMaxHue.y;

	// We measure a value akin to saturation by taking the delta between maxValue and
	// minValue.
			
	float gap = maxValue - minValue;
			
	if (gap == 0.0)
		{
		return minMaxHue;
		}
				
	float saturation = gap / maxValue;
			
	float saturationFactor = -vibranceShift;
			
	#if 1
			
	// saturationFactor *= 1.0 - minValue;
	// Limit the effect in bright highlights because we don't want
	// to fight with issues from highlight recovery.
	// That really only matters for saturation increases. We're happy
	// to decrease highlight saturation.
				
	// In the shadows, we want to move more aggressively toward
	// desaturation because we lack good color information there.
				
	float shadowRolloff = min( 1.0, 16.0 * maxValue );
	shadowRolloff *= 2.0 - shadowRolloff;
	// Smooth it out by doing a self-screen.
				
	// We will mask the brightness change based on the saturation so that only
	// saturation values in the middle of the range will see any brightness
	// shift.
			
	float saturationMask = saturation * (1.0 - saturation);
			
	saturationMask *= 2.0 - saturationMask; // self screen
			
	float brightnessFactor = -vibranceShift * shadowRolloff * saturationMask;
			
	// lerp( saturation, saturation * saturation * (0.5 + 0.5 * saturation) * shadowRolloff, saturationFactor) * (1.0 - 0.25 * saturationFactor)
			
	saturation *= 1.0 - saturationFactor + saturationFactor * saturation * (0.5 + 0.5 * saturation) * shadowRolloff;
			
	saturation *= 1.0 - 0.25 * saturationFactor;
			
	// lerp( maxValue, maxValue * maxValue, brightnessFactor )

	maxValue *= 1.0 - brightnessFactor * (1.0 - maxValue);
			
	minValue = maxValue - saturation * maxValue;
				
	#else

	// Here is an alternative definition for negative vibrance based on applying a non-linear
	// limit to saturation.

	// We want to limit the saturation by scaling down and then scaling back up
			
	if (saturationFactor == 1.0)
		{
				
		maxValue = minValue = 0.5 * (maxValue + minValue);

		}
				
	else
		{
				
		// Scale the slope up at 0 and then scale everything down
				
		float newSaturation = saturation / (1.0 - saturationFactor + saturationFactor * saturation);
				
		newSaturation *= 1.0 - saturationFactor;
				
		// Darken maxValue in proportion to how much saturation we lost
		// and how much brighter it was than minValue to begin with.
				
		maxValue -= 0.5 * gap * saturationFactor;
				
		minValue = maxValue - newSaturation * maxValue;
				
		}

	#endif

	return vec3 (minValue,
				 maxValue,
				 minMaxHue.z);
		
	}

/*****************************************************************************/

float ContrastBump (float lum)
	{
	
	float contrastBump;
	
	if (lum <= 0.5)
		{
		
		float f = (lum - 0.25) * 4.0;
		
		contrastBump = f * f - 1.0;
		
		}
		
	else
		{
		
		float f = (lum - 0.75) * 4.0;
		
		contrastBump = 1.0 - f * f;
		
		}
		
	// The above curve generates something with a slope of 8 at the center
	// point. Scale it back down to have a slope of 1 at the center point
	// so we can worry simply about adding or removing some to adjust the
	// slope.
		
	contrastBump *= 0.125;
	
	return contrastBump;
	
	}

/*****************************************************************************/

float ShiftBrightnessAndContrast (float lum,
								  float brightnessShift,
								  float contrastShift,
								  float contrastSplitSlope,
								  float invContrastSplitSlope)
	{
	
	// We approximate an exponential function based on stops using a range
	// of 4 stops either direction.
	
	float dbSlope = pow (2.0, brightnessShift * 4.0);//dodgeBurnSlope (brightnessShift);
	
	if (contrastShift == 0.0)
		{
		
		float lum_db = dbSlope * lum;
		
		lum = lum_db / (1.0 + lum_db - lum);
		
		}
		
	else
		{
		
		float slope1 = dbSlope * contrastSplitSlope;
		
		float slope2 = invContrastSplitSlope;
		
		float lum_scaled_1 = slope1 * lum;
		
		lum = lum_scaled_1 / (1.0 + lum_scaled_1 - lum);
		
		float contrastSlope = pow (2.0, 0.75 * contrastShift);
		
		float contrastBump = ContrastBump (lum);
			
		if (1.0 <= contrastSlope)
			{
			
			// We can do a certain portion of the contrast slope adjustment
			// using a curve similar to the global contrast curve up through
			// a gain to a slope of 1.5.
		
			lum += contrastBump * min (0.5, contrastSlope - 1.0);
				
			contrastSlope = max (1.0, contrastSlope * 0.6666666667);	// Factor out the portion we've applied
				
			}
			
		else
			{
			
			lum += contrastBump * max (-0.5, contrastSlope - 1.0);
			
			contrastSlope = min (1.0, contrastSlope * 2.0); // Factor out the portion we've applied.
			
			}
			
		// Handle all other cases including negative contrast via the gentler curve
		// based on the div operation.
		
		if (contrastSlope != 1.0)
			{
			
			float lum_expanded = 2.0 * lum - 1.0;
		
			float lum_expanded_scaled = lum_expanded * contrastSlope;
			
			float lum_expanded_scaled_b = lum_expanded_scaled / (1.0 + (contrastSlope - 1.0) * abs (lum_expanded));
			
			lum = 0.5 + 0.5 * lum_expanded_scaled_b;
			
			}
			
		// Convert back out of non-linear space.
		
		float lum_scaled_2 = slope2 * lum;
		
		lum = lum_scaled_2 / (1.0 + lum_scaled_2 - lum);
		
		}
	
	return lum;

	}

/*****************************************************************************/

vec3 DoTuneHSLMasked32 (vec3 minMaxHue,
						float lMask,		 // local Brightness mask value
						float cMask,		 // local Contrast mask value
						float sMask)		 // local Saturation mask value
	{
	
	float contrastSplit = uToneContrastSplit;
	
	const float kMaxSaturationGain = 6.0;
	
	const float kBaseSaturationFactor = 1.0 - (1.0 / kMaxSaturationGain);
	
	const float tableScale = (1.0 / 6.0);
	
	float contrastSplitSlope = (1.0 - contrastSplit) / contrastSplit;
	
	float invContrastSplitSlope = contrastSplit / (1.0 - contrastSplit);
	
	const float kExponentialSaturationFactor = 2.5;
		// Somewhat less than log2( 6.0 ). We gain more slowly past 100%.

	float minValue = minMaxHue.x;
	float maxValue = minMaxHue.y;
	float hue = minMaxHue.z;

	float newHue = hue;

	float initialGap = maxValue - minValue;

	float maskBrightnessShift =  lMask;
	float maskContrastShift   =  cMask;
	float maskSaturationShift =  sMask;

	bool doMaskBrightnessShift = maskBrightnessShift != 0.0;
	bool doMaskContrastShift = maskContrastShift != 0.0;

	bool doMaskBrightnessContrastShift = doMaskBrightnessShift || doMaskContrastShift;

	float hslSaturationShift = 0.0;

	if (initialGap != 0.0)
		{

		// Interpolate tables.

		float index = hue * tableScale;
		
		// Remap index from [0,1] range to account for texel centers.

		index = mix (uHSLGrayCoords.x,
					 uHSLGrayCoords.y,
					 index);

		newHue = texture2D (uHSLGrayTex, vec2 (index, uHSLGrayRows.x)).r;

		hslSaturationShift = texture2D (uHSLGrayTex, vec2 (index, uHSLGrayRows.y)).r;

		float hslLuminanceShift  = texture2D (uHSLGrayTex, vec2 (index, uHSLGrayRows.z)).r;

		if (hslLuminanceShift != 0.0)
			{

			// We measure a value akin to saturation by taking the delta between max and min.
			// Note that we use the gap before any other processing was applied.

			float gap2 = initialGap * (2.0 - initialGap);

			float gap4 = gap2 * (2.0 - gap2);
			float gap8 = gap4 * (2.0 - gap4);

			hslLuminanceShift *= gap8;

			// Do the actual adjustment on max and minValue.

			minValue = minValue * (1.0 + hslLuminanceShift * (1.0 - minValue));
			maxValue = maxValue * (1.0 + hslLuminanceShift * (1.0 - maxValue));

			minValue = minValue * (1.0 + hslLuminanceShift * (1.0 - minValue));
			maxValue = maxValue * (1.0 + hslLuminanceShift * (1.0 - maxValue));

			}

		}

	// Adjust the luminance.

	if (doMaskBrightnessContrastShift)
		{

		minValue = ShiftBrightnessAndContrast (minValue, maskBrightnessShift, maskContrastShift,
											   contrastSplitSlope, invContrastSplitSlope);

		maxValue = ShiftBrightnessAndContrast (maxValue, maskBrightnessShift, maskContrastShift,
											   contrastSplitSlope, invContrastSplitSlope);

		}

	// Figure out how to handle saturation.

	if (minValue < maxValue)
		{

		float saturationShift = hslSaturationShift + maskSaturationShift;

		if (saturationShift < 0.0)
			{

			saturationShift = max (-1.0, saturationShift);

			float L = 0.5 * (maxValue + minValue);

			minValue += saturationShift * (minValue - L);
			maxValue += saturationShift * (maxValue - L);

			}

		else if (0.0 < saturationShift)
			{

			float gap = maxValue - minValue;

			float saturation = gap / maxValue;

			// Begin CALC_SKIN_PROTECTION

			float shiftedHue = hue + 1.0;

			if (6.0 <= shiftedHue)
				shiftedHue -= 6.0;

			float minSide = clamp (1.0 - (kMinScaledCentralSkinHue - shiftedHue) * kMinSideFalloff, 
								   0.0,
								   1.0);	

			float maxSide = clamp (1.0 - (shiftedHue - kMaxScaledCentralSkinHue) * kMaxSideFalloff, 
								   0.0,
								   1.0);

			float protection = min (minSide, 
									maxSide);

			protection *= (1.0 - saturation * saturation);							

			// End CALC_SKIN_PROTECTION

			// We use the skin protection to reduce the contribution from the
			// localized corrections. Unfortunately, the global corrections
			// are already well defined so we can't use this with them.

			float availableForProtection = clamp (maskSaturationShift, 0.0, saturationShift);

			saturationShift -= 0.5 * availableForProtection * protection;

			// Note: Conceivably we should do this with values from before
			// the luminance adjustment. The 1.0 logic, howecer, uses values
			// from after so we do so as well.

			// Limit the effect in bright highlights because we don't want
			// to fight with issues from highlight recovery.

			saturationShift *= 1.0 - minValue;

			// We don't want to apply the effect in the deepest shadows because
			// most of the information there is just noise.

			float shadowRolloff = min (1.0, 16.0 * maxValue);

			// Smooth it out by doing a self-screen.

			shadowRolloff *= 2.0 - shadowRolloff;

			saturationShift *= shadowRolloff;

			// Split the saturation into logic below 1.0 in which case we
			// use the 1.0 curve and values above 1.0 where we use an
			// exponential function.

			if (saturationShift <= 1.0)
				{

				float saturationFactor = kBaseSaturationFactor * saturationShift;

				saturation = saturation / (1.0 - saturationFactor * (1.0 - saturation));

				}

			else
				{

				float saturationFactor = 6.0 * pow (2.0, kExponentialSaturationFactor * (saturationShift - 1.0));

				float scaledSaturation = saturationFactor * saturation;

				saturation = scaledSaturation / (1.0 + scaledSaturation - saturation);

				}

			minValue = maxValue - saturation * maxValue;

			}

		}

	minMaxHue.x = minValue;
	minMaxHue.y = maxValue;
	minMaxHue.z = newHue;

	return minMaxHue;
		
	}

/*****************************************************************************/

vec3 DoTuneLuminanceMasked32 (vec3 rgb,
							  float localLuminanceMask,
							  float localContrastMask)
	{

	float contrastSplit = uToneContrastSplit;
	
	float contrastSplitSlope = (1.0 - contrastSplit) / contrastSplit;
	
	float invContrastSplitSlope = contrastSplit / (1.0 - contrastSplit);
	
	float brightnessShift = localLuminanceMask;
	float contrastShift   = localContrastMask;
			
	float lum = rgb.x;
			
	lum = ShiftBrightnessAndContrast (lum, 
									  brightnessShift, 
									  contrastShift,
									  contrastSplitSlope, 
									  invContrastSplitSlope);
	
	return vec3 (lum);
			
	}

/*****************************************************************************/

vec3 evalSlopeCurve32 (vec3 y,
					   vec3 shadowSlope,
					   vec3 highlightSlope)
	{
	
	vec3 deltaSlope = vec3 (3.0) - highlightSlope - shadowSlope;
	
	return y * ((vec3 (1.0) - y) * (shadowSlope + deltaSlope * y) + y * y);
	
	}

/*****************************************************************************/

// We limit the strength that we go to. We could actually go as high as 2.0, but
// that results in slopes of zero, so we just let things get really, really close.
// We use the limit logic rather than just changing the slope ranges so that we
// remain compatible with the beta version for low saturations.
	
const float kColorToningLimit = 1.9375;

// The angle for the blue-yellow axis in color toning. A value of 1.0 would
// mean yellow was at 60 degrees and was pure yellow. Smaller values move
// yellow toward red. We actually want a moderately orange color here because
// that's closer to a reasonable temperature shift.

const float kYellowAngleFract = 0.8125;
	
/*****************************************************************************/

vec3 DoLocalColorToning (vec3 rgb,
						 float byMask,
						 float gmMask)
	{

	// BULLSHIT: Easy to vectorize.
	
	// OPTIMIZE_ME_MCH: We would obviously like to stop evaluating these
	// on every pass through the code...
	
	// The theory behind these slopes comes from the split toning logic.
	// We fix the angles and calculate a maximum range set of slopes.
	
	const float s_min_slope = 0.5;
//	const float s_max_slope = 2.5;
	const float h_min_slope = 0.5;
//	const float h_max_slope = 2.5;
	
	// We use the same weights as local contrast. Alternatively, we could use
	// the ProPhoto RGB weights which place a lot more emphasis on green.
	
	const float w_r = 0.25;
	const float w_g = 0.50;
	const float w_b = 1.0 - w_r - w_g;
	
	const float U = w_r + kYellowAngleFract * w_g;
	
	// Note: I'm ignoring the maximum slope values at this point. They don't seem to become relevant.
	
	// Compute the maximum value for the delta when going toward yellow. This gets driven by
	// only being able to take blue down to s_min_slope.
	
	const float sY_delta = (1.0 - s_min_slope) / U;
	
	// Similarly, compute it when going toward blue.
	
	const float sB_delta = (1.0 - s_min_slope) / (1.0 - U);
	
	const float sBY_delta = sY_delta < sB_delta ? sY_delta : sB_delta;
	
	// The logic for magenta/green is very similar. The chief issue is that we swap roles for
	// green and blue and hence get a different weight value.
	
	const float mfract = 1.5 - kYellowAngleFract;	// 90 degree rotation
	
	const float V = w_r + mfract * w_b;
	
	// Compute the maximum value for the delta when going toward yellow. This gets driven by
	// only being able to take green down to s_min_slope.
	
	const float sM_delta = (1.0 - s_min_slope) / V;
	
	// Similarly, compute it when going toward green.
	
	const float sG_delta = (1.0 - s_min_slope) / (1.0 - V);
	
	const float sMG_delta = sM_delta < sG_delta ? sM_delta : sG_delta;
	
	const float sBYMG_delta = sBY_delta < sMG_delta ? sBY_delta : sMG_delta;
	
	// Determine the actual deltas to use. We may decide that we don't want to constrain them
	// to have the same range.
	
	const float sY_final_delta = sBYMG_delta;
	const float sB_final_delta = sBYMG_delta;
	
	const float sM_final_delta = sBYMG_delta;
	const float sG_final_delta = sBYMG_delta;
	
	// Now, we can do the same thing on the high end.
	
	const float hY_delta = (1.0 - h_min_slope) / (1.0 - U);
	const float hB_delta = (1.0 - h_min_slope) / U;
	
	const float hBY_delta = hY_delta < hB_delta ? hY_delta : hB_delta;
	
	const float hM_delta = (1.0 - h_min_slope) / (1.0 - V);
	const float hG_delta = (1.0 - h_min_slope) / V;
	
	const float hMG_delta = hM_delta < hG_delta ? hM_delta : hG_delta;
	
	const float hBYMG_delta = hBY_delta < hMG_delta ? hBY_delta : hMG_delta;
	
	const float hY_final_delta = hBYMG_delta;
	const float hB_final_delta = hBYMG_delta;
	
	const float hM_final_delta = hBYMG_delta;
	const float hG_final_delta = hBYMG_delta;
	
	// Now, it's just a matter of evaluating the slopes.
	
	const float k_sY_b = 1.0 - sY_final_delta * U;
	const float k_sY_r = k_sY_b + sY_final_delta;
	const float k_sY_g = k_sY_b + kYellowAngleFract * sY_final_delta;
	
	const float k_sB_r = 1.0 - sB_final_delta * (1.0 - U);
	const float k_sB_b = k_sB_r + sB_final_delta;
	const float k_sB_g = k_sB_r + (1.0 - kYellowAngleFract) * sB_final_delta;
	
	const float k_hY_b = 1.0 + hY_final_delta * U;
	const float k_hY_r = k_hY_b - hY_final_delta;
	const float k_hY_g = k_hY_b - kYellowAngleFract * hY_final_delta;
	
	const float k_hB_r = 1.0 + hB_final_delta * (1.0 - U);
	const float k_hB_b = k_hB_r - hB_final_delta;
	const float k_hB_g = k_hB_r - (1.0 - kYellowAngleFract) * hB_final_delta;
	
	// We want values with 1 subtracted off to make interpolation with 1 easier.
	
	const float k_sY_r_1 = k_sY_r - 1.0;
	const float k_sY_g_1 = k_sY_g - 1.0;
	const float k_sY_b_1 = k_sY_b - 1.0;
	
	const float k_hY_r_1 = k_hY_r - 1.0;
	const float k_hY_g_1 = k_hY_g - 1.0;
	const float k_hY_b_1 = k_hY_b - 1.0;
	
	const float k_sB_r_1 = k_sB_r - 1.0;
	const float k_sB_g_1 = k_sB_g - 1.0;
	const float k_sB_b_1 = k_sB_b - 1.0;
	
	const float k_hB_r_1 = k_hB_r - 1.0;
	const float k_hB_g_1 = k_hB_g - 1.0;
	const float k_hB_b_1 = k_hB_b - 1.0;
	
	// Now, it's just a matter of evaluating the slopes.
	
	const float k_sM_g = 1.0 - sM_final_delta * V;
	const float k_sM_r = k_sM_g + sM_final_delta;
	const float k_sM_b = k_sM_g + mfract * sM_final_delta;
	
	const float k_sG_r = 1.0 - sG_final_delta * (1.0 - V);
	const float k_sG_g = k_sG_r + sG_final_delta;
	const float k_sG_b = k_sG_r + (1.0 - mfract) * sG_final_delta;
	
	const float k_hM_g = 1.0 + hM_final_delta * V;
	const float k_hM_r = k_hM_g - hM_final_delta;
	const float k_hM_b = k_hM_g - mfract * hM_final_delta;
	
	const float k_hG_r = 1.0 + hG_final_delta * (1.0 - V);
	const float k_hG_g = k_hG_r - hG_final_delta;
	const float k_hG_b = k_hG_r - (1.0 - mfract) * hG_final_delta;
	
	// We want values with 1 subtracted off to make interpolation with 1 easier.
	
	const float k_sM_r_1 = k_sM_r - 1.0;
	const float k_sM_b_1 = k_sM_b - 1.0;
	const float k_sM_g_1 = k_sM_g - 1.0;
	
	const float k_hM_r_1 = k_hM_r - 1.0;
	const float k_hM_b_1 = k_hM_b - 1.0;
	const float k_hM_g_1 = k_hM_g - 1.0;
	
	const float k_sG_r_1 = k_sG_r - 1.0;
	const float k_sG_b_1 = k_sG_b - 1.0;
	const float k_sG_g_1 = k_sG_g - 1.0;
	
	const float k_hG_r_1 = k_hG_r - 1.0;
	const float k_hG_b_1 = k_hG_b - 1.0;
	const float k_hG_g_1 = k_hG_g - 1.0;
	
	const float kColorToningLimit2 = kColorToningLimit * kColorToningLimit;
	
	float sBY_r_1, sBY_g_1, sBY_b_1, hBY_r_1, hBY_g_1, hBY_b_1, fract_BY;

	float by = byMask;
	float mg = gmMask;

	// We scale down to keep the saturation growing at a reasonable
	// linear rate. Conceivably, we would like to have it grow
	// exponentially based on max_slope / min_slope, but that's generally
	// a bit messy. Essentially, we would scale max_slope up by a^dist
	// and scale min_slope down by b^dist and then recalculate mid_slope
	// to keep the ratios right and then readjust to keep it balanced
	// around 1.0. So, instead we just project the infinite range into the unit
	// circle.

	float scale = kColorToningLimit / sqrt (kColorToningLimit2 + by * by + mg * mg);

	by *= scale;
	mg *= scale;

	if (by < 0.0)
		{

		sBY_r_1 = k_sB_r_1;
		sBY_g_1 = k_sB_g_1;
		sBY_b_1 = k_sB_b_1;

		hBY_r_1 = k_hB_r_1;
		hBY_g_1 = k_hB_g_1;
		hBY_b_1 = k_hB_b_1;

		fract_BY = -by;

		}

	else
		{

		sBY_r_1 = k_sY_r_1;
		sBY_g_1 = k_sY_g_1;
		sBY_b_1 = k_sY_b_1;

		hBY_r_1 = k_hY_r_1;
		hBY_g_1 = k_hY_g_1;
		hBY_b_1 = k_hY_b_1;

		fract_BY = by;

		}

	float sMG_r_1, sMG_g_1, sMG_b_1, hMG_r_1, hMG_g_1, hMG_b_1, fract_MG;

	if (mg < 0.0)
		{

		sMG_r_1 = k_sM_r_1;
		sMG_g_1 = k_sM_g_1;
		sMG_b_1 = k_sM_b_1;

		hMG_r_1 = k_hM_r_1;
		hMG_g_1 = k_hM_g_1;
		hMG_b_1 = k_hM_b_1;

		fract_MG = -mg;

		}

	else
		{

		sMG_r_1 = k_sG_r_1;
		sMG_g_1 = k_sG_g_1;
		sMG_b_1 = k_sG_b_1;

		hMG_r_1 = k_hG_r_1;
		hMG_g_1 = k_hG_g_1;
		hMG_b_1 = k_hG_b_1;

		fract_MG = mg;

		}

	// BULLSHIT: vectorize

	// We treat these as basis vectors in the space of slope combinations.

	//vec3 s_r = vec3 (1.0) + fract_BY * sBY_1 + fract_MG * sMG_1;
	//vec3 h_r = vec3 (1.0) + fract_BY * hBY_1 + fract_MG * hMG_1;

	float s_r = 1.0 + fract_BY * sBY_r_1 + fract_MG * sMG_r_1;
	float s_g = 1.0 + fract_BY * sBY_g_1 + fract_MG * sMG_g_1;
	float s_b = 1.0 + fract_BY * sBY_b_1 + fract_MG * sMG_b_1;

	float h_r = 1.0 + fract_BY * hBY_r_1 + fract_MG * hMG_r_1;
	float h_g = 1.0 + fract_BY * hBY_g_1 + fract_MG * hMG_g_1;
	float h_b = 1.0 + fract_BY * hBY_b_1 + fract_MG * hMG_b_1;

	rgb = evalSlopeCurve32 (rgb, 
							vec3 (s_r, s_g, s_b), 
							vec3 (h_r, h_g, h_b));

	return rgb;

	}

/*****************************************************************************/

vec3 RGBTone (vec3 rgb,
			  float rowOffset)
	{

	float rr = BigTableLookup (rgb.r, rowOffset).r;
	float bb = BigTableLookup (rgb.b, rowOffset).r;

	float gg = bb + ((rr - bb) * (rgb.g - rgb.b) / (rgb.r - rgb.b));

	return vec3 (rr, 
				 gg, 
				 bb);

	}

/*****************************************************************************/

vec3 DoToneCurve (vec3 rgb,
				  float rowOffset)
	{
	
	float r = rgb.r;
	float g = rgb.g;
	float b = rgb.b;

	vec3 result;

	if (r >= g)
		{

		if (g > b)
			{

			// Case 1: r >= g > b

			result = RGBTone (rgb, 
							  rowOffset).rgb;

			}

		else if (b > r)
			{

			// Case 2: b > r >= g

			result = RGBTone (vec3 (b, r, g),
							  rowOffset).gbr;

			}

		else if (b > g)
			{

			// Case 3: r >= b > g

			result = RGBTone (vec3 (r, b, g),
							  rowOffset).rbg;

			}

		else
			{

			// Case 4: r >= g == b

			//DNG_ASSERT (r >= g && g == b, "Logic Error 2");

			float gg = BigTableLookup (g, rowOffset).r;

			result = vec3 (BigTableLookup (r, rowOffset).r,
						   gg,
						   gg);

			}

		}

	else
		{

		if (r >= b)
			{

			// Case 5: g > r >= b

			result = RGBTone (vec3 (g, r, b),
							  rowOffset).grb;

			}

		else if (b > g)
			{

			// Case 6: b > g > r

			result = RGBTone (vec3 (b, g, r),
							  rowOffset).bgr;

			}

		else
			{

			// Case 7: g >= b > r

			result = RGBTone (vec3 (g, b, r),
							  rowOffset).brg;

			}

		}

	return result;
	
	}

/*****************************************************************************/

// Draw gray around image instead of showing replicated pixels.

vec3 DoWarpMask (vec3 image,
				 vec2 pt)
	{

	const vec3 kMaskValue = vec3 (0.125);

	const vec2 kPixelOffset = vec2 (0.5, 0.5);

	vec2 size = uDefaultCropSize * uRenderScale;

	vec2 ptPixels = ToDefaultCrop (pt) * size;

	vec2 clipTL =        kPixelOffset;
	vec2 clipBR = size - kPixelOffset;

	float alpha = (clamp (1.0 - max (clipTL.x   - ptPixels.x,
									 ptPixels.x - clipBR.x),
						  0.0,
						  1.0) *
				   clamp (1.0 - max (clipTL.y   - ptPixels.y,
									 ptPixels.y - clipBR.y),
						  0.0,
						  1.0));

	image = mix (kMaskValue,
				 image, 
				 vec3 (alpha));

	return image;	
	
	}

/*****************************************************************************/

vec3 DoTuneHSL (vec3 minMaxHue)
	{
	
	const float kMaxSaturationGain = 6.0;
	
	const float kBaseSaturationFactor = 1.0 - (1.0 / kMaxSaturationGain);
	
	float tableScale = (1.0 / 6.0);
	
	float initialGap = minMaxHue.y - minMaxHue.x;

	if (initialGap == 0.0)
		{
		return minMaxHue;
		}

	float minValue = minMaxHue.x;
	float maxValue = minMaxHue.y;
	float hue = minMaxHue.z;

	// Interpolate tables.

	float index = hue * tableScale;

	// Remap index from [0,1] range to account for texel centers.

	index = mix (uHSLGrayCoords.x,
				 uHSLGrayCoords.y,
				 index);

	hue = texture2D (uHSLGrayTex, vec2 (index, uHSLGrayRows.x)).r;

	float saturationShift = texture2D (uHSLGrayTex, vec2 (index, uHSLGrayRows.y)).r;
	float luminanceShift  = texture2D (uHSLGrayTex, vec2 (index, uHSLGrayRows.z)).r;

	// Adjust the luminance.

	if (luminanceShift != 0.0)
		{

		// We measure a value akin to saturation by taking the delta between maxValue
		// and minValue. Note that we use the gap before any other processing was
		// applied.

		float gap2 = initialGap * (2.0 - initialGap);

		float gap4 = gap2 * (2.0 - gap2);
		float gap8 = gap4 * (2.0 - gap4);

		luminanceShift *= gap8;

		minValue = minValue * (1.0 + luminanceShift * (1.0 - minValue));
		maxValue = maxValue * (1.0 + luminanceShift * (1.0 - maxValue));

		minValue = minValue * (1.0 + luminanceShift * (1.0 - minValue));
		maxValue = maxValue * (1.0 + luminanceShift * (1.0 - maxValue));

		}

	// Handle any decrease in saturation by moving toward HSL's L component.

	if (saturationShift < 0.0)
		{

		float L = 0.5 * (maxValue + minValue);

		minValue += saturationShift * (minValue - L);
		maxValue += saturationShift * (maxValue - L);

		}

	// Apply any increase in saturation.

	else if (0.0 < saturationShift)
		{

		float gap = maxValue - minValue;

		float saturation = gap / maxValue;

		float saturationFactor = kBaseSaturationFactor;

		saturationFactor *= saturationShift;

		// Limit the effect in bright highlights because we don't want
		// to fight with issues from highlight recovery.

		saturationFactor *= 1.0 - minValue;

		// We don't want to apply the effect in the deepest shadows because
		// most of the information there is just noise.

		float shadowRolloff = min (1.0, 16.0 * maxValue);

		// Smooth it out by doing a self-screen.

		shadowRolloff *= 2.0 - shadowRolloff;

		saturationFactor *= shadowRolloff;

		saturation = saturation / (1.0 - saturationFactor * (1.0 - saturation));

		minValue = maxValue - saturation * maxValue;

		}

	return vec3 (minValue, maxValue, hue);

	}

/*****************************************************************************/

vec3 DoGrayMixer (vec3 rgb)
	{
	
	float tableScale = (1.0 / 6.0);
	
	float r = rgb.r;
	float g = rgb.g;
	float b = rgb.b;

	float minValue, maxValue, hue;

	// Mirror 0: Force g >= b

	bool mirror0 = (b > g);

	if (mirror0)
		{

		float temp = g;

		g = b;
		b = temp;

		}

	// Mirror 1: Force r >= b

	bool mirror1 = (b > r);

	if (mirror1)
		{

		float temp = r;

		r = b;
		b = temp;

		}

	// Mirror 2: Force r >= g

	bool mirror2 = (g > r);

	if (mirror2)
		{

		float temp = g;

		g = r;
		r = temp;

		}

	if (r == b)
		{
		return vec3 (b);
		}

	else
		{
		hue = (g - b) / (r - b);
		}

	if (mirror2)
		{
		hue = 2.0 - hue;
		}

	if (mirror1)
		{
		hue = 4.0 - hue;
		}

	if (mirror0)
		{
		hue = 6.0 - hue;
		}

	minValue = b;
	maxValue = r;
	hue = hue;

	float gap = maxValue - minValue;

	// Interpolate tables.

	#if 0

	float tableIndex = hue * tableScale;

	int32 index = (int32) tableIndex;

	float fract = tableIndex - (float) index;

	float grayBase	= weights [index];
	float graySlope = weights [index + 1] - grayBase;

	float gray = grayBase + fract * graySlope;

	#else

	// Let the hardware do the lerp.

	float tableIndex = hue * tableScale;

	// Remap index to account for texel centers.

	tableIndex = mix (uHSLGrayCoords.x,
					  uHSLGrayCoords.y,
					  tableIndex);

	float gray = texture2D (uHSLGrayTex, 
							vec2 (tableIndex, 
								  uHSLGrayRows.w)).r;

	#endif

	float result;

	// To avoid noise build up from increased contrast with negative values,
	// we scale the gap to reflect the available space on the downward side
	// of minValue.

	if (gray < 0.0)
		{

		// Translate into an alternative space. To do the blending if gray
		// level is encoded. Using linear space produces overly noisy results.

		// We approximate a square root with a piecewise curve. This actually
		// is relatively close to x ^ (1 / 2.2) in the lower portions of the
		// curve. It's much closer to the identity line for higher values than
		// a square root let alone a gamma 1 / 2.2 curve. This seems to produce
		// pretty good results, however, and it trades square roots for divides
		// which only kick in part of the time.

		float minT;

		if (9.0 * minValue < 1.0)
			minT = (3.0 * minValue) / (0.25 + 6.75 * minValue);
		else
			minT = 0.25 + 0.75 * minValue;

		float maxT;

		if (9.0 * maxValue < 1.0)
			maxT = (3.0 * maxValue) / (0.25 + 6.75 * maxValue);
		else
			maxT = 0.25 + 0.75 * maxValue;

		float resultT = clamp (minT + gray * (maxT - minT), 0.0, 1.0);

		if (3.0 * resultT < 1.0)
			result = resultT / (12.0 - 27.0 * resultT);
		else
			result = (resultT - 0.25) / 0.75;

		}

	else
		{

		result = minValue + gray * gap;

		}

	result = clamp (result, 0.0, 1.0);

	return vec3 (result);

	}

/*****************************************************************************/

float ScaleDelta (float delta, float scale)
	{
	
	float sd = scale * delta;
	
	// Divide by 1.0 + max (scale - 1.0, 0.0) * Abs_float (delta) This is 1.0
	// for scale <= 1.0, so for the scalar case we branch.
	
	if (scale > 1.0)
		{
		sd = sd / (1.0 + (scale - 1.0) * abs (delta));
		}
		
	return sd;
	
	}

/*****************************************************************************/

// Copied from cr_utils.h.

const float kNonLinearMapping_sqrt_a = 1.0 / 16.0;

const float kNonLinearMapping_a = kNonLinearMapping_sqrt_a *
								   kNonLinearMapping_sqrt_a;
								   
const float kNonLinearMapping_scale1 = 1.0644512214;		// 1 / (sqrt (1 + a) - sqrt (a))
const float kNonLinearMapping_scale2 = 0.93945122137;		// 1 / scale1

/*****************************************************************************/

float LinearToNonLinear (float x)
	{
	
	//return pow (x, 1.0 / 1.8);

	return (sqrt (x + kNonLinearMapping_a) - kNonLinearMapping_sqrt_a) *
	       kNonLinearMapping_scale1;
	
	}

/*****************************************************************************/

float NonLinearToLinear (float y)
	{
	
	//return pow (y, 1.8);
	
	y = y * kNonLinearMapping_scale2 + kNonLinearMapping_sqrt_a;
	
	return (y * y) - kNonLinearMapping_a;
	
	}

/*****************************************************************************/

vec3 DoLocalContrast (vec3 rgb,
					  vec2 pt,
					  float localClarityMask)
	{

	const float kFudgeLimit = 0.5;

	float amount = uClarityAmount * kFudgeLimit;

	#if 1

	// Combined positive + negative implementation (based on RefLocalizedDetail16).

	// Roughly log( 1.5 ) / log( 2 ) but adjusted to fit nicely as a binary value and
	// hence reliable across machines.

	const float kExponentialClarityFactor = 0.5859375;

	float c = localClarityMask;

	float cc = pow (2.0, c * kExponentialClarityFactor);

	// Get linear gray channel.

	float d = dot (kWeights, 
				   Clip01 (rgb));

	// Get mask.
		
	float cb_linear = texture2D (uLocalContrastMask,
								 pt).r;

	// Map to non-linear space.

	float dd = LinearToNonLinear (d);
	float cb = LinearToNonLinear (cb_linear);

	cc *= (1.0 + amount);
			
	float combinedDelta = 0.0;
			
	if (cc < 1.0)
		{

		float cc_factor = 0.5 + 0.5 * cc;
				
		combinedDelta = (dd - cb) * (cc * cc_factor * cc_factor - 1.0);
				
		}
				
	else if (cc > 1.0)
		{
				
		combinedDelta = 0.5 * ScaleDelta (dd - cb, 
										  2.0 * (cc - 1.0));
				
		}

	// Compute tone mask.

	float x = dd * 2.0 - 1.0;

	x = x * x;
	x = x * x;

	x = 1.0 - x;

	x = x * x;
	x = x * x;

	// Add deltas.
			
	dd = clamp (dd + x * combinedDelta, 
				0.0, 
				1.0);

	dd = NonLinearToLinear (dd);

	float scale = dd / max (d, 1.0/65535.0);

	return Clip01 (rgb * scale);

	#else

	// Old implementation, based on RefLocalContrast16.

	// Get mask.

	float m = texture2D (uLocalContrastMask,
						 pt).r;

	// Get linear gray channel.

	float d = dot (kWeights, 
				   Clip01 (rgb));

	// Map to non-linear space.

	float ss = LinearToNonLinear (m);
	float dd = LinearToNonLinear (d);

	// Compute tone mask.

	float x = dd * 2.0 - 1.0;

	x = x * x;
	x = x * x;

	x = 1.0 - x;

	x = x * x;
	x = x * x;

	// Add delta.

	float delta = dd - ss;

	dd = clamp (dd + delta * x * amount, 0.0, 1.0);

	dd = NonLinearToLinear (dd);

	float scale = dd / max (d, 1.0/65535.0);

	return Clip01 (rgb * scale);

	#endif

	}

/*****************************************************************************/

float Scale (float x, float s)
	{
	
	float sx = s * x;
	
	return sx / (1.0 + sx - x);
	
	}

/******************************************************************************/

float PowerScreen (float x, float s)
	{
	
	if (x <= 0.0)
		{
		
		return pow (1.0 + x, s) - 1.0;
		
		}
		
	else
		{
		
		return 1.0 - pow (1.0 - x, s);
		
		}
	
	}

/******************************************************************************/

// Note: x represents squared radius (not linear radius).

float DoPostCropVignette (float x,
						  float amount)
	{

	float fMidpoint = uPostCropVignetteMidpoint;

	float fFeather = uPostCropVignetteFeather;

	// For process version 2 (i.e., CR 6 / LR 3), there have been requests from the
	// prerelease group to be able to achieve the "look" of the original lens
	// vignette feature via post-crop vignette -- essentially, the power falloff.
	// Here, I have attempted to implement this by blending process version 1's
	// post-crop vignette function (i.e., from CR 5 / LR 2) with a stronger form of
	// the lens vignette function, using the feather parameter to lerp between the
	// two. -erichan 10/28/2009.

	// If we're planning to blend the post-crop vignette result with the lens
	// vignette result (i.e., for process version 2), then compute the lens vignette
	// result first.

	float resultLensVignette = 0.0;

	if (uBlendStyleLensVignette && (amount < 0.0))
		{

		resultLensVignette = DoManualLensVignette (x,
												   amount,
												   fMidpoint,
												   5.0);

		}

	float feather = uPostCropVignetteFeather;
	
	// For the formula below for calculating fs, the feedback from the beta
	// group says we should make 50% be somewhat more feathered. So we run
	// it through a little mapping function...
	
	feather = 1.5 * feather / (1.0 + 0.5 * feather);
	
	const float kMaxSlope = 8.0;
	
	float fs = 1.0 / (feather + (1.0 - feather) / kMaxSlope);
	
	fs = fs * fs;
	
	float target = sqrt (0.5) + (2.0 * fMidpoint - 1.0) * (sqrt (0.5) - sqrt (0.20)) * (0.75 + 0.25 * fMidpoint);
	
	target = target * target;

	float ts = (1.0 - target) / target;
	
	x = ts * x / (1.0 + (ts - 1.0) * x);		// Move target to 0.5
	
	x = 2.0 * x - 1.0;							// Map to -1..+1
	
	x = PowerScreen (x, fs);					// Contrast slope application
	
	x = 0.5 * (x + 1.0);						// Map back to 0..1

	float finalResult;

	if (amount < 0.0)
		{

		// Compute result of blending black with white in sRGB.
		
		float blended_sRGB = 1.0 - x;
		
		// Now convert back to linear.
		
		float blended_linear = sRGB_EvaluateInverse (blended_sRGB);
			
		float scaled_sRGB = 1.0 + amount;
		
		float scaled_linear = sRGB_EvaluateInverse (scaled_sRGB);
			
		float result = 1.0 - (1.0 - scaled_linear) * (1.0 - blended_linear);

		// If we're not planning to blend this result with the lens vignette result,
		// then stop here.

		if (!uBlendStyleLensVignette)
			{
			return result;
			}

		// We use the feather parameter to control the blend between the original
		// post-crop vignette result (i.e., process version 1) with the original lens
		// vignette result. We push the feather parameter through the sRGB tone curve
		// to make the interpolation factor more perceptually uniform across the
		// slider range [0, 100].

		float lerpFactor = sRGB_EvaluateInverse (fFeather);

		// Interpolate between the two results. This means that folks who crave the
		// original lens vignette "look" can achieve it by setting Feather to 100
		// (i.e., lerpFactor becomes 1).

		finalResult = mix (result, 
						   resultLensVignette, 
						   lerpFactor);

		}
		
	else
		{
		
		// Compute result of blending white with black in sRGB.
		
		float blended_sRGB = x;
		
		// Now convert back to linear.

		float blended_linear = sRGB_EvaluateInverse (blended_sRGB);
			
		float scaled_sRGB = amount;
		
		float scaled_linear = sRGB_EvaluateInverse (scaled_sRGB);
			
		finalResult = 1.0 - scaled_linear * blended_linear;
		
		}

	return finalResult;
	
	}

/*****************************************************************************/

// Compute crop ellipse parameters A, B, C, D, and E, such that:
//
// x = row + D;
// y = col + E;
//
// A * x * x + B * x * y + C * y * y == 1.0

float RefEllipseMask16 (vec2 pt)
	{
	
	float x = (pt.y * uDefaultCropSize.y) + uPostCropVignetteEllipseD;

	float Axx = uPostCropVignetteEllipseA * x * x;
		
	float Bx = uPostCropVignetteEllipseB * x;
			
	float y = pt.x * uDefaultCropSize.x + uPostCropVignetteEllipseE;
							
	float r2 = Axx + (Bx + uPostCropVignetteEllipseC * y) * y;
			
	r2 = min (r2, 1.0);
			
	float z = DoPostCropVignette (r2, uPostCropVignetteAmount);
			
	return z;

	}

/*****************************************************************************/

float RefRoundedRectMask16 (vec2 pt)
	{
	
	float flatX = 0.0;
	float flatY = 0.0;

	float croppedAspectRatio = uPostCropVignetteCropAspectRatio;

	float roundness = uPostCropVignetteRoundness;
	
	if (croppedAspectRatio >= 1.0)
		{
		
		flatX = 0.45 * (-roundness);
		
		flatY = max (0.0, 0.5 - ((0.5 - flatX) * croppedAspectRatio));
		
		}
		
	else
		{
		
		flatY = 0.45 * (-roundness);
		
		flatX = max (0.0, 0.5 - ((0.5 - flatY) / croppedAspectRatio));
		
		}
	
	float scaleX = 0.5 / (0.5 - flatX);
	float scaleY = 0.5 / (0.5 - flatY);
			
	pt.x *= uDefaultCropSize.x;
	pt.y *= uDefaultCropSize.y;

	// Need to swizzle because the matrix was originally designed to be used with a
	// dng_point object which has (x,y) components swpped.
			
	pt = (mul(uPostCropVignetteCropTransform, vec3 (pt.yx, 1.0))).yx;
			
	float x = abs (pt.x - 0.5);
	float y = abs (pt.y - 0.5);
			
	x = max (x - flatX, 0.0) * scaleX;
	y = max (y - flatY, 0.0) * scaleY;
			
	float r2 = (x * x + y * y) * 2.0;
							
	r2 = min (r2, 1.0);

	float z = DoPostCropVignette (r2, uPostCropVignetteAmount);

	return z;

	}

/*****************************************************************************/

float DoPostCropVignette (vec2 pt)
	{
	
	if (uPostCropVignetteRoundness >= 0.0)
		{
		return RefEllipseMask16 (pt);
		}

	else
		{
		return RefRoundedRectMask16 (pt);
		}
	
	}

/*****************************************************************************/

const float kGrainShadowMin = 0.00;
const float kGrainShadowMax = 0.05;

const float kGrainHighlightMin = 0.95;
const float kGrainHighlightMax = 1.00;

const float kGrainShadowScale  = 1.0 / (kGrainShadowMax - kGrainShadowMin);
const float kGrainShadowOffset = -kGrainShadowMin * kGrainShadowScale;

const float kGrainHighlightScale  = 1.0 / (kGrainHighlightMin - kGrainHighlightMax);
const float kGrainHighlightOffset = -kGrainHighlightMax * kGrainHighlightScale;

const vec3 kvGrainShadowScale = vec3 (kGrainShadowScale);
const vec3 kvGrainShadowOffset = vec3 (kGrainShadowOffset);

const vec3 kvGrainHighlightScale = vec3 (kGrainHighlightScale);
const vec3 kvGrainHighlightOffset = vec3 (kGrainHighlightOffset);

const vec3 kZero3 = vec3 (0.0);
const vec3 kOne3  = vec3 (1.0);

/*****************************************************************************/

#if 1 // too many samplers, let dead strip remove the var to get back to 8 samplers

vec3 DoGrain (vec3 image,
			  vec2 pt)
    {
    return image;
    }

#else

vec3 DoGrain (vec3 image,
			  vec2 pt)
	{

	pt = ToDefaultCrop (pt);

	const vec3 kZero3 = vec3 (0.0);
	const vec3 kOne3  = vec3 (1.0);
	
	float grain = texture2D (uGrainTex, pt).r;

	// Map to gamma space.

	image = sRGB_Evaluate (image);

	// Center grain [0,1] around zero [-0.5,+0.5].

	vec3 mask = (clamp (image * kvGrainShadowScale    + kvGrainShadowOffset   , kZero3, kOne3) *
				 clamp (image * kvGrainHighlightScale + kvGrainHighlightOffset, kZero3, kOne3));

	grain -= 0.5;

	// Scale the grain.

	grain *= (uGrainAmount * 2.0);

	// Add to image, clip, and undo the gamma.

	image = Clip01 (image + mask * vec3 (grain));

	image = sRGB_EvaluateInverse (image);

	return image;
	
	}
    
#endif

/*****************************************************************************/

vec3 DoFillLight (vec3 rgb,
				  vec2 pt)
	{

	// Load the mask.
	
	float mask = texture2D (uFillLightMaskTex, pt).x;

	// Run the mask thru the curve.

	mask = BigTableLookup (mask, kBigTableRow_FillLightCurve).x;

	// Modulate.

	return Clip01 (rgb * mask);
	
	}

/*****************************************************************************/

vec3 DoDarkPostCropVignette (vec3 image,
							 vec2 pt)
	{

	float m = DoPostCropVignette (pt);

	float scale = m;

	if (uPostCropVignetteHighlightContrast > 0.0 &&
		!uHasPostCropVignettePaintOverlay)
		{

		float maxScale = mix (m,
							  1.0,
							  uPostCropVignetteHighlightContrast);

		const float kHLThresh =	 0.18;
		const float kHLScale  =	 1.0 / (1.0 - kHLThresh);
		const float kHLOffset = -kHLThresh * kHLScale;

		vec3 hsv = RGBtoHSV (image);

		float v = hsv.z;

		float t = clamp ((v * kHLScale) + kHLOffset, 0.0, 1.0);

		scale = mix (m, maxScale, t);

		hsv.z = v * scale;

		image = HSVtoRGB (hsv);

		return image;

		}

	// Normal case.

	return image * scale;

	}

/*****************************************************************************/

vec3 DoWhitePostCropVignette (vec3 image,
							  vec2 pt)
	{
	
	float m = DoPostCropVignette (pt);

	// Invert image, apply mask, and re-invert.

	return vec3 (1.0) - Clip01 ((vec3 (1.0) - image)) * m;
	
	}

/*****************************************************************************/

vec3 DoPostCropVignette (vec3 image,
						 vec2 pt)
	{

	pt = ToDefaultCrop (pt);
	
	if (uPostCropVignetteAmount < 0.0)
		{
		
		// Black direction.

		image = DoDarkPostCropVignette (image, pt);
		
		}

	else
		{
		
		// White direction.

		image = DoWhitePostCropVignette (image, pt);
		
		}

	return image;
	
	}

/*****************************************************************************/

vec3 DoColorTable2D (vec3 rgb,
					 sampler2D table,
#if qDirectX
                     SamplerState tableState,
#endif
					 vec3 firstCoords,
					 vec3 lastCoords)
	{

	// Map to HSV, where H is in [0,6) and both S and V are in [0,1].

	vec3 hsv = RGBtoHSV (rgb);

	// Map H to [0,1).

	vec3 hsvIndex = hsv;

	hsvIndex.x *= (1.0 / 6.0);

	// Adjust coordinates to account for texel centers.

	hsvIndex = mix (firstCoords,
					lastCoords,
					hsvIndex);

	// Lookup into 2.5D table. Note that we index via .yx (i.e., .sh instead of .hs)
	// because the table data was uploaded in that order.
#if qDirectX
	vec3 hsvDelta = table.Sample(tableState,
							   hsvIndex.yx).xyz;
#else

	vec3 hsvDelta = texture2D (table, 
							   hsvIndex.yx).xyz;
#endif
        
	// Apply delta.

	hsv.x += (hsvDelta.x / 60.0);

	hsv.yz *= hsvDelta.yz;

	hsv.yz = min (hsv.yz, vec2 (1.0));

	// Go back to RGB.

	rgb = HSVtoRGB (hsv);	

	return rgb;
	
	}

/*****************************************************************************/

vec3 DoColorTable3D (vec3 rgb,
					 sampler3D table,
#if qDirectX
                     SamplerState tableState,
#endif
					 vec3 firstCoords,
					 vec3 lastCoords)
	{

	// Map to HSV, where H is in [0,6) and both S and V are in [0,1].

	vec3 hsv = RGBtoHSV (rgb);

	// Map H to [0,1).

	vec3 hsvIndex = hsv;

	hsvIndex.x *= (1.0 / 6.0);

	// Adjust coordinates to account for texel centers.

	hsvIndex = mix (firstCoords,
					lastCoords,
					hsvIndex);

	// Lookup into 3D table. Note that we index via .yxz (i.e., .shv instead of .hsv)
	// because the table data was uploaded in that order. The host is responsible for
	// setting up (1) GL_LINEAR filtering, and (2) texture addressing so that the "S"
	// and "R" index dimensions (corresponding to the saturation and value
	// dimensions) are fetched with CLAMP_TO_EDGE semantics, and the "T" index
	// dimension (corresponding to the hue dimension) is fetched with REPEAT
	// semantics. This way, we get the appropriate hue angle wraparound math.

#if qDirectX
	vec3 hsvDelta = table.Sample(tableState,
							   hsvIndex.yxz).xyz;
#else
	vec3 hsvDelta = texture3D (table, 
							   hsvIndex.yxz).xyz;
#endif
        
	// Apply delta.

	hsv.x += (hsvDelta.x / 60.0);

	hsv.yz *= hsvDelta.yz;

	hsv.yz = min (hsv.yz, vec2 (1.0));

	// Go back to RGB.

	rgb = HSVtoRGB (hsv);	

	return rgb;
	
	}

/*****************************************************************************/

vec3 DivMapEvaluate (vec3 x,
					 float alpha)
	{
	
	vec3 scaled = vec3 (alpha) * x;
			
	return scaled / (vec3 (1.0) + scaled - x);
	
	}

/*****************************************************************************/

vec3 DivMapEvaluateInverse (vec3 y,
							float alpha)
	{
	
	vec3 scaled = y / vec3 (alpha);
			
	return scaled / (vec3 (1.0) + scaled - y);
	
	}

/*****************************************************************************/

// Do the math directly (instead of using LUTs).

vec3 DoSplitTone (vec3 rgb)
	{

	vec3 y = rgb;
	
	y = DivMapEvaluate (y, uSplitToneBalanceMapAlpha);

	y = y * ((vec3 (1.0) - y) * (uSplitToneShadowFactor + uSplitToneDeltaFactor * y) + y * y);
		
	return DivMapEvaluateInverse (y, uSplitToneBalanceMapAlpha);
	
	}

/*****************************************************************************/

vec3 DoColorMatrix (vec3 abc)
	{

	vec3 result;

	// Put values in descending WB gains order, just like we do in the reference
	// implementation.

	abc = mul(abc, uWBSelectorMatrix);

	// Separate the standard and highlight recovery cases.

	if ((uWhiteBalanceRecover == 1.0) && (uWhiteBalanceColorLimit == 1.0))
		{
		
		// Standard case.

		vec3 wbABC = min (uWhiteBalance * abc, vec3 (1.0));

		result = Clip01 (mul(uABC2RGB, wbABC));
				
		}

	else
		{

		// Highlight recovery case.

		const float kMinValue = 0.0;
		const float kMaxValue = 1.0;

		const float kExtra2 = 1.0 / 2.0;
		const float kExtra3 = 1.0 / 3.0;

		float scale0 = uWhiteBalance.r;
		float scale1 = uWhiteBalance.g;

		float recover = uWhiteBalanceRecover;

		float colorLimit = uWhiteBalanceColorLimit;
	
		float rLimit = 1.0 / recover;
		
		float base2 = (kMaxValue - colorLimit) * kExtra3;
		float base1 = (scale1    -  kMaxValue) * kExtra2 + base2;
		
		float eScale = recover / colorLimit; 

		float c0 = abc.r;
		float c1 = abc.g;
		float c2 = abc.b;

		c0 = c0 * scale0;
		c1 = c1 * scale1;

		float uc0 = c0;
		float uc1 = c1;
		float uc2 = c2;

		float e = 0.0;

		bool didClip = false;

		if (c0 > colorLimit)
			{

			didClip = true;

			c0 = min (c0, rLimit);

			if (c0 > scale1)
				e = (c0 - scale1) + base1;

			else if (c0 > kMaxValue)
				e = (c0 - kMaxValue) * kExtra2 + base2;

			else
				e = (c0 - colorLimit) * kExtra3;

			c0 = colorLimit;

			}

		if (c1 > colorLimit)
			{

			didClip = true;

			c1 = min (c1, rLimit);

			if (c1 > kMaxValue)
				e += (c1 - kMaxValue) * kExtra2 + base2;

			else
				e += (c1 - colorLimit) * kExtra3;

			c1 = colorLimit;

			}

		if (c2 > colorLimit)
			{

			didClip = true;

			e += (c2 - colorLimit) * kExtra3;

			c2 = colorLimit;

			}

		vec3 rgb = mul(uABC2RGB, vec3 (c0, c1, c2));

		if (rgb.r > colorLimit)
			{

			didClip = true;

			rgb.r = colorLimit;

			}

		if (rgb.g > colorLimit)
			{

			didClip = true;

			rgb.g = colorLimit;

			}

		if (rgb.b > colorLimit)
			{

			didClip = true;

			rgb.b = colorLimit;

			}

		rgb = max (rgb, vec3 (kMinValue));

		float scale = recover + e * eScale;

		rgb *= scale;

		rgb = min (rgb, vec3 (kMaxValue));

		// If the clipped values no longer equal the unclipped values (note that ci
		// <= uci), then run the unclipped values through the matrix and use this as
		// a limit on the converted values. This helps prevent green sunsets caused
		// by impossible colors.

		if (didClip)
			{

			uc0 = min (uc0, 2.0);
			uc1 = min (uc1, 2.0);
			uc2 = min (uc2, 2.0);

			vec3 rrggbb = mul(uABC2RGB, vec3 (uc0, uc1, uc2));

			rrggbb *= recover;

			// Change for CR 3.1: Limit this delta to the old gap between the min and
			// max channels to avoid adding too much color to near neutrals.

			float oldGap = max (max (abs (rgb.r - rgb.g),
									 abs (rgb.r - rgb.b)),
								abs (rgb.g - rgb.b));

			if (rrggbb.r > rgb.r) rgb.r = min (rrggbb.r, min (rgb.r + oldGap, kMaxValue));
			if (rrggbb.g > rgb.g) rgb.g = min (rrggbb.g, min (rgb.g + oldGap, kMaxValue));
			if (rrggbb.b > rgb.b) rgb.b = min (rrggbb.b, min (rgb.b + oldGap, kMaxValue));

			}

		result = rgb;

		}

	return result;

	}

/*****************************************************************************/

float RenderGradient (vec4 grad,
					  vec2 pt)
	{

	// Let Z and F be the zero and full endpoints of the gradient, in pixel
	// coordinates.
	//
	// Let P be a point in the image, in pixel coordinates.
	//
	// Let D be (F - Z) / dot (F - Z, F - Z).
	//
	// Then f (P) = dot (P - Z, D) is a linear scalar-valued function of P with
	// output:
	//
	//	 0		when P is on the line passing through point Z and perpendicular to
	//			line segment ZF.
	//
	//	 1		when P is on the line passing through point F and perpendicular to
	//			line segment ZF.
	//
	// grad.xy contains Z.
	// grad.zw contains D.

	float m = clamp (dot (pt - grad.xy, grad.zw),
					 0.0,
					 1.0);

	// Do easing (cf. kGradientRenderLeastSmoothed).

	m = 1.0 - m * m;
	m = 1.0 - m * m;

	return m;
	
	}

/*****************************************************************************/

// Transform from linear space to a flat-noise encoded space.

vec3 FlatNoiseEncode (vec3 rgb)
	{

	vec3 eval = sqrt (uDenoiseCurveNoiseScale * rgb + vec3 (uDenoiseCurveNoiseOffset));

	vec3 integ = (2.0 / uDenoiseCurveNoiseScale) * eval;

	return (integ - vec3 (uDenoiseCurveIntegratedOffset)) * vec3 (uDenoiseCurveIntegratedScale);

	}

/*****************************************************************************/

// Transform from flat-noise encoded space to linear space (i.e., inverse of
// FlatNoiseEncode).

vec3 FlatNoiseDecode (vec3 rgb)
	{
	
	vec3 z = vec3 (uDenoiseCurveIntegratedOffset) + (rgb / uDenoiseCurveIntegratedScale);

	z *= (uDenoiseCurveNoiseScale * 0.5);

	return ((z * z) - vec3 (uDenoiseCurveNoiseOffset)) / uDenoiseCurveNoiseScale;
	
	}

/*****************************************************************************/

// Denoise via precomputed delta function.

vec3 DoDenoise (vec3 rgb,
				vec2 pt,
				float localNoiseMask)
	{

	pt = ToDefaultCrop (pt);

	// 1. Go to flat noise space.

	rgb = FlatNoiseEncode (rgb);

	// 2. Go to YCC.

	vec3 ycc = uDenoiseRGBtoYCC * rgb;

	// 3. Denoise.

#if kCombineStage3Denoise
	vec3 delta = texture2DArray (uStage3DenoiseImage, vec3 (pt, 1)).rgb; 
#else
	vec3 delta = texture2D (uDenoiseMask, pt).rgb; 
#endif

	delta -= vec3 (0.5);

	float amount = uLumaDenoiseAmount;

	// Do we have local adjustments?

	if (localNoiseMask != 0.0)
		{

		// Yes.

		// Map mask value from (-inf,+inf) to (-1,+1)

		localNoiseMask = MapInfRangeToSignedNormRange (localNoiseMask);

		// Cut off at (-1/2,+1/2). Thus full unsmoothing / smoothing can be achieved
		// with a single local adjustment.

		localNoiseMask = clamp (localNoiseMask, -0.5, +0.5);

		// Distinguish plus & minus directions.

		if (localNoiseMask < 0.0)
			{

			// Minus direction. Interpolate between no smoothing and global smoothing.
			
			float t = (localNoiseMask + 0.5) * 2.0;

			amount = mix (0.0, uLumaDenoiseAmount, t);
			
			}

		else
			{

			// Plus direction. Interpolate between global smoothing and full smoothing.
			
			float t = localNoiseMask * 2.0;

			amount = mix (uLumaDenoiseAmount, 1.0, t);
			
			}

		}

	// Adjust the luminance component of the delta.

	delta.x *= amount;

	// Add the delta to the original signal to the get the denoised signal.
	
	ycc += delta;

	// 4. Back to RGB.

	rgb = uDenoiseYCCtoRGB * ycc;

	rgb = Clip01 (rgb);

	// 5. Back to linear space.

	rgb = FlatNoiseDecode (rgb);

	// Done.

	return rgb;
	
	}

/*****************************************************************************/

[Code]

//void main ()
//	{

	// Get the stage 3 image.

	vec2 dstPt = inputs.uv0;

	vec2 srcPtR = dstPt;
	vec2 srcPt  = dstPt;
	vec2 srcPtB = dstPt;

	vec3 image = GetStage3 (dstPt,
							srcPtR,
							srcPt,
							srcPtB);

	// Render gradients.

	float localExposureMask = 0.0;

	float localClarityMask = 0.0;

	float localSatMask = 0.0;

	float localLuminanceMask = 0.0;

	float localContrastMask = 0.0;			 // not to be confused with clarity!

	float localBlueYellowMask = 0.0;

	float localGreenMagentaMask = 0.0;

	float localNoiseMask = 0.0;

	if (uHasGradients)
		{

		vec2 ptThis = dstPt * uStage3Size;

		for (int k = 0; k < uNumGradients; k++)
			{

			float g = RenderGradient (uGradients [k], 
									  ptThis);

            /* replaced with code below
			localExposureMask += (g * uLocalExposureGradientWeights [k]);

			localClarityMask += (g * uLocalClarityGradientWeights [k]);

			localSatMask += (g * uLocalSaturationGradientWeights [k]);

			localLuminanceMask += (g * uLocalLuminanceGradientWeights [k]);


			localContrastMask += (g * uLocalContrastGradientWeights [k]);

			localBlueYellowMask += (g * uLocalBlueYellowGradientWeights [k]);

			localGreenMagentaMask += (g * uLocalGreenMagentaGradientWeights [k]);

			localNoiseMask += (g * uLocalNoiseWeights [k]);
            */
            
            // all packed into vec4 array now
            localExposureMask   += g * uGradientsWeights[2*k+0].x;
            localClarityMask    += g * uGradientsWeights[2*k+0].y;
            localSatMask        += g * uGradientsWeights[2*k+0].z;
            localLuminanceMask  += g * uGradientsWeights[2*k+0].w;

            localContrastMask     += g * uGradientsWeights[2*k+1].x;
            localBlueYellowMask   += g * uGradientsWeights[2*k+1].y;
            localGreenMagentaMask += g * uGradientsWeights[2*k+1].z;
            localNoiseMask        += g * uGradientsWeights[2*k+1].w;
			}

		}

	// Show local contrast mask?

	if (uVisMode == kVisClarity)
		{

		outputs.target0 = vec4 (texture2D (uLocalContrastMask, srcPt).rrr, 1.0);

		return;

		}

	// Show fill light mask?

	else if (uVisMode == kVisFillLight)
		{

		outputs.target0 = vec4 (texture2D (uFillLightMaskTex, srcPt).rrr, 1.0);

		return;

		}

	// Show grain buffer?

	else if (uVisMode == kVisGrain)
		{
#if 1
        outputs.target0 = vec4(0.0, 0.0, 0.0, 1.0);
#else
		outputs.target0 = vec4 (texture2D (uGrainTex, srcPt).rrr, 1.0);
#endif
		return;

		}
 
    // kVisNormal continues...
 

	// Transform to RIMM space (ProPhoto RGB primaries, linear gamma). This step
	// includes white balance and possibly highlight recovery.

	image = DoColorMatrix (image);

	// Denoise.

	if (uHasDenoise)
		{

		image = DoDenoise (image,
						   srcPt,
						   localNoiseMask);

		}

	// Apply hue sat map.

	if (uHasHueSatMap)
		{

		image = DoColorTable3D (image,
								uHueSatMap,
#if qDirectX
                                uHueSatMapSampler
#endif
								uHueSatMapCoordsFirst,
								uHueSatMapCoordsLast);

		}

	// Defringe highlights.

	// ...

	// Lens vignette (manual & profile).

	image = DoLensVignette (image, 
							srcPt);

	// Post-crop vignette (highlight priority).

	if (uHasPostCropVignetteHighlightPriority)
		{
		image = DoPostCropVignette (image, dstPt);
		}

	// Fill light.

	if (uHasFillLight)
		{
		image = DoFillLight (image, srcPt);
		}

	// Gradient test (local exposure, PV 2010).

	if (uHasGradients)
		{

		localExposureMask = pow (2.0, localExposureMask * kLocalExposureScale);

		image *= localExposureMask;

		}

	// Exposure ramp.

	image = DoRamp (image);

	// Defringe edges.

	// ...

	// Separate color & grayscale processing paths.

	if (uConvertToGray)
		{

		// Grayscale path.

		if (uHasLookTable)
			{

			image = DoColorTable3D (image, 
									uLookTable,
#if qDirectX
                                    uLookTableSampler
#endif
									uLookTableCoordsFirst,
									uLookTableCoordsLast);

			}

		// Gray mixer.

		image = DoGrayMixer (image);

		// Clarity.

		if (uHasLocalContrast)
			{

			image = DoLocalContrast (image, 
									 dstPt,
									 localClarityMask);

			}

		// Red eye.

		// ...

		// Luminance tuner.

		if (localLuminanceMask != 0.0 ||
			localContrastMask  != 0.0)
			{

			image = DoTuneLuminanceMasked32 (image,
											 localLuminanceMask,
											 localContrastMask);

			}

		}

	else
		{

		// Color path.

		// Saturation.

		image = DoSaturation (image);

		// Defringe edges.

		// ...

		// Clarity.

		if (uHasLocalContrast)
			{

			image = DoLocalContrast (image, 
									 dstPt,
									 localClarityMask);

			}

		// Red eye.

		// ...

		// Look table.

		if (uHasLookTable)
			{

			image = DoColorTable3D (image,
									uLookTable,
#if qDirectX
                                    uLookTableSampler
#endif
									uLookTableCoordsFirst,
									uLookTableCoordsLast);

			}

		// Vibrance and HSL.

		image = DoRGBtoMMH (image);

		if (uVibrance > 0.0)
			{
			image = DoIncreaseVibrance (image, uVibrance);
			}

		else if (uVibrance < 0.0)
			{
			image = DoDecreaseVibrance (image, uVibrance);
			}

		if (uHasHSL)
			{

			//image = DoTuneHSL (image);

			image = DoTuneHSLMasked32 (image,
									   localLuminanceMask,
									   localContrastMask,
									   localSatMask);

			}

		image = DoMMHtoRGB (image);

		}

	// Local color toning.

	if (localBlueYellowMask	  != 0.0 ||
		localGreenMagentaMask != 0.0)
		{
		
		image = DoLocalColorToning (image,
									localBlueYellowMask,
									localGreenMagentaMask);
		
		}

	// Post-crop vignette (color priority).

	if (uHasPostCropVignetteColorPriority)
		{
		image = DoPostCropVignette (image, dstPt);
		}

	// Tone curve.

	image = DoToneCurve (image,
						 kBigTableRow_ToneCurve);

	// Split tone.

	if (uHasSplitTone)
		{
		image = DoSplitTone (image);
		}

	// Sharpen.

	// ...

	// Grain.

	if (uHasGrain)
		{
		image = DoGrain (image, dstPt);
		}

	// Post-crop vignette (paint overlay).

	if (uHasPostCropVignettePaintOverlay)
		{
		image = DoPostCropVignette (image, dstPt);
		}

	// Clipping warnings.

	if (uShowShadowWarning &&
		(image.r < 0.001 ||
		 image.g < 0.001 ||
		 image.b < 0.001))
		{
		image = vec3 (0.0, 0.0, 1.0);
		}

	else if (uShowHighlightWarning &&
		(image.r > 0.999 ||
		 image.g > 0.999 ||
		 image.b > 0.999))
		{
		image = vec3 (1.0, 0.0, 0.0);
		}

	// Draw antialiased warp mask (so we don't see padded boundary).

	image = DoWarpMask (image, srcPt);

	// Convert to monitor space.
	image = DoColorConvert (image);

	// Done.

	outputs.target0 = vec4 (image, 1.0);

//	}

/******************************************************************************/

[Material] sname=ACRPipe vs=ACRPipeVS ps=ACRPipePS

