/*****************************************************************************/

// Camera Raw Process Version 2012 interactive image processing pipeline.

/*****************************************************************************/

[Fragment] name=PipeFragment

/*****************************************************************************/

[Param]

/*****************************************************************************/

// Input image.

uniform sampler2D uStage3Image;

/*****************************************************************************/

// Big table (2D texture atlas).

uniform sampler2D uBigTable;

uniform float uBigTableCol0;
uniform float uBigTableCol1;

uniform float uBigTable_ToneCurve;
uniform float uBigTable_ProfileVignette;
uniform float uBigTable_ProfileDistortion;
uniform float uBigTable_CurveRed;
uniform float uBigTable_CurveGreen;
uniform float uBigTable_CurveBlue;
uniform float uBigTable_PostCropVignette;
uniform float uBigTable_GrayMixer;
uniform float uBigTable_UndoToneCurve;
uniform float uBigTable_HSL;

// FOO(ToneCurve)

// #define FOO(x) uniform float uBigTable_ ## x

// #define FOO(x) "uBigTable_" #x ","

// #define FOO(x) ti ## x ## ,

/*****************************************************************************/

// Image size parameters.

// Default crop area, for a given pyramid level, in pixels.

uniform vec2  uDefaultCropSize;

// Stage 3 image size, for a given pyramid level, in pixels.

uniform vec2  uStage3Size;

// Scale & offset to facilitate a transformation from stage 3 image-relative
// coordinates to default crop area-relative coordinates.

uniform vec2  uToDefaultCropScale;
uniform vec2  uToDefaultCropOffset;

// Normalization scale factors to map from relative coordinates to a scale
// where the radius from the center to the corner is 1.0. Accounts correctly
// for the image's aspect ratio (e.g., 3:2).

uniform vec2  uNormalizeDistance;

uniform float uRenderScale;

/*****************************************************************************/

// Exposure and tone parameters.

uniform bool  uHasUndoToneCurve;

/*****************************************************************************/

// Color and gray parameters.

uniform bool  uConvertToGray;

uniform float uGrayMixerCoord0;
uniform float uGrayMixerCoord1;

uniform bool  uHasVibranceHSL;
uniform bool  uHasHSL;
uniform float uHSLCoord0;
uniform float uHSLCoord1;
uniform float uToneContrastSplit;
uniform float uVibrance;

/*****************************************************************************/

// Warp parameters.

// Do we have a warp?

uniform bool  uHasWarp;

// Crop transform.

uniform mat3  uBackwardCropTransform;

// Perspective transform.

uniform bool  uHasPerspective;
uniform float uPerspectiveMinScale;
uniform float uPerspectiveMaxScale;
uniform mat3  uPerspectiveMatrix;

// Manual distortion.

uniform bool  uHasManualDistort;
uniform float uManualDistortB;

// Profile-based distortion.

uniform bool  uHasProfileWarp;
uniform vec2  uProfileWarpCenter;
uniform vec2  uProfileWarpToTable;
uniform vec2  uProfileWarpFromTable;

// Lateral CA.

uniform bool	  uHasLateralCA;
uniform sampler2D uLateralCAMap;

/*****************************************************************************/

// Lens vignette.

uniform bool  uHasLensVignette;

uniform bool  uHasLensManualVignette;
uniform float uLensVignetteAmount;
uniform float uLensVignetteMidpoint;

uniform bool  uHasLensProfileVignette;
uniform vec2  uVignetteProfileScale;
uniform vec2  uVignetteProfileOffset;

/*****************************************************************************/

// Post-crop vignette.

uniform bool uHasPostCropVignetteHighlightPriority;
uniform bool uHasPostCropVignetteColorPriority;
uniform bool uHasPostCropVignettePaintOverlay;

uniform float uPostCropVignetteAmount;
uniform float uPostCropVignetteMidpoint;
uniform float uPostCropVignetteFeather;
uniform float uPostCropVignetteRoundness;
uniform float uPostCropVignetteHighlightContrast;

// For non-negative roundness.

uniform float uPostCropVignetteEllipseA;
uniform float uPostCropVignetteEllipseB;
uniform float uPostCropVignetteEllipseC;
uniform float uPostCropVignetteEllipseD;
uniform float uPostCropVignetteEllipseE;

// For negative roundness.

uniform mat3  uPostCropVignetteCropTransform;
uniform float uPostCropVignetteCropAspectRatio;

uniform bool  uBlendStyleLensVignette;

/*****************************************************************************/

// Split tone.

uniform bool  uHasSplitTone;

uniform float uSplitToneBalanceMapAlpha;
uniform vec3  uSplitToneShadowFactor;
uniform vec3  uSplitToneDeltaFactor;

/*****************************************************************************/

// RGB curves.

uniform bool  uHasRGBCurves;

/*****************************************************************************/

// Transparency.

uniform bool  uDoTransparency;

uniform bool  uDoCheckerboard;
uniform float uCheckerboardSize;
uniform vec2  uCheckerboardOrigin;

uniform bool  uFlattenTransparency;
uniform bool  uMultiplyTransparency;

uniform float uMatteGrayLevel;
uniform bool  uOnlyPurelyTransparent;

/*****************************************************************************/

// Other params.

uniform float uParamExposure;

uniform mat3  uSaturationMatrix;

/*****************************************************************************/

[Function]

/*****************************************************************************/

// It is assumed that the inputs pt is in relative coordinates, relative to
// the full stage 3 image area (i.e., the origin is the top-left of the full
// stage 3 image). This routine transforms the coordinates to be relative to
// the default crop area, which is often slightly smaller than the stage 3
// image area.

vec2 ToDefaultCrop (vec2 pt)
	{

	return (pt * uToDefaultCropScale) + uToDefaultCropOffset;

	}

/*****************************************************************************/

// BULLSHIT: Can optimize this by precomputing new constants and doing mul and
// add instead.

vec2 FromDefaultCrop (vec2 pt)
	{

	return (pt - uToDefaultCropOffset) / uToDefaultCropScale;

	}

/*****************************************************************************/

float NormalizedSquaredDistanceFromCenter (vec2 pt)
	{

	vec2 diff = (pt - vec2 (0.5, 0.5)) * uNormalizeDistance;

	return dot (diff, diff);

	}

/*****************************************************************************/

float NormalizedSquaredPinnedDistanceFromCenter (vec2 pt)
	{

	return min (NormalizedSquaredDistanceFromCenter (pt),
				1.0);
	
	}

/*****************************************************************************/

vec4 BigTableLookup (float index,
					 float rowOffset,
					 float coord0,
					 float coord1)
	{

	// Map input from logical space to texture space.

	index = mix (coord0, coord1, index); 

	return texture2D (uBigTable, vec2 (index, rowOffset));

	}

/*****************************************************************************/

vec4 BigTableLookup (float index,
					 float rowOffset)
	{

	return BigTableLookup (index,
						   rowOffset,
						   uBigTableCol0,
						   uBigTableCol1);

	}

/*****************************************************************************/

// Begin color routines.

/*****************************************************************************/

float sRGB_Evaluate (float x)
	{

	if (x <= 0.0031308)
		return x * 12.92;
		
	else
		return 1.055 * pow (x, 1.0 / 2.4) - 0.055;
	
	}

/*****************************************************************************/

vec3 sRGB_Evaluate (vec3 x)
	{

	return vec3 (sRGB_Evaluate (x.r),
				 sRGB_Evaluate (x.g),
				 sRGB_Evaluate (x.b));

	}

/*****************************************************************************/

float sRGB_EvaluateInverse (float y)
	{

	if (y <= 0.0031308 * 12.92)
		return y * (1.0 / 12.92);
		
	else
		return pow ((y + 0.055) * (1.0 / 1.055), 2.4);
	
	}

/*****************************************************************************/

vec3 sRGB_EvaluateInverse (vec3 x)
	{

	return vec3 (sRGB_EvaluateInverse (x.r),
				 sRGB_EvaluateInverse (x.g),
				 sRGB_EvaluateInverse (x.b));

	}

/*****************************************************************************/

// Based on DNG_RGBtoHSV (see dng_utils.h).

vec3 RGBtoHSV (vec3 src)
	{

	float h, s, v;
	
	v = max (src.r, max (src.g, src.b));

	float gap = v - min (src.r, min (src.g, src.b));
	
	if (gap > 0.0)
		{

		if (src.r == v)
			{
			
			h = (src.g - src.b) / gap;
			
			if (h < 0.0)
				{
				h += 6.0;
				}
				
			}
			
		else if (src.g == v) 
			{
			h = 2.0 + (src.b - src.r) / gap;
			}
			
		else
			{
			h = 4.0 + (src.r - src.g) / gap;
			}
			
		s = gap / v;
		
		}
		
	else
		{
		h = 0.0;
		s = 0.0;
		}

	return vec3 (h, s, v);
	
	}

/*****************************************************************************/

// Based on DNG_HSVtoRGB (see dng_utils.h).

vec3 HSVtoRGB (vec3 hsv)
	{

	float r, g, b;

	float h = hsv.x;
	float s = hsv.y;
	float v = hsv.z;

	if (s > 0.0)
		{
		
		if (h < 0.0)
			h += 6.0;
			
		if (h >= 6.0)
			h -= 6.0;
			
		float i = floor (h);

		float f = h - i;
		
		float p = v * (1.0 - s);

		float q = v * (1.0 - s * f);

		float t = (v * (1.0 - s * (1.0 - f)));

		if (i == 0.0)
			{
			r = v; g = t; b = p;
			}

		else if (i == 1.0)
			{
			r = q; g = v; b = p;
			}

		else if (i == 2.0)
			{
			r = p; g = v; b = t;
			}
		
		else if (i == 3.0)
			{
			r = p; g = q; b = v;
			}

		else if (i == 4.0)
			{
			r = t; g = p; b = v;
			}

		else
			{
			r = v; g = p; b = q;
			}
		
		}
		
	else
		{
		r = v;
		g = v;
		b = v;
		}

	return vec3 (r, g, b);
	
	}

/*****************************************************************************/

vec3 DoSaturation (vec3 rgb)
	{

	return saturate (uSaturationMatrix * rgb);

	}

/*****************************************************************************/

vec3 DoRGBtoMMH (vec3 rgb)
	{

	float r = rgb.r;
	float g = rgb.g;
	float b = rgb.b;

	// Mirror 0: Force g >= b

	bool mirror0 = (b > g);

	if (mirror0)
		{

		float temp = g;

		g = b;
		b = temp;

		}

	// Mirror 1: Force r >= b

	bool mirror1 = (b > r);

	if (mirror1)
		{

		float temp = r;

		r = b;
		b = temp;

		}

	// Mirror 2: Force r >= g

	bool mirror2 = (g > r);

	if (mirror2)
		{

		float temp = g;

		g = r;
		r = temp;

		}

	float hue;

	if (r == b)
		{
		hue = 0.0;
		}
	else
		{
		hue = (g - b) / (r - b);
		}

	if (mirror2)
		{
		hue = 2.0 - hue;
		}

	if (mirror1)
		{
		hue = 4.0 - hue;
		}

	if (mirror0)
		{
		hue = 6.0 - hue;
		}

	return vec3 (b, r, hue);
	
	}

/*****************************************************************************/

vec3 DoMMHtoRGB (vec3 mmh)
	{

	float hue = mmh.z;

	float r = mmh.y;
	float b = mmh.x;

	// We might have a hue angle that has been bumped up once for reds being
	// low and again for being off the end.

	if (hue > 6.0)
		{
		hue -= 6.0;
		}

	if (hue > 6.0)
		{
		hue -= 6.0;
		}

	bool mirror0 = (hue > 3.0);

	if (mirror0)
		{
		hue = 6.0 - hue;
		}

	bool mirror1 = (hue > 2.0);

	if (mirror1)
		{
		hue = 4.0 - hue;
		}

	bool mirror2 = (hue > 1.0);

	if (mirror2)
		{
		hue = 2.0 - hue;
		}

	float g = b + (r - b) * hue;

	if (mirror2)
		{

		float temp = g;

		g = r;
		r = temp;

		}

	if (mirror1)
		{

		float temp = r;

		r = b;
		b = temp;

		}

	if (mirror0)
		{

		float temp = g;

		g = b;
		b = temp;

		}

	return vec3 (r, g, b);

	}

/*****************************************************************************/

// Skin protection works on hues shifted by one sextant and stored using a
// 0..6 scale. We recognize a min and max value for a central range together
// with falloffs on the man and max sides.

const float kMinSkinHue        = -60.0;
const float kMinCentralSkinHue =   0.0;											
const float kMaxCentralSkinHue =  30.0;										
const float kMaxSkinHue        =  45.0;												

const float kMinScaledSkinHue		 = ( kMinSkinHue		+ 60.0 ) / 60.0;	
const float kMinScaledCentralSkinHue = ( kMinCentralSkinHue + 60.0 ) / 60.0;	
const float kMaxScaledCentralSkinHue = ( kMaxCentralSkinHue + 60.0 ) / 60.0;	
const float kMaxScaledSkinHue		 = ( kMaxSkinHue		+ 60.0 ) / 60.0;	

const float kMinSideFalloff = 1.0 / (kMinScaledCentralSkinHue - kMinScaledSkinHue); 
const float kMaxSideFalloff = 1.0 / (kMaxScaledSkinHue - kMaxScaledCentralSkinHue); 

/*****************************************************************************/

vec3 DoIncreaseVibrance (vec3 minMaxHue,
						 float vibranceShift)
	{
	
	// We use different gains for saturation in skin tones and elsewhere.

	const float kMaxSaturationGain = 6.0;
	const float kMaxSkinSaturationGain = 1.75;
	
	const float kStandardFactor = 1.0 - ( 1.0 / kMaxSaturationGain	   );
	const float kSkinFactor		= 1.0 - ( 1.0 / kMaxSkinSaturationGain );
	const float kSkinProtectionSlope = kSkinFactor - kStandardFactor;

	float minValue = minMaxHue.x;
	float maxValue = minMaxHue.y;

	// We measure a value akin to saturation by taking the delta between
	// maxValue and minValue.

	float gap = maxValue - minValue;

	if (gap == 0.0)
		{
		return minMaxHue;
		}

	float saturation = gap / maxValue;

	float shiftFactor = vibranceShift;

	// We protect skin tones by masking on low saturation colors in a
	// particular hue range.

	float hue = minMaxHue.z;

	// Begin CALC_SKIN_PROTECTION

	float shiftedHue = hue + 1.0;

	if (6.0 <= shiftedHue)
		shiftedHue -= 6.0;

	float minSide = saturate (1.0 - (kMinScaledCentralSkinHue - shiftedHue) * kMinSideFalloff);
	float maxSide = saturate (1.0 - (shiftedHue - kMaxScaledCentralSkinHue) * kMaxSideFalloff);

	float protection = min (minSide, 
							maxSide);

	protection *= (1.0 - saturation * saturation);							

	// End CALC_SKIN_PROTECTION

	// Masked by protection, multiply shift factor with itself. This means that
	// in skin tones, not only is vibrance weaker in its total effect, it kicks
	// in more slowly as well.

	shiftFactor = shiftFactor * (shiftFactor * protection + 1.0 - protection);

	float saturationFactor = kStandardFactor + kSkinProtectionSlope * protection;
		// Up to a 6x gain in saturation. Flesh tones are much restricted
		// in what they can get.

	saturationFactor *= shiftFactor;
		// The shift factor is the processed parameter value.

	saturationFactor *= 1.0 - minValue;
		// Limit the effect in bright highlights because we don't want
		// to fight with issues from highlight recovery.

	// We don't want to apply the effect in the deepest shadows because
	// most of the information there is just noise.

	float shadowRolloff = min ( 1.0, 16.0 * maxValue );
	shadowRolloff *= 2.0 - shadowRolloff;
		// Smooth it out by doing a self-screen.

	saturationFactor *= shadowRolloff;

	// We will mask the brightness change based on the saturation so that only
	// saturation values in the middle of the range will see any brightness
	// shift.

	float saturationMask = saturation * (1.0 - saturation);

	saturationMask *= 2.0 - saturationMask;

	float brightnessFactor = shiftFactor * shadowRolloff * saturationMask;

	saturation = saturation / (1.0 - saturationFactor * (1.0 - saturation));

	maxValue *= 1.0 + 0.25 * brightnessFactor * (1.0 - maxValue); // self screen

	minValue = maxValue - saturation * maxValue;

	return vec3 (minValue, 
				 maxValue,
				 hue);
		
	}

/*****************************************************************************/

vec3 DoDecreaseVibrance (vec3 minMaxHue,
						 float vibranceShift)
	{

	float minValue = minMaxHue.x;
	float maxValue = minMaxHue.y;

	// We measure a value akin to saturation by taking the delta between
	// maxValue and minValue.
			
	float gap = maxValue - minValue;
			
	if (gap == 0.0)
		{
		return minMaxHue;
		}
				
	float saturation = gap / maxValue;
			
	float saturationFactor = -vibranceShift;
			
	// BULLSHIT: I think that all of this math is based on saturation has a
	// per-pixel input and vibranceShift as a global (uniform) input. Since
	// saturation should be in [0,1], this means that we can precompute all
	// this math into a LUT and stick it into bigTable. So, we should be able
	// to do something like this: -erichan 4/14/2013.

	/*

	 vec2 temp = BigTableLookup (saturation, uBigTable_DecreaseVibrance).xy;

	 float brightnessFactor = temp.x;
	 float saturation       = temp.y;

	 maxValue *= 1.0 - brightnessFactor * (1.0 - maxValue);
	 minValue = maxValue - saturation * maxValue;

	*/

	#if 1
			
	// saturationFactor *= 1.0 - minValue;
	// Limit the effect in bright highlights because we don't want
	// to fight with issues from highlight recovery.
	// That really only matters for saturation increases. We're happy
	// to decrease highlight saturation.
				
	// In the shadows, we want to move more aggressively toward
	// desaturation because we lack good color information there.
				
	float shadowRolloff = min( 1.0, 16.0 * maxValue );
	shadowRolloff *= 2.0 - shadowRolloff;
	// Smooth it out by doing a self-screen.
				
	// We will mask the brightness change based on the saturation so that only
	// saturation values in the middle of the range will see any brightness
	// shift.
			
	float saturationMask = saturation * (1.0 - saturation);
			
	saturationMask *= 2.0 - saturationMask; // self screen
			
	float brightnessFactor = -vibranceShift * shadowRolloff * saturationMask;
			
	// lerp( saturation, saturation * saturation * (0.5 + 0.5 * saturation) * shadowRolloff, saturationFactor) * (1.0 - 0.25 * saturationFactor)
			
	saturation *= 1.0 - saturationFactor + saturationFactor * saturation * (0.5 + 0.5 * saturation) * shadowRolloff;
			
	saturation *= 1.0 - 0.25 * saturationFactor;
			
	// lerp( maxValue, maxValue * maxValue, brightnessFactor )

	maxValue *= 1.0 - brightnessFactor * (1.0 - maxValue);
			
	minValue = maxValue - saturation * maxValue;
				
	#else

	// Here is an alternative definition for negative vibrance based on applying a non-linear
	// limit to saturation.

	// We want to limit the saturation by scaling down and then scaling back up
			
	if (saturationFactor == 1.0)
		{
				
		maxValue = minValue = 0.5 * (maxValue + minValue);

		}
				
	else
		{
				
		// Scale the slope up at 0 and then scale everything down
				
		float newSaturation = saturation / (1.0 - saturationFactor + saturationFactor * saturation);
				
		newSaturation *= 1.0 - saturationFactor;
				
		// Darken maxValue in proportion to how much saturation we lost
		// and how much brighter it was than minValue to begin with.
				
		maxValue -= 0.5 * gap * saturationFactor;
				
		minValue = maxValue - newSaturation * maxValue;
				
		}

	#endif

	return vec3 (minValue,
				 maxValue,
				 minMaxHue.z);
		
	}

/*****************************************************************************/

vec3 DoVibrance (vec3 rgb)
	{
	
	if (uVibrance > 0.0)
		{
		rgb = DoIncreaseVibrance (rgb, uVibrance);
		}

	else if (uVibrance < 0.0)
		{
		rgb = DoDecreaseVibrance (rgb, uVibrance);
		}
	
	return rgb;
	
	}

/*****************************************************************************/

float ContrastBump (float lum)
	{
	
	float contrastBump;
	
	if (lum <= 0.5)
		{
		
		float f = (lum - 0.25) * 4.0;
		
		contrastBump = f * f - 1.0;
		
		}
		
	else
		{
		
		float f = (lum - 0.75) * 4.0;
		
		contrastBump = 1.0 - f * f;
		
		}
		
	// The above curve generates something with a slope of 8 at the center
	// point. Scale it back down to have a slope of 1 at the center point
	// so we can worry simply about adding or removing some to adjust the
	// slope.
		
	contrastBump *= 0.125;
	
	return contrastBump;
	
	}

/*****************************************************************************/

float DodgeBurnSlope (float x)
	{
	
	return pow (2.0, x * 4.0);
	
	}

/*****************************************************************************/

float ShiftBrightnessAndContrast (float lum,
								  float brightnessShift,
								  float contrastShift,
								  float contrastSplitSlope,
								  float invContrastSplitSlope)
	{
	
	// We approximate an exponential function based on stops using a range
	// of 4 stops either direction.
	
	//float dbSlope = pow (2.0, brightnessShift * 4.0);//dodgeBurnSlope (brightnessShift);
	float dbSlope = DodgeBurnSlope (brightnessShift);
	
	if (contrastShift == 0.0)
		{
		
		float lum_db = dbSlope * lum;
		
		lum = lum_db / (1.0 + lum_db - lum);
		
		}
		
	else
		{
		
		float slope1 = dbSlope * contrastSplitSlope;
		
		float slope2 = invContrastSplitSlope;
		
		float lum_scaled_1 = slope1 * lum;
		
		lum = lum_scaled_1 / (1.0 + lum_scaled_1 - lum);
		
		float contrastSlope = pow (2.0, 0.75 * contrastShift);
		
		float contrastBump = ContrastBump (lum);
			
		if (1.0 <= contrastSlope)
			{
			
			// We can do a certain portion of the contrast slope adjustment
			// using a curve similar to the global contrast curve up through
			// a gain to a slope of 1.5.
		
			lum += contrastBump * min (0.5, contrastSlope - 1.0);
				
			contrastSlope = max (1.0, contrastSlope * 0.6666666667);	// Factor out the portion we've applied
				
			}
			
		else
			{
			
			lum += contrastBump * max (-0.5, contrastSlope - 1.0);
			
			contrastSlope = min (1.0, contrastSlope * 2.0); // Factor out the portion we've applied.
			
			}
			
		// Handle all other cases including negative contrast via the gentler curve
		// based on the div operation.
		
		if (contrastSlope != 1.0)
			{
			
			float lum_expanded = 2.0 * lum - 1.0;
		
			float lum_expanded_scaled = lum_expanded * contrastSlope;
			
			float lum_expanded_scaled_b = lum_expanded_scaled / (1.0 + (contrastSlope - 1.0) * abs (lum_expanded));
			
			lum = 0.5 + 0.5 * lum_expanded_scaled_b;
			
			}
			
		// Convert back out of non-linear space.
		
		float lum_scaled_2 = slope2 * lum;
		
		lum = lum_scaled_2 / (1.0 + lum_scaled_2 - lum);
		
		}
	
	return lum;

	}

/*****************************************************************************/

vec3 DoTuneHSLMasked32 (vec3 minMaxHue,
						float lMask,		 // local Brightness mask value
						float cMask,		 // local Contrast mask value
						float sMask)		 // local Saturation mask value
	{
	
	float contrastSplit = uToneContrastSplit;
	
	const float kMaxSaturationGain = 6.0;
	
	const float kBaseSaturationFactor = 1.0 - (1.0 / kMaxSaturationGain);
	
	const float tableScale = (1.0 / 6.0);
	
	float contrastSplitSlope = (1.0 - contrastSplit) / contrastSplit;
	
	float invContrastSplitSlope = contrastSplit / (1.0 - contrastSplit);
	
	const float kExponentialSaturationFactor = 2.5;
		// Somewhat less than log2( 6.0 ). We gain more slowly past 100%.

	float minValue = minMaxHue.x;
	float maxValue = minMaxHue.y;
	float hue = minMaxHue.z;

	float newHue = hue;

	float initialGap = maxValue - minValue;

	float maskBrightnessShift =  lMask;
	float maskContrastShift   =  cMask;
	float maskSaturationShift =  sMask;

	bool doMaskBrightnessShift = maskBrightnessShift != 0.0;
	bool doMaskContrastShift = maskContrastShift != 0.0;

	bool doMaskBrightnessContrastShift = doMaskBrightnessShift || doMaskContrastShift;

	float hslSaturationShift = 0.0;

	if (initialGap != 0.0)
		{

		// Interpolate tables.

		float index = hue * tableScale;
		
		vec3 hslLookup = BigTableLookup (index,
										 uBigTable_HSL,
										 uHSLCoord0,
										 uHSLCoord1).rgb;

		newHue					 = hslLookup.x;
		hslSaturationShift		 = hslLookup.y;
		float hslLuminanceShift	 = hslLookup.z;

		if (hslLuminanceShift != 0.0)
			{

			// We measure a value akin to saturation by taking the delta
			// between max and min. Note that we use the gap before any other
			// processing was applied.

			float gap2 = initialGap * (2.0 - initialGap);

			float gap4 = gap2 * (2.0 - gap2);
			float gap8 = gap4 * (2.0 - gap4);

			hslLuminanceShift *= gap8;

			// Do the actual adjustment on max and minValue.

			minValue = minValue * (1.0 + hslLuminanceShift * (1.0 - minValue));
			maxValue = maxValue * (1.0 + hslLuminanceShift * (1.0 - maxValue));

			minValue = minValue * (1.0 + hslLuminanceShift * (1.0 - minValue));
			maxValue = maxValue * (1.0 + hslLuminanceShift * (1.0 - maxValue));

			}

		}

	// Adjust the luminance.

	if (doMaskBrightnessContrastShift)
		{

		minValue = ShiftBrightnessAndContrast (minValue, maskBrightnessShift, maskContrastShift,
											   contrastSplitSlope, invContrastSplitSlope);

		maxValue = ShiftBrightnessAndContrast (maxValue, maskBrightnessShift, maskContrastShift,
											   contrastSplitSlope, invContrastSplitSlope);

		}

	// Figure out how to handle saturation.

	if (minValue < maxValue)
		{

		float saturationShift = hslSaturationShift + maskSaturationShift;

		if (saturationShift < 0.0)
			{

			saturationShift = max (-1.0, saturationShift);

			float L = 0.5 * (maxValue + minValue);

			minValue += saturationShift * (minValue - L);
			maxValue += saturationShift * (maxValue - L);

			}

		else if (0.0 < saturationShift)
			{

			float gap = maxValue - minValue;

			float saturation = gap / maxValue;

			// Begin CALC_SKIN_PROTECTION

			float shiftedHue = hue + 1.0;

			if (6.0 <= shiftedHue)
				shiftedHue -= 6.0;

			float minSide = clamp (1.0 - (kMinScaledCentralSkinHue - shiftedHue) * kMinSideFalloff, 
								   0.0,
								   1.0);	

			float maxSide = clamp (1.0 - (shiftedHue - kMaxScaledCentralSkinHue) * kMaxSideFalloff, 
								   0.0,
								   1.0);

			float protection = min (minSide, 
									maxSide);

			protection *= (1.0 - saturation * saturation);							

			// End CALC_SKIN_PROTECTION

			// We use the skin protection to reduce the contribution from the
			// localized corrections. Unfortunately, the global corrections
			// are already well defined so we can't use this with them.

			float availableForProtection = clamp (maskSaturationShift, 0.0, saturationShift);

			saturationShift -= 0.5 * availableForProtection * protection;

			// Note: Conceivably we should do this with values from before
			// the luminance adjustment. The 1.0 logic, howecer, uses values
			// from after so we do so as well.

			// Limit the effect in bright highlights because we don't want
			// to fight with issues from highlight recovery.

			saturationShift *= 1.0 - minValue;

			// We don't want to apply the effect in the deepest shadows because
			// most of the information there is just noise.

			float shadowRolloff = min (1.0, 16.0 * maxValue);

			// Smooth it out by doing a self-screen.

			shadowRolloff *= 2.0 - shadowRolloff;

			saturationShift *= shadowRolloff;

			// Split the saturation into logic below 1.0 in which case we
			// use the 1.0 curve and values above 1.0 where we use an
			// exponential function.

			if (saturationShift <= 1.0)
				{

				float saturationFactor = kBaseSaturationFactor * saturationShift;

				saturation = saturation / (1.0 - saturationFactor * (1.0 - saturation));

				}

			else
				{

				float saturationFactor = 6.0 * pow (2.0, kExponentialSaturationFactor * (saturationShift - 1.0));

				float scaledSaturation = saturationFactor * saturation;

				saturation = scaledSaturation / (1.0 + scaledSaturation - saturation);

				}

			minValue = maxValue - saturation * maxValue;

			}

		}

	minMaxHue.x = minValue;
	minMaxHue.y = maxValue;
	minMaxHue.z = newHue;

	return minMaxHue;
		
	}

/*****************************************************************************/

vec3 DoVibranceHSL (vec3 rgb)
	{

	// Go to min-max-hue space.

	vec3 mmh = DoRGBtoMMH (rgb);

	// Vibrance.

	mmh = DoVibrance (mmh);

	// HSL.

	if (uHasHSL)
		{

		// BULLSHIT: currently set local adjustments to 0.

		float localLuminanceMask = 0.0;
		float localContrastMask  = 0.0;
		float localSatMask       = 0.0;

		mmh = DoTuneHSLMasked32 (mmh,
								 localLuminanceMask,
								 localContrastMask,
								 localSatMask);

		}

	// Back to RGB space.

	rgb = DoMMHtoRGB (mmh);

	return rgb;	
	
	}

/*****************************************************************************/

vec3 DoColorPath (vec3 rgb)
	{
	
	// Saturation.

	rgb = DoSaturation (rgb);

	// Minus clarity.

	#if 0

	if (uHasLocalContrast)
		{

		rgb = DoLocalContrast (rgb, 
							   dstPt,
							   localClarityMask);

		}

	#endif

	// Red eye.

	// ...

	// Look table.

	#if 0

	if (uHasLookTable)
		{

		rgb = DoColorTable3D (rgb,
							  uLookTable,
							  uLookTableCoordsFirst,
							  uLookTableCoordsLast);

		}

	#endif

	// Vibrance and HSL.

	if (uHasVibranceHSL)
		{

		rgb = DoVibranceHSL (rgb);

		}

	return rgb;
	
	}

/*****************************************************************************/

vec3 DoGrayMixer (vec3 rgb)
	{
	
	float tableScale = (1.0 / 6.0);
	
	float r = rgb.r;
	float g = rgb.g;
	float b = rgb.b;

	float minValue, maxValue, hue;

	// Mirror 0: Force g >= b

	bool mirror0 = (b > g);

	if (mirror0)
		{

		float temp = g;

		g = b;
		b = temp;

		}

	// Mirror 1: Force r >= b

	bool mirror1 = (b > r);

	if (mirror1)
		{

		float temp = r;

		r = b;
		b = temp;

		}

	// Mirror 2: Force r >= g

	bool mirror2 = (g > r);

	if (mirror2)
		{

		float temp = g;

		g = r;
		r = temp;

		}

	if (r == b)
		{
		return vec3 (b);
		}

	else
		{
		hue = (g - b) / (r - b);
		}

	if (mirror2)
		{
		hue = 2.0 - hue;
		}

	if (mirror1)
		{
		hue = 4.0 - hue;
		}

	if (mirror0)
		{
		hue = 6.0 - hue;
		}

	minValue = b;
	maxValue = r;
	hue = hue;

	float gap = maxValue - minValue;

	// Interpolate tables.

	#if 0

	float tableIndex = hue * tableScale;

	int32 index = (int32) tableIndex;

	float fract = tableIndex - (float) index;

	float grayBase	= weights [index];
	float graySlope = weights [index + 1] - grayBase;

	float gray = grayBase + fract * graySlope;

	#else

	// Let the hardware do the lerp.

	float tableIndex = hue * tableScale;

	// Remap index to account for texel centers.

	/*
	tableIndex = mix (uHSLGrayMixerCoords.x,
					  uHSLGrayMixerCoords.y,
					  tableIndex);

	float gray = texture2D (uHSLGrayTex, 
							vec2 (tableIndex, 
								  uHSLGrayRows.w)).r;
	*/

	tableIndex = saturate (tableIndex);

	float gray = BigTableLookup (tableIndex,
								 uBigTable_GrayMixer,
								 uGrayMixerCoord0,
								 uGrayMixerCoord1).x;

	#endif

	float result;

	// To avoid noise build up from increased contrast with negative values,
	// we scale the gap to reflect the available space on the downward side of
	// minValue.

	if (gray < 0.0)
		{

		// Translate into an alternative space. To do the blending if gray
		// level is encoded. Using linear space produces overly noisy results.

		// We approximate a square root with a piecewise curve. This actually
		// is relatively close to x ^ (1 / 2.2) in the lower portions of the
		// curve. It's much closer to the identity line for higher values than
		// a square root let alone a gamma 1 / 2.2 curve. This seems to
		// produce pretty good results, however, and it trades square roots
		// for divides which only kick in part of the time.

		float minT;

		if (9.0 * minValue < 1.0)
			minT = (3.0 * minValue) / (0.25 + 6.75 * minValue);
		else
			minT = 0.25 + 0.75 * minValue;

		float maxT;

		if (9.0 * maxValue < 1.0)
			maxT = (3.0 * maxValue) / (0.25 + 6.75 * maxValue);
		else
			maxT = 0.25 + 0.75 * maxValue;

		float resultT = saturate (minT + gray * (maxT - minT));

		if (3.0 * resultT < 1.0)
			result = resultT / (12.0 - 27.0 * resultT);
		else
			result = (resultT - 0.25) / 0.75;

		}

	else
		{

		result = minValue + gray * gap;

		}

	result = saturate (result);

	return vec3 (result);

	}

/*****************************************************************************/

vec3 DoGrayPath (vec3 rgb)
	{
	
	// Look table.

	#if 0

	if (uHasLookTable)
		{

		rgb = DoColorTable3D (rgb,
							  uLookTable,
							  uLookTableCoordsFirst,
							  uLookTableCoordsLast);

		}

	#endif

	// Gray mixer.

	rgb = DoGrayMixer (rgb);

	// Minus clarity.

	#if 0

	if (uHasLocalContrast)
		{

		rgb = DoLocalContrast (rgb, 
							   dstPt,
							   localClarityMask);

		}

	#endif

	// Red eye.

	// ...

	// Luminance tuner.

	#if 0

	if (localLuminanceMask != 0.0 ||
		localContrastMask  != 0.0)
		{

		rgb = DoTuneLuminanceMasked32 (rgb,
									   localLuminanceMask,
									   localContrastMask);

		}

	#endif

	return rgb;
	
	}

/*****************************************************************************/

vec3 DoColorGray (vec3 rgb)
	{
	
	if (uConvertToGray)
		{

		rgb = DoGrayPath (rgb);

		}

	else
		{
			
		rgb = DoColorPath (rgb);
			
		}

	return rgb;
	
	}

/*****************************************************************************/

// End color routines.

/*****************************************************************************/

vec3 RGBTone (vec3 rgb,
			  float tableIndex)
	{

	float rr = BigTableLookup (rgb.r, tableIndex).r;
	float bb = BigTableLookup (rgb.b, tableIndex).r;

	float gg = bb + ((rr - bb) * (rgb.g - rgb.b) / (rgb.r - rgb.b));

	return vec3 (rr, 
				 gg, 
				 bb);

	}

/*****************************************************************************/

vec3 DoToneCurve (vec3 rgb,
				  float tableIndex)
	{

	float r = rgb.r;
	float g = rgb.g;
	float b = rgb.b;

	vec3 result;

	if (r >= g)
		{

		if (g > b)
			{

			// Case 1: r >= g > b

			result = RGBTone (rgb, 
							  tableIndex).rgb;

			}

		else if (b > r)
			{

			// Case 2: b > r >= g

			result = RGBTone (vec3 (b, r, g),
							  tableIndex).gbr;

			}

		else if (b > g)
			{

			// Case 3: r >= b > g

			result = RGBTone (vec3 (r, b, g),
							  tableIndex).rbg;

			}

		else
			{

			// Case 4: r >= g == b

			//DNG_ASSERT (r >= g && g == b, "Logic Error 2");

			float gg = BigTableLookup (g, tableIndex).r;

			result = vec3 (BigTableLookup (r, tableIndex).r,
						   gg,
						   gg);

			}

		}

	else
		{

		if (r >= b)
			{

			// Case 5: g > r >= b

			result = RGBTone (vec3 (g, r, b),
							  tableIndex).grb;

			}

		else if (b > g)
			{

			// Case 6: b > g > r

			result = RGBTone (vec3 (b, g, r),
							  tableIndex).bgr;

			}

		else
			{

			// Case 7: g >= b > r

			result = RGBTone (vec3 (g, b, r),
							  tableIndex).brg;

			}

		}

	return result;
	
	}

/*****************************************************************************/

vec3 DoToneCurve (vec3 rgb)
	{

	return DoToneCurve (rgb, uBigTable_ToneCurve);

	}

/*****************************************************************************/

vec3 UndoToneCurve (vec3 rgb)
	{

	return DoToneCurve (rgb, uBigTable_UndoToneCurve);

	}

/*****************************************************************************/

vec3 DoRGBCurves (vec3 rgb)
	{
	
	return vec3 (BigTableLookup (rgb.r, uBigTable_CurveRed  ).x,
				 BigTableLookup (rgb.g, uBigTable_CurveGreen).x,
				 BigTableLookup (rgb.b, uBigTable_CurveBlue ).x);
	
	}

/*****************************************************************************/

vec3 DivMapEvaluate (vec3 x,
					 float alpha)
	{
	
	vec3 scaled = vec3 (alpha) * x;
			
	return scaled / (vec3 (1.0) + scaled - x);
	
	}

/*****************************************************************************/

vec3 DivMapEvaluateInverse (vec3 y,
							float alpha)
	{
	
	vec3 scaled = y / vec3 (alpha);
			
	return scaled / (vec3 (1.0) + scaled - y);
	
	}

/*****************************************************************************/

// Do the math directly (instead of using LUTs).

vec3 DoSplitTone (vec3 rgb)
	{

	vec3 y = rgb;
	
	y = DivMapEvaluate (y, uSplitToneBalanceMapAlpha);

	y = y * ((vec3 (1.0) - y) * (uSplitToneShadowFactor + uSplitToneDeltaFactor * y) + y * y);
		
	return DivMapEvaluateInverse (y, uSplitToneBalanceMapAlpha);
	
	}

/*****************************************************************************/

float DoProfileLensVignette (vec2 pt)
	{

	// Map pre-warp relative coordinates to squared radius. The
	// uVignetteProfileScale and uVignetteProfileOffset constants account for
	// the fact that the optical center may not correspond to the image
	// center.

	vec2 dxdy = pt * uVignetteProfileScale + uVignetteProfileOffset;

	float rr = dot (dxdy, dxdy);

	// Let the hardware deal with restricting the useful range of the rr
	// coordinate to [0, 1], i.e., via the clamp-to-edge behavior.

	return BigTableLookup (rr, uBigTable_ProfileVignette).x;

	}

/*****************************************************************************/

float DoManualLensVignette (float rr,
							float amount,
							float midpoint,
							float amountScale)
	{

	// See cr_vignette.cpp.
	
	float power = 1.0 + 4.0 * midpoint;

	float scale = pow (2.0, abs (amount) * amountScale) - 1.0;

	float y = 1.0 + scale * pow (rr, power);

	if (amount < 0.0)
		{

		y = 1.0 / y;

		}

	return y;

	}

/*****************************************************************************/

float DoManualLensVignette (float rr)
	{

	return DoManualLensVignette (rr,
								 uLensVignetteAmount,
								 uLensVignetteMidpoint,
								 2.0);

	}

/*****************************************************************************/

vec3 DoLensVignette (vec3 image, 
					 vec2 pt)
	{


	float scale = 1.0;

	if (uHasLensManualVignette)
		{

		float rr = NormalizedSquaredPinnedDistanceFromCenter (pt);

		scale *= DoManualLensVignette (rr);

		}

	if (uHasLensProfileVignette)
		{

		scale *= DoProfileLensVignette (pt);

		}

	scale = max (scale, 0.0);

	return saturate (image * scale);

	}

/*****************************************************************************/

// Begin post-crop vignette routines.

/*****************************************************************************/

float PowerScreen (float x, float s)
	{
	
	if (x <= 0.0)
		{
		
		return pow (1.0 + x, s) - 1.0;
		
		}
		
	else
		{
		
		return 1.0 - pow (1.0 - x, s);
		
		}
	
	}

/*****************************************************************************/

// Note: x represents squared radius (not linear radius).

float DoPostCropVignette (float x,
						  float amount)
	{

	float fMidpoint = uPostCropVignetteMidpoint;

	float fFeather = uPostCropVignetteFeather;

	// For process version 2 (i.e., CR 6 / LR 3), there have been requests
	// from the prerelease group to be able to achieve the "look" of the
	// original lens vignette feature via post-crop vignette -- essentially,
	// the power falloff. Here, I have attempted to implement this by blending
	// process version 1's post-crop vignette function (i.e., from CR 5 / LR
	// 2) with a stronger form of the lens vignette function, using the
	// feather parameter to lerp between the two. -erichan 10/28/2009.

	// If we're planning to blend the post-crop vignette result with the lens
	// vignette result (i.e., for process version 2), then compute the lens
	// vignette result first.

	float resultLensVignette = 0.0;

	if (uBlendStyleLensVignette && (amount < 0.0))
		{

		resultLensVignette = DoManualLensVignette (x,
												   amount,
												   fMidpoint,
												   5.0);

		}

	float feather = uPostCropVignetteFeather;
	
	// For the formula below for calculating fs, the feedback from the beta
	// group says we should make 50% be somewhat more feathered. So we run it
	// through a little mapping function...
	
	feather = 1.5 * feather / (1.0 + 0.5 * feather);
	
	const float kMaxSlope = 8.0;
	
	float fs = 1.0 / (feather + (1.0 - feather) / kMaxSlope);
	
	fs = fs * fs;
	
	float target = sqrt (0.5) + (2.0 * fMidpoint - 1.0) * (sqrt (0.5) - sqrt (0.20)) * (0.75 + 0.25 * fMidpoint);
	
	target = target * target;

	float ts = (1.0 - target) / target;
	
	x = ts * x / (1.0 + (ts - 1.0) * x);		// Move target to 0.5
	
	x = 2.0 * x - 1.0;							// Map to -1..+1
	
	x = PowerScreen (x, fs);					// Contrast slope application
	
	x = 0.5 * (x + 1.0);						// Map back to 0..1

	float finalResult;

	if (amount < 0.0)
		{

		// Compute result of blending black with white in sRGB.
		
		float blended_sRGB = 1.0 - x;
		
		// Now convert back to linear.
		
		float blended_linear = sRGB_EvaluateInverse (blended_sRGB);
			
		float scaled_sRGB = 1.0 + amount;
		
		float scaled_linear = sRGB_EvaluateInverse (scaled_sRGB);
			
		float result = 1.0 - (1.0 - scaled_linear) * (1.0 - blended_linear);

		// If we're not planning to blend this result with the lens vignette result,
		// then stop here.

		if (!uBlendStyleLensVignette)
			{
			return result;
			}

		// We use the feather parameter to control the blend between the original
		// post-crop vignette result (i.e., process version 1) with the original lens
		// vignette result. We push the feather parameter through the sRGB tone curve
		// to make the interpolation factor more perceptually uniform across the
		// slider range [0, 100].

		float lerpFactor = sRGB_EvaluateInverse (fFeather);

		// Interpolate between the two results. This means that folks who crave the
		// original lens vignette "look" can achieve it by setting Feather to 100
		// (i.e., lerpFactor becomes 1).

		finalResult = mix (result, 
						   resultLensVignette, 
						   lerpFactor);

		}
		
	else
		{
		
		// Compute result of blending white with black in sRGB.
		
		float blended_sRGB = x;
		
		// Now convert back to linear.

		float blended_linear = sRGB_EvaluateInverse (blended_sRGB);
			
		float scaled_sRGB = amount;
		
		float scaled_linear = sRGB_EvaluateInverse (scaled_sRGB);
			
		finalResult = 1.0 - scaled_linear * blended_linear;
		
		}

	return finalResult;
	
	}

/*****************************************************************************/

// Compute crop ellipse parameters A, B, C, D, and E, such that:
//
// x = row + D;
// y = col + E;
//
// A * x * x + B * x * y + C * y * y == 1.0

float RefEllipseMask32 (vec2 pt)
	{
	
	float x = (pt.y * uDefaultCropSize.y) + uPostCropVignetteEllipseD;

	float Axx = uPostCropVignetteEllipseA * x * x;
		
	float Bx = uPostCropVignetteEllipseB * x;
			
	float y = pt.x * uDefaultCropSize.x + uPostCropVignetteEllipseE;
							
	float r2 = Axx + (Bx + uPostCropVignetteEllipseC * y) * y;

	#if 1

	// Table math.

	float z = BigTableLookup (r2, uBigTable_PostCropVignette).x;

	#else

	// Direct math.
			
	r2 = min (r2, 1.0);
			
	float z = DoPostCropVignette (r2, uPostCropVignetteAmount);

	#endif
			
	return z;

	}

/*****************************************************************************/

// BULLSHIT: Optimize this: 
// - factored out non-per-pixel code
// - vectorize

float RefRoundedRectMask32 (vec2 pt)
	{
	
	float flatX = 0.0;
	float flatY = 0.0;

	float croppedAspectRatio = uPostCropVignetteCropAspectRatio;

	float roundness = uPostCropVignetteRoundness;
	
	if (croppedAspectRatio >= 1.0)
		{
		
		flatX = 0.45 * (-roundness);
		
		flatY = max (0.0, 0.5 - ((0.5 - flatX) * croppedAspectRatio));
		
		}
		
	else
		{
		
		flatY = 0.45 * (-roundness);
		
		flatX = max (0.0, 0.5 - ((0.5 - flatY) / croppedAspectRatio));
		
		}
	
	float scaleX = 0.5 / (0.5 - flatX);
	float scaleY = 0.5 / (0.5 - flatY);
			
	pt.x *= uDefaultCropSize.x;
	pt.y *= uDefaultCropSize.y;

	// Need to swizzle because the matrix was originally designed to be used
	// with a dng_point object which has (x,y) components swpped.
			
	pt = (uPostCropVignetteCropTransform * vec3 (pt.yx, 1.0)).yx;
			
	float x = abs (pt.x - 0.5);
	float y = abs (pt.y - 0.5);
			
	x = max (x - flatX, 0.0) * scaleX;
	y = max (y - flatY, 0.0) * scaleY;
			
	float r2 = (x * x + y * y) * 2.0;

	#if 1

	// Table math.

	float z = BigTableLookup (r2, uBigTable_PostCropVignette).x;

	#else

	// Direct math.
							
	r2 = min (r2, 1.0);

	float z = DoPostCropVignette (r2, uPostCropVignetteAmount);

	#endif

	return z;

	}

/*****************************************************************************/

float DoPostCropVignette (vec2 pt)
	{
	
	if (uPostCropVignetteRoundness >= 0.0)
		{
		return RefEllipseMask32 (pt);
		}

	else
		{
		return RefRoundedRectMask32 (pt);
		}
	
	}

/*****************************************************************************/

vec3 DoDarkPostCropVignette (vec3 image,
							 vec2 pt)
	{

	float m = DoPostCropVignette (pt);

	float scale = m;

	if (uPostCropVignetteHighlightContrast > 0.0 &&
		!uHasPostCropVignettePaintOverlay)
		{

		float maxScale = mix (m,
							  1.0,
							  uPostCropVignetteHighlightContrast);

		const float kHLThresh =	 0.18;
		const float kHLScale  =	 1.0 / (1.0 - kHLThresh);
		const float kHLOffset = -kHLThresh * kHLScale;

		vec3 hsv = RGBtoHSV (image);

		float v = hsv.z;

		float t = saturate ((v * kHLScale) + kHLOffset);

		scale = mix (m, maxScale, t);

		hsv.z = v * scale;

		image = HSVtoRGB (hsv);

		return image;

		}

	// Normal case.

	return image * scale;

	}

/*****************************************************************************/

vec3 DoWhitePostCropVignette (vec3 image,
							  vec2 pt)
	{
	
	float m = DoPostCropVignette (pt);

	// Invert image, apply mask, and re-invert.

	return vec3 (1.0) - saturate ((vec3 (1.0) - image)) * m;
	
	}

/*****************************************************************************/

vec3 DoPostCropVignette (vec3 image,
						 vec2 pt)
	{

	pt = ToDefaultCrop (pt);
	
	if (uPostCropVignetteAmount < 0.0)
		{
		
		// Black direction.

		image = DoDarkPostCropVignette (image, pt);
		
		}

	else
		{
		
		// White direction.

		image = DoWhitePostCropVignette (image, pt);
		
		}

	return image;
	
	}

/*****************************************************************************/

// End post-crop vignette routines.

/*****************************************************************************/

vec2 DoPerspective (vec2 pt)
	{

	vec3 xyh = uPerspectiveMatrix * vec3 (pt, 1.0);

	xyh.z = clamp (xyh.z, 
				   uPerspectiveMinScale,
				   uPerspectiveMaxScale);

	pt = xyh.xy / xyh.z;

	return pt;

	}

/*****************************************************************************/

vec2 DoDistort (vec2 pt,
				float radiusScale)
	{

	// Subtract center, scale the deltas, and add the center back.

	const vec2 center = vec2 (0.5, 0.5);

	return center + (radiusScale * (pt - center));

	}

/*****************************************************************************/

vec2 DoManualDistort (vec2 pt)
	{

	// Manual distortion is computed relative to the default crop area
	// (not the full stage 3 image area).

	vec2 ptDefaultCrop = ToDefaultCrop (pt);

	float rr = NormalizedSquaredPinnedDistanceFromCenter (ptDefaultCrop);

	float scale = (1.0 - uManualDistortB) + uManualDistortB * rr;

	pt = DoDistort (pt, 
					scale);

	return pt;

	}

/*****************************************************************************/

vec2 DoProfileDistort (vec2 pt)
	{

	// Profile-based distortion is computed relative to the full stage 3 image area
	// (not the default crop area).

	vec2 diff = (pt - uProfileWarpCenter) * uProfileWarpToTable;

	float rr = dot (diff, diff);

	float r = sqrt (rr);

	vec2 ratio = BigTableLookup (r, uBigTable_ProfileDistortion).xx;

	diff = diff * ratio;

	return uProfileWarpCenter + (diff * uProfileWarpFromTable);

	}

/*****************************************************************************/

float DoWarpMask (vec2 srcPt)
	{
	
	const vec2 kPixelOffset = vec2 (0.5, 0.5);

	vec2 size = uDefaultCropSize * uRenderScale;

	vec2 ptPixels = ToDefaultCrop (srcPt) * size;

	vec2 clipTL =        kPixelOffset;
	vec2 clipBR = size - kPixelOffset;

	float alpha = (saturate (1.0 - max (clipTL.x   - ptPixels.x,
										ptPixels.x - clipBR.x)) *

				   saturate (1.0 - max (clipTL.y   - ptPixels.y,
										ptPixels.y - clipBR.y)));

	return alpha;	
	
	}

/*****************************************************************************/

// Draw gray around image instead of showing replicated pixels.

// BULLSHIT: Need to update this for various transparency options.

vec4 DoTransparency (vec4 image,
					 vec2 srcPt,
					 vec2 dstPt)
	{

	// Compute transparency.

	// Warp mask.

	image.a *= DoWarpMask (srcPt);

	// Crop mask.

	// image.a *= DoCropMask (srcPtCrop);

	// Zap white.

	if (uOnlyPurelyTransparent && (image.a > 0.0))
		{
		image.a = 1.0;
		}

	// Optionally flatten transparency.

	if (uFlattenTransparency)
		{

		// Initialize matte value to specified value.

		float matteValue = uMatteGrayLevel;

		// Optionally replace with checkerboard.

		if (uDoCheckerboard)
			{

			const float kDarkCheck = 0.625;

			vec2 rounded = mod (floor (dstPt * uStage3Size) - uCheckerboardOrigin,
								vec2 (uCheckerboardSize, uCheckerboardSize));

			float checkerThresh = uCheckerboardSize * 0.5 - 0.5;

			bool test1 = (rounded.x < checkerThresh);
			bool test2 = (rounded.y < checkerThresh);

			matteValue = mix (matteValue, 
							  kDarkCheck, 
							  test1 != test2);

			}

		// Flatten.

		image.rgb = mix (vec3 (matteValue),
						 image.rgb, 
						 vec3 (image.a));

		image.a = 1.0;

		}

	// Optionally multiply alpha into color data.

	if (uMultiplyTransparency)
		{
		image.rgb *= image.a;
		}

	return image;	
	
	}

/*****************************************************************************/

void DoLateralCA (vec2 pt,
				  out vec2 ptR,
				  out vec2 ptB)
	{

	// BULLSHIT: This only works for 3-color images. For 4-color images, we
	// need to correct for 3 planes, with 2 scalars each, so a total of 6
	// values. We would need 2 lookups.

	// Map to default crop coordinates.

	pt = ToDefaultCrop (pt);

	// Apply correction.

	vec4 offset = texture2D (uLateralCAMap, pt);

	ptR = pt + offset.xy;
	ptB = pt + offset.zw;

	// Back to stage 3 coordinates.

	ptR = FromDefaultCrop (ptR);
	ptB = FromDefaultCrop (ptB);

	}

/*****************************************************************************/

vec4 GetStage3 (vec2 dstPt,
				out vec2 srcPt,
				out vec2 srcPtCrop)
	{

	vec2 pt = dstPt;

	// Apply crop transform. Need to swizzle because the matrix was originally
	// designed to be used with a dng_point object which has (x,y) components
	// swpped.

	pt = (uBackwardCropTransform * vec3 (pt.yx, 1.0)).yx;

	// Store a copy of the coordinates immediately after having been mapped
	// thru the backward crop transform. This copy will not be mapped thru the
	// backward warp transform.

	srcPtCrop = pt;

	// Detect no-warp case.

	if (!uHasWarp)
		{

		srcPt = pt;

		return texture2D (uStage3Image, pt);

		}
		
	// Work backwards from dst (ideal, corrected) to src (real, uncorrected).

	// Perspective (Horizontal, Vertical, Rotate, Scale, Aspect, Upright).

	if (uHasPerspective)
		{

		pt = DoPerspective (pt);

		}

	// Manual distortion.

	if (uHasManualDistort)
		{

		pt = DoManualDistort (pt);

		}

	// Profile distortion.

	if (uHasProfileWarp)
		{

		pt = DoProfileDistort (pt);

		}

	// Lateral CA.

	vec2 ptR = pt;
	vec2 ptB = pt;

	vec4 stage3;

	if (uHasLateralCA)
		{
		
		DoLateralCA (pt, ptR, ptB);

		// BULLSHIT: Need to handle 4-color images.

		vec4 stage3base = texture2D (uStage3Image, pt);

		stage3 = vec4 (texture2D (uStage3Image, ptR).r,
					   stage3base.g,
					   texture2D (uStage3Image, ptB).b,
					   stage3base.a);
		
		}

	else
		{
		
		stage3 = texture2D (uStage3Image, pt);
		
		}

	srcPt = pt;

	return stage3;

	}

/*****************************************************************************/

// dstPt is specified in stage 3 image-relative coordinates. In this
// coordinate system, (0,0) is the top-left corner of the top-left pixel of
// the stage 3 image, and (1,1) is the bottom-right corner of the bottom-right
// pixel of the stage 3 image.

vec4 Process2012 (vec2 dstPt)
	{
	
	// Get image pixel values and source image positions.
	// 
	// srcPt is the source image point before the crop transform and before
	// all warps. srcPtCrop is the source image point immediately before the
	// crop transform, but after all warps; this is useful for doing the
	// post-crop vignette math.

	vec2 srcPt;
	vec2 srcPtCrop;

	vec4 stage3 = GetStage3 (dstPt, 
							 srcPt, 
							 srcPtCrop);

	// BULLSHIT: deal with 4-color images.

	vec3 result = stage3.rgb;

	float alpha = stage3.a;

	// Sketch of PV 2012 pipe below.

	// gray image path: 
	// - gray to rgb
	// + non-raw: undo render curve

	// 3-color path:
	// + non-raw: undo render curve
	// - moire: skip
	// - abc2rgb

	// 4-color path:
	// - abcd2rgb
	// - moire: skip

	if (uHasUndoToneCurve)
		{

		result = UndoToneCurve (result);

		}

	/*

	  Moire strategy:

		  - convert to YCC
		  - apply (possibly stale) suppression map/texture (CC delta signal)
		  - back to RGB

	 */

	// SKIP: moire

	// 3-color images: abc to rgb

	if (false)
		{
		
		// BULLSHIT: make image easier to see.

		//result.r *= 2.0;
		//result.b *= 1.4;
		float r = 0.5667;
		float b = 0.7570;

		float scaleR = 1.0 / r;
		float scaleB = 1.0 / b;

		result.r *= scaleR;
		result.b *= scaleB;

		}

	// SKIP: denoise

	/*

	  denoise strategy:

	  - to flat noise
	  - to ycc
	  - apply (possibly stale) denoise delta signal in YCC space
	  - to rgb
	  - to linear space

	 */

	// Color table 1.

	// ...

	// Lens vignette (manual & profile).

	if (uHasLensVignette)
		{

		result = DoLensVignette (result, srcPt);

		}

	// Post-crop vignette (highlight priority).

	if (uHasPostCropVignetteHighlightPriority)
		{

		result = DoPostCropVignette (result, srcPtCrop);

		}

	// Adaptive Tone Mapping.

	// ...

	// Exposure stage (Exposure, Whites, Blacks).

	// ...

	if (false)
		{

		float scale = pow (2.0, uParamExposure);

		result *= scale;

		}

	// color path:
	// + saturation
	// - minus clarity
	// - red eye
	// - look table
	// - HSL, including local

	// gray path:
	// - red eye
	// - look table
	// + gray mixer
	// + gray to rgb
	// - minus clarity
	// - luminance tuner

	// Separate color & grayscale processing paths. Includes minus clarity,
	// look table, red eye, gray mixer, saturation, and HSL/vibrance.

	result = DoColorGray (result);

	// Local color toning/tinting.

	#if 0

	if (localBlueYellowMask	  != 0.0 ||
		localGreenMagentaMask != 0.0)
		{
		
		image = DoLocalColorToning (image,
									localBlueYellowMask,
									localGreenMagentaMask);
		
		}

	#endif

	// Post-crop vignette (color priority).

	if (uHasPostCropVignetteColorPriority)
		{

		result = DoPostCropVignette (result, srcPtCrop);

		}

	// Tone curve (Contrast, profile, non-raw, parametric, point).

	result = DoToneCurve (result);

	// SKIP: Defringe.

	/*

	  Defringe strategy:

		  - go to Lab space
		  - apply (possibly stale) CC delta signal
		  - back to RGB space

	 */

	// RGB curves.

	if (uHasRGBCurves)
		{

		result = DoRGBCurves (result);

		}
	
	// Split tone.

	if (uHasSplitTone)
		{

		result = DoSplitTone (result);

		}

	// SKIP: sharpen.

	/*

	  Sharpen strategy:

	    - Apply sharpen delta/scale signal and maybe some composite math

	 */

	// Apply grain, but no blur.

	#if 0

	if (uHasGrain)
		{

		image = DoGrain (image, dstPt);

		}

	#endif

	// Post-crop vignette (paint overlay).

	if (uHasPostCropVignettePaintOverlay)
		{

		result = DoPostCropVignette (result, srcPtCrop);

		}

	// Rollover mask.

	// ...

	// Find focus.

	// ...

	// SKIP: output sharpen

	// Color transform 1.

	// If gray but split toning, etc. is on, then convert to ProPhoto.

	// ...

	// Clip & threshold warnings. There are two: one for option-key feedback
	// for some controls, and one for highlight & shadow color clipping.
	
	// ...

	// Transparency.

	#if 0

    // Just ignore the tranparancy (and show the replicated edge pixels).
    
    transparencyHandling_Ignore,
    
    // Encode the transparency in an alpha channel.
    
    transparencyHandling_Alpha,
    
    // Encode the transparancy in an alpha channel, and also multiply the
    // final image data by the alpha channel (e.g. "associated alpha" in TIFF).
    
    transparancyHandling_MultipliedAlpha,
    
    // Flatten the transparancy as a gray matte.
    
    transparencyHandling_GrayMatte,
    
    // Flatten the transparancy as a white matte.
    
    transparencyHandling_WhiteMatte,
    
    // Zap purely transparent pixels to white.
    
    transparencyHandling_ZapWhite,
    
    // Flatten the transparancy as a Photoshop-style transparancy checkerboard.
    
    transparancyHandling_Checkerboard

	#endif

	// Color transform 2 (e.g., XYZ for display, or can skip).

	// ...

	// Color transform 3 (e.g., XYZ for display, or can skip).

	// ...

	// Transparency part 2.

	// ...

	// By default, ignore transparency.

	vec4 final = vec4 (result, 1.0);

	if (uDoTransparency)
		{

		// Initialize alpha to transparency image data.

		final.a = alpha;

		final = DoTransparency (final,
								srcPt, 
								dstPt);

		}

	return final;
	
	}

/*****************************************************************************/
