/*****************************************************************************/

// Camera Raw interactive image processing pipeline.

/*****************************************************************************/

// Includes.

[Include] name=ACR_PV2012_Header.glsl
[Include] name=ACR_PV2012_Shared.glsl
[Include] name=ACR_PV2012_Pipe.glsl

/*****************************************************************************/

// Vertex shader.

[Source] name=ACR_VS

[Param]

    in vec3 position;  
    in vec2 uv0; 
    in vec4 color;

    out vec4 positiont; 
    out vec2 uv0;
    out vec4 color;

    uniform mat4 worldToClipTfm;

[Code]

	{

	outputs.uv0       = inputs.uv0;
	outputs.color     = inputs.color;
	outputs.positiont = vec4(inputs.position, 1.0);
    outputs.positiont = mul(outputs.positiont, worldToClipTfm);

	}

/*****************************************************************************/

// Pixel shader.

[Source] name=ACR_PS

/*****************************************************************************/

[Param]

	in vec4 positiont;
	in vec4 color;
	in vec2 uv0;

	out vec4 target0;

/*****************************************************************************/

[Ref] name=PipeFragment

/*****************************************************************************/

[Code]

/*****************************************************************************/

	{

	// Pixel position.

	vec2 dstPt = inputs.uv0;

	// BULLSHIT: For now the UV coordinates are specified relative to the
	// tile, i.e., 0,0 is top-left of tile and 1,1 is bottom-right of tile.
	// This works because the tile is actually the whole image area, so UV
	// coordinates are the same as the negative's normalized (relative) image
	// coordinates. Once we start breaking up the image into subtiles, we need
	// to think about how we specify the coordinates. The two basic ways to do
	// this are: (1) do the math in the client app when specifying the texture
	// coordinates associated with the vertices of the subtiles, so that the
	// interpolated UV coordinates as inputs into the pixel shader already
	// have the correct relative values in the negative's space, or (2)
	// continue to use tile-relative coordinates but do the affine mapping
	// (scale & offset) in the PS to return to the negative space. probably
	// faster to do the former.
	// 
	// e.g.,
	// dstPt = dstPt * uTileToImageScale + uTileToImageOffset;

	// ...

	// BULLSHIT: We'll need to sort out the coordinate convention. For now we
	// need to flip this upside-down on GL.

	dstPt.y = 1.0 - dstPt.y;

	// Run the pipe.

	outputs.target0 = Process2012 (dstPt);

	}

/*****************************************************************************/

[Material] name=ACR_PV2012 vs=ACR_VS ps=ACR_PS

/*****************************************************************************/
