// ----------------------------------------------------------------------------
//
// Copyright (C) 2012 Adobe Systems Incorporated
// All Rights Reserved.
//
// NOTICE:  All information contained  herein is,  and remains the property of
// Adobe Systems Incorporated and its suppliers, if any.  The intellectual and
// technical  concepts  contained  herein  are  proprietary  to  Adobe Systems
// Incorporated  and  its suppliers  and may  be covered  by U.S.  and Foreign
// Patents, patents in process, and are protected by trade secret or copyright
// law.  Dissemination of this information or reproduction of this material is
// strictly forbidden  unless prior  written permission is obtained from Adobe
// Systems Incorporated.
//
// ----------------------------------------------------------------------------

// include this for the fallback shaders, and Helper fragment
[Include] name=TestShaders.glsl


////////////////////////////////////////////////

[Source] name=MyDrawQuadVS

[Param]

    in vec3 position;  
    in vec2 uv0; 
    in vec4 color;

    out vec4 positiont; 
    out vec2 uv0;
    out vec4 color;

    uniform mat4 videoMatrix;

[Code]

	outputs.uv0       = inputs.uv0;
	outputs.color     = inputs.color;
	//outputs.positiont = vec4(inputs.position, 1.0); 
	outputs.positiont = mul( vec4(inputs.position, 1.0), videoMatrix);

///////////////////////////////////////////

[Source] name=MyDrawQuadPS

[Param]

    in vec4 positiont;
    in vec2 uv0;
    in vec4 color;

    out vec4 target0;

[Code]

    outputs.target0 = inputs.color;

///////////////////////////////////////////

[Source] name=MyDrawTexQuadPS

[Param]

    in vec4 positiont;
    in vec2 uv0;
    in vec4 color;

    out vec4 target0;

	uniform sampler2D videoTexture;
	
[Code]

    outputs.target0 = inputs.color * texture2D(videoTexture, inputs.uv0.xy);

///////////////////////////////////////////

// same as MyDrawTexQuad, but repeats red to all channels and does alpha test
//  used for debug text draw.  TODO: move to TestShaders.
[Source] name=MyDrawTexRedQuadPS

[Param]

    in vec4 positiont;
    in vec2 uv0;
    in vec4 color;

    out vec4 target0;

	uniform sampler2D videoTexture;
	
[Code]
    vec4 c = texture2D(videoTexture, inputs.uv0.xy);
    c = c.rrrr; // TODO: handle rgba, rg (ra), and r
    
    clip(c.a * inputs.color.a - 40.5/255.0); // alpha test, must be negative to discard
    
    //c.rgb = 1.0;
    //c.a = 1.0;
    c *= inputs.color;
    
    outputs.target0 = c;
     
////////////////////////////////////////////////

[Source] name=VideoVS

[Param]

    in vec3 position;  
    in vec2 uv0; 
    in vec4 color;

    out vec2 uv0;
    out vec4 positiont; 

    uniform mat4 videoMatrix;

[Code]

	outputs.uv0       = inputs.uv0;
	outputs.positiont = mul( vec4(inputs.position, 1.0), videoMatrix);

//////////////////////////////////////////////////////////

[Fragment] name=ReinterpretColor

[Header]
    #define DOINTERPOLATION 1

[Param]

    #if DOINTERPOLATION

    // TODO: have to comment for now, or splicer will splice these (no preprocessor yet)
    uniform sampler3D lookuptable;
    uniform float     rgbScale;
    uniform float     rgbOffset;

    #endif

[Function]

    #if DOINTERPOLATION

    vec4 ReinterpretColor(vec4 videoColor)
    {
		float alpha_orig = videoColor.a;
        vec3 videoScaledColor = videoColor.rgb*rgbScale+vec3(rgbOffset, rgbOffset, rgbOffset);
        videoColor = vec4(texture3D(lookuptable, videoScaledColor).rgb, alpha_orig);
        return videoColor;
    }

    #else

    vec4 ReinterpretColor(vec4 videoColor)
    {
        return videoColor;
    }

    #endif

//////////////////////////////////////////////////////////

[Fragment] name=AntialiasPixel

[Header]
    #define USE_TEXTURE_BORDER_FOR_FILTERING 0

[Param]

[Function]

    #if USE_TEXTURE_BORDER_FOR_FILTERING

    vec4 AntialiasPixel(vec4 videoColor, vec2 sampleCoord)
    {
        return vec4(videoColor.rgb, srcAlpha*videoColor.a);
    }

    #else

    vec4 AntialiasPixel(vec4 videoColor, vec2 sampleCoord)
    {
		float alpha = clamp( min((0.5 - abs(sampleCoord.x-0.5) )*dstPixelSize.x, (0.5 - abs(sampleCoord.y-0.5) )*dstPixelSize.y) , 0.0, 1.0);
		return vec4(videoColor.rgb, alpha * srcAlpha * videoColor.a);
    }

    #endif



//////////////////////////////////////////////////////////

[Source] name=VideoPS

[Header]
    #define DOLAYER 0

[Param]

    uniform sampler2D videoTexture;

   
    #if DOLAYER
    // TODO: have to comment for now, or splicer will splice these (no preprocessor yet)
    // uniform sampler2D foreground;
    #endif

    in vec2 uv0;
    in vec2 uv1;

    out vec4 target0;

[Ref] name=ReinterpretColor

[Code]

    vec4 videoColor = texture2D(videoTexture, inputs.uv1.xy);
    videoColor      = ReinterpretColor(videoColor);

    #if DOLAYER
    // lerp between video and foreground color
    vec4 foregroundColor  = texture2D(foreground, inputs.uv0.xy);
    vec3 finalColor       = mix(videoColor.rgb, foregroundColor.rgb, foregroundColor.a);
    videoColor            = vec4(finalColor.rgb, 1.0);
    #endif

    outputs.target0 = videoColor;



// BGRA to BGRA
// BGRA to BGRA component Texture2D BGRA Fragment Shader (pass thru)

//////////////////////////////////////////////////////////

// BGRA to BGRA
[Source] name=BGRAToBGRAVideoPS

[Param]



    uniform sampler2D	videoTexture;
	uniform float		srcAlpha;
	uniform vec2		dstPixelSize;

    in vec2 uv0;

    out vec4 target0;

[Ref] name=ReinterpretColor

[Ref] name=AntialiasPixel

[Code]
    vec4 videoColor = texture2D(videoTexture, inputs.uv0.xy);
    videoColor = ReinterpretColor(videoColor);
	outputs.target0 = AntialiasPixel(videoColor, inputs.uv0.xy);

//////////////////////////////////////////////////////////

// ARGB to BGRA
[Source] name=ARGBToBGRAVideoPS

[Param]

    uniform sampler2D	videoTexture;
	uniform float		srcAlpha;
	uniform vec2		dstPixelSize;

    in vec2 uv0;

    out vec4 target0;

[Ref] name=ReinterpretColor

[Ref] name=AntialiasPixel

[Code]
	vec4 videoColor = texture2D(videoTexture, inputs.uv0.xy).yxwz; // totally logical: stuffing ARGB in BGRA texture means here we need to swizzle with yxwz...
    videoColor = ReinterpretColor(videoColor);
	outputs.target0 = AntialiasPixel(videoColor, inputs.uv0.xy);

//////////////////////////////////////////////////////////

//		YCRCB 4:2:2 to 8-bit per component Texture2D BGRA Fragment Shader	
[Source] name=VUYAToBGRAVideoPS

[Param]

    uniform sampler2D	videoTexture;
 	uniform float		srcAlpha;
	uniform vec2		dstPixelSize;
       
    in vec2 uv0;

    out vec4 target0;

[Param]

    uniform vec3 stdR;
    uniform vec3 stdG;
    uniform vec3 stdB;
    uniform vec3 stdBias;

    // not used
    // uniform vec2		textureStep;
    // uniform vec2		normalizedSize;
    // uniform vec2		inverseTextureStep;
    // uniform vec2		inverseNormalizedSize;

[Function]

	vec4 vuya_texture_sampler(	/*in sampler2D texture,*/ in vec2 texcoord)
	{
		vec4 texColor0 = texture2D(videoTexture, texcoord);
		texColor0.rgb += stdBias;
		
		vec4 outColor = vec4(dot(stdR, texColor0.rgb),
                             dot(stdG, texColor0.rgb),
                             dot(stdB, texColor0.rgb), texColor0.a );
		return outColor;
	}

[Ref] name=ReinterpretColor

[Ref] name=AntialiasPixel
	
[Code]

    vec4 videoColor = vuya_texture_sampler(inputs.uv0.xy);
    videoColor = ReinterpretColor(videoColor);
	outputs.target0 = AntialiasPixel(videoColor, inputs.uv0.xy);


//////////////////////////////////////////////////////////

// This does YUYV to RGB conversion (ARGB3888, but  the width)
//		texel coords => | 0.0 0.5 | 1.0 1.5 | 2.0 2.5 | .......
//		flip 4:2:2 for inverting raster order
//
//		We sample UV step by 4 samples at a time yuyvyuyv and interp the UV's
//		We sampel Y 2 samples at a time yuyv an interp teh y's
//
//		get the pixels	BGRA | BGRA
//					    YUYV | YUYV

[Source] name=YUY2ToBGRAVideoPS
	
[Param]

    uniform sampler2D	videoTexture;
            
    in vec2 uv0;

    out vec4 target0;

[Param]

    uniform vec3 stdR; 
    uniform vec3 stdG;
    uniform vec3 stdB;
    uniform vec3 stdBias;

	uniform vec2		textureStep;
	uniform vec2		normalizedSize;
	uniform vec2		inverseTextureStep;
	uniform vec2		inverseNormalizedSize;
	uniform float		srcAlpha;
	uniform vec2		dstPixelSize;

[Function]

	vec2 ConvertNormalizedToRECTCoordinates(in vec2 texcoord)
	{
		return texcoord * inverseTextureStep * inverseNormalizedSize;
	}
	
	vec2 ConvertRECTToNormalizedCoordinates(in vec2 texcoord)
	{
		return texcoord * textureStep * normalizedSize;
	}
	
	vec4 myTexture2DRect(in vec2 texcoord)
	{
		return texture2D(videoTexture, ConvertRECTToNormalizedCoordinates(texcoord));
	}
	
	vec4 uyvy32_texture_sampler(in vec2 texcoord)
	{
		vec4 outColor;
		vec4 texColor0;
		vec4 texColor1;
		vec4 texColor2;
		vec4 texColor3;
		vec4 sampledYUV;
	
		vec2 tc0, tc1;
		vec2 tc2, tc3;
		float texelOffset;
		float uvBlend, yBlend;
		
		yBlend = fract(texcoord.x*0.5); 
		uvBlend = fract(texcoord.x*0.5); 
	
		texelOffset = 0.5;
	
		tc0.x = floor(texcoord.x * 0.5) * 2.0 + texelOffset;
		tc1.x = tc0.x + 1.0;
		tc2.x = tc0.x + 2.0;
		tc3.x = tc0.x + 3.0;
		
		tc0.x = min(tc0.x, inverseTextureStep.x - 2.0 + texelOffset);
		tc1.x = min(tc1.x, inverseTextureStep.x - 1.0 + texelOffset);
		tc2.x = min(tc2.x, inverseTextureStep.x - 2.0 + texelOffset);
		tc3.x = min(tc3.x, inverseTextureStep.x - 1.0 + texelOffset);
	
		tc0.y = texcoord.y;
		tc1.y = texcoord.y;
		tc2.y = texcoord.y;
		tc3.y = texcoord.y;
		
	
		texColor0 = myTexture2DRect(tc0);
		texColor1 = myTexture2DRect(tc1);
		texColor2 = myTexture2DRect(tc2);
		texColor3 = myTexture2DRect(tc3);
	
		sampledYUV.a = 1.0;
		sampledYUV.r = mix(texColor0.r, texColor1.r, yBlend);
		sampledYUV.g = mix(texColor0.g, texColor2.g, uvBlend);
		sampledYUV.b = mix(texColor1.g, texColor3.g, uvBlend);
		
		sampledYUV.rgb = sampledYUV.rgb + stdBias;
	
		outColor = vec4(dot(stdR, sampledYUV.rgb), dot(stdG, sampledYUV.rgb), dot(stdB, sampledYUV.rgb), 1.0 );
		return outColor;
	}
	
[Ref] name=ReinterpretColor

[Ref] name=AntialiasPixel

[Code]

    vec2 uv0 = inputs.uv0;
    uv0.y = 1 - uv0.y;

    vec4 videoColor = uyvy32_texture_sampler(uv0);
    videoColor = ReinterpretColor(videoColor);
	outputs.target0 = AntialiasPixel(videoColor, inputs.uv0.xy);


//////////////////////////////////////////////////////////


// This does UYVY to RGB conversion (ARGB3888, but  the width)
//		texel coords => | 0.0 0.5 | 1.0 1.5 | 2.0 2.5 | .......
//		flip 4:2:2 for inverting raster order
//
//		get the pixels	BGRA | BGRA
//					    UYVY | UYVY

[Source] name=UYVYToBGRAVideoPS

[Param]
    in vec2 uv0;
    out vec4 target0;

	uniform vec3	stdR;
	uniform vec3	stdG;
	uniform vec3	stdB;
	uniform vec3	stdBias;
	
	uniform vec2		textureStep;
	uniform vec2		normalizedSize;
	uniform vec2		inverseTextureStep;
	uniform vec2		inverseNormalizedSize;
	uniform sampler2D	videoTexture;
	uniform float		srcAlpha;
	uniform vec2		dstPixelSize;

[Function]

	vec2 ConvertNormalizedToRECTCoordinates(in vec2 texcoord)
	{
		return texcoord * inverseTextureStep * inverseNormalizedSize;
	}
	
	vec2 ConvertRECTToNormalizedCoordinates(in vec2 texcoord)
	{
		return texcoord * textureStep * normalizedSize;
	}
	
	vec4 myTexture2DRect(in vec2 texcoord)
	{
		return texture2D(videoTexture, ConvertRECTToNormalizedCoordinates(texcoord));
	}
	
	vec4 uyvy32_texture_sampler(in vec2 texcoord)
	{
		vec4 outColor;
		vec4 texColor0;
		vec4 texColor1;
		vec4 texColor2;
		vec4 texColor3;
		vec4 sampledYUV;
	
		vec2 tc0, tc1;
		vec2 tc2, tc3;
		float texelOffset;
		float uvBlend, yBlend;
		
		yBlend = fract(texcoord.x*0.5); 
		uvBlend = fract(texcoord.x*0.5); 
	
		texelOffset = 0.5;
	
		tc0.x = floor(texcoord.x * 0.5) * 2.0 + texelOffset;
		tc1.x = tc0.x + 1.0;
		tc2.x = tc0.x + 2.0;
		tc3.x = tc0.x + 3.0;
		
		tc0.x = min(tc0.x, inverseTextureStep.x - 2.0 + texelOffset);
		tc1.x = min(tc1.x, inverseTextureStep.x - 1.0 + texelOffset);
		tc2.x = min(tc2.x, inverseTextureStep.x - 2.0 + texelOffset);
		tc3.x = min(tc3.x, inverseTextureStep.x - 1.0 + texelOffset);
	
		tc0.y = texcoord.y;
		tc1.y = texcoord.y;
		tc2.y = texcoord.y;
		tc3.y = texcoord.y;
		
	
		texColor0 = myTexture2DRect(tc0);
		texColor1 = myTexture2DRect(tc1);
		texColor2 = myTexture2DRect(tc2);
		texColor3 = myTexture2DRect(tc3);
	
		sampledYUV.a = 1.0;
		sampledYUV.r = mix(texColor0.g, texColor1.g, yBlend);
		sampledYUV.g = mix(texColor0.r, texColor2.r, uvBlend);
		sampledYUV.b = mix(texColor1.r, texColor3.r, uvBlend);
		
		sampledYUV.rgb = sampledYUV.rgb + stdBias;
	
		outColor = vec4(dot(stdR, sampledYUV.rgb), dot(stdG, sampledYUV.rgb), dot(stdB, sampledYUV), 1.0 );
		return outColor;
	}

[Ref] name=ReinterpretColor

[Ref] name=AntialiasPixel

[Code]
	vec2 uv0 = inputs.uv0;
	
	vec2	invertedTexCoord;
	vec2	sampleCoordinate2D = uv0.xy;
	  
	invertedTexCoord.x = uv0.x;
	invertedTexCoord.y = ( -1.0 * uv0.y + 1.0 );
	
	sampleCoordinate2D = ConvertNormalizedToRECTCoordinates(invertedTexCoord * normalizedSize);	
	vec4 videoColor = uyvy32_texture_sampler(sampleCoordinate2D);
	videoColor = ReinterpretColor(videoColor);
	outputs.target0 = AntialiasPixel(videoColor, inputs.uv0.xy);



//////////////////////////////////////////////////////////


//		YCRCB 4:2:0 to 8-bit per component Texture2D BGRA Fragment Shader
//			later I need to fix to do proper chroma interpolation for interstitial/co-sited chromas
//			flip 4:2:0 for inverting raster order


[Source] name=420FrameToBGRAVideoPS

[Param]

    uniform sampler2D	videoTexture;
    uniform sampler2D	uTexture;
    uniform sampler2D	vTexture;
	uniform float		srcAlpha;
	uniform vec2		dstPixelSize;

    in vec2 uv0;

    out vec4 target0;

[Param]

    uniform vec3	stdR;
    uniform vec3	stdG;
    uniform vec3	stdB;
    uniform vec3	stdBias;

    // not used
    //uniform vec2	textureStep;
    //uniform vec2	normalizedSize;
    //uniform vec2	inverseTextureStep;
    //uniform vec2	inverseNormalizedSize;

[Function]

	vec3 yuv2rgb(vec3 yuv)
	{
		vec3 rgb;
	
		yuv += stdBias;
		rgb = vec3( dot(stdR, yuv),
                    dot(stdG, yuv),
                    dot(stdB, yuv) );
	
		return rgb;
	}
    
[Ref] name=ReinterpretColor

[Ref] name=AntialiasPixel
	
[Code]

    vec2 uv0 = inputs.uv0;
    uv0.y = 1 - uv0.y;

    float y = texture2D(videoTexture, uv0).x;
    float u = texture2D(uTexture, uv0).x;
    float v = texture2D(vTexture, uv0).x;

    vec4 videoColor = vec4(yuv2rgb(vec3(y, u, v)), 1.0);
    videoColor = ReinterpretColor(videoColor);
	outputs.target0 = AntialiasPixel(videoColor, inputs.uv0.xy);


//////////////////////////////////////////////////////////


//		YCRCB 4:2:0 to 8-bit per component Texture2D BGRA Fragment Shader
//			later I need to fix to do proper chroma interpolation for interstitial/co-sited chromas
//			flip 4:2:0 for inverting raster order

[Source] name=420FieldToBGRAVideoPS

[Param]

    uniform vec3	stdR;
    uniform vec3	stdG;
    uniform vec3	stdB;
    uniform vec3	stdBias;

    //uniform vec2	textureStep;
    //uniform vec2	normalizedSize;
    uniform vec2	inverseTextureStep;
    //uniform vec2	inverseNormalizedSize;

    uniform sampler2D videoTexture;
    
    uniform sampler2D	uTexture;
    uniform sampler2D	vTexture;
    uniform sampler2D	uAltFieldTexture;
    uniform sampler2D	vAltFieldTexture;
	uniform float		srcAlpha;
	uniform vec2		dstPixelSize;

    in vec2 uv0;

    out vec4 target0;

[Function]

	vec3 yuv2rgb(in vec3 yuv)
	{
		vec3 rgb;
	
		yuv += stdBias.rgb;
		rgb = vec3( dot(stdR, yuv),
                    dot(stdG, yuv),
                    dot(stdB, yuv) );
		
		return rgb;
	}
	
[Ref] name=ReinterpretColor

[Ref] name=AntialiasPixel

[Code]

    vec2 uv0 = inputs.uv0;
    uv0.y = 1 - uv0.y;

    float y = texture2D(videoTexture, uv0).x;

    // TODO: all 4 of these textures will be looked up, the branch isn't saving anything.
    //   All 4 texels in a 2x2 must take the same path for branch to skip.
    //   fields are by nature odd/even scanlines.  
    
    bool isField0 = (floor(fract(uv0.y * inverseTextureStep.y * 0.5) * 2.0) != 0.0);
    float u, v;
    if (isField0)
    {
        u = texture2D(uAltFieldTexture, uv0).x;
        v = texture2D(vAltFieldTexture, uv0).x;
    }
    else
    {
        u = texture2D(uTexture, uv0).x;
        v = texture2D(vTexture, uv0).x;
    }

    vec4 videoColor = vec4(yuv2rgb(vec3(y, u, v)), 1.0);
    videoColor = ReinterpretColor(videoColor);
	outputs.target0 = AntialiasPixel(videoColor, inputs.uv0.xy);

//////////////////////////////////////////////////////////

[Material]  name=MyDrawQuad vs=MyDrawQuadVS ps=MyDrawQuadPS

[Material]  name=MyDrawTexQuad vs=MyDrawQuadVS ps=MyDrawTexQuadPS

[Material]  name=MyDrawTexRedQuad vs=MyDrawQuadVS ps=MyDrawTexRedQuadPS

[Material]  name=Video vs=VideoVS ps=VideoPS

[Material]  name=BGRAToBGRAVideo vs=VideoVS ps=BGRAToBGRAVideoPS

[Material]  name=ARGBToBGRAVideo vs=VideoVS ps=ARGBToBGRAVideoPS

[Material]  name=VUYAToBGRAVideo vs=VideoVS ps=VUYAToBGRAVideoPS

[Material]  name=YUY2ToBGRAVideo vs=VideoVS ps=YUY2ToBGRAVideoPS

[Material]  name=UYVYToBGRAVideo vs=VideoVS ps=UYVYToBGRAVideoPS

[Material]  name=420FrameToBGRAVideo vs=VideoVS ps=420FrameToBGRAVideoPS

[Material]  name=420FieldToBGRAVideo vs=VideoVS ps=420FieldToBGRAVideoPS

