/*****************************************************************************/

// Header.

[Fragment] name=Helper

[Header]
    #ifndef qOpenGLES
    #define qOpenGLES 0
    #endif

    #ifndef qOpenGL
    #define qOpenGL 0
    #endif

    #ifndef qDirectX
    #define qDirectX 0
    #endif

	#if qDirectX
	// dx11 defaults to column-major, want row-major
    #pragma pack_matrix(row_major)
	#endif
	
	// pull this test in from fragment automatically
	//  don't include precision on VS, only on pS
	#if qOpenGLES || (qOpenGL == 210)
       
		#define rsqrt(x)    inversesqrt(x)
		#define saturate(x) clamp((x), 0.0, 1.0)
		#define clip(x)     if ((x) < 0.0) discard // ps
		#define lerp(x,y,t) mix((x),(y),(t))
        #define frac(x)     fract(x)
        #define mul(x, y)   ((x) * (y))

        // ES 2.0 has 3d texturing optional, emulate with 2D tex and Width added by splicer
        //#define texture3D(x, uv) texture2D((x), vec2(((uv).x + (uv).z) * x##Width, (uv).y)

        // just get it to compile under ES 2.0
		#define texture3D(t, uv) texture2D(t, (uv).xy)
		
		// this may need to be replaced by gpu_gen ?
		//  like sampler1D
        #define sampler3D sampler2D

        #define cbuffer uniform

	#elif (qOpenGL >= 320)
        
		// vs inout/ps input
		#define POSITION  0
		#define TEXCOORD0 1
		#define TEXCOORD1 2
		#define TEXCOORD2 3
		#define TEXCOORD3 4
		#define TEXCOORD4 5
		#define TEXCOORD5 6
		#define TEXCOORD6 7
		#define TANGENT   5
		#define NORMAL    6
		#define COLOR     7
		
		// consider stripping this from vs shaders
		// ps output
		#define TARGET0   0
		#define TARGET1   1
		#define TARGET2   2
		#define TARGET3   3
		#define TARGET4   4
		#define TARGET5   5
		#define TARGET6   6
		#define TARGET7   7
		
		#define cbuffer uniform
	
		// dx -> gl
		#define rsqrt(x)    inversesqrt(x)
		#define saturate(x) clamp((x), 0.0, 1.0)
		#define clip(x)     if ((x) < 0.0) discard // ps
		#define lerp(x,y,t) mix((x),(y),(t))
        #define frac(x)     fract(x)
        #define mul(x, y)   ((x) * (y))

		// ps
		#define texture2D(t, uv)   texture(t, uv)
		#define texture3D(t, uv)   texture(t, uv)
		#define textureCube(t, uv) texture(t, uv)
		
	#elif qDirectX
		#define mat4 float4x4
        #define mat3 float3x3
        #define mat2 float2x2

		#define vec4 float4
		#define vec3 float3 
		#define vec2 float2 
		
		// gl -> dx
		#define mix(x,y,t) lerp(x,y,t)
        #define fract(x)   frac(x)

		// ps
		#define texture2D(t, uv)   t.Sample(t##Sampler, uv)
		#define texture3D(t, uv)   t.Sample(t##Sampler, uv)
		#define textureCube(t, uv) t.Sample(t##Sampler, uv)

	#endif
	
	// gpu_sdk remaps 1D -> 2D textures, 3D -> 2D for some platforms
	#define texture1D(t, uv) texture2D(t, vec2(uv, 0.0))

/*****************************************************************************/
