[ShaderDX] name=SharedVS version=1000
#line 2
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float4x4 modelToWorldTfm ;
    float4x4 worldToViewTfm ;
    float4x4 viewToClipTfm ;
    float4x4 worldToClipTfm ;
    float4x4 modelToClipTfm ;
    float debugCycles ;
}
struct Inputs
{
    float4 position : POSITION ;
}
;
struct Outputs
{
    float4 positiont : SV_Position ;
}
;
void MainVS ( in Inputs inputs , out Outputs outputs )
{
    float4 pos = inputs . position ;
    pos = mul ( pos , modelToWorldTfm ) ;
    pos = mul ( pos , worldToViewTfm ) ;
    pos = mul ( pos , viewToClipTfm ) ;
    pos = mul ( pos , worldToClipTfm ) ;
    pos = mul ( pos , modelToClipTfm ) ;
    pos . x *= debugCycles ;
    outputs . positiont = pos ;
}
[ShaderDX] name=SharedPS version=1000
#line 35
    #pragma pack_matrix ( row_major )
struct Inputs
{
    float4 positiont : SV_Position ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = float4 ( 0 , 0 , 0 , 0 ) ;
}
[ShaderDX] name=TestShaderVS version=1000
#line 52
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float2 foregroundInverseSize ;
    float2 videoInverseSize ;
}
struct Inputs
{
    float4 position : POSITION ;
    float2 uv0 : TEXCOORD0 ;
    float2 uv1 : TEXCOORD1 ;
}
;
struct Outputs
{
    float4 positiont : SV_Position ;
    float2 uv0 : TEXCOORD0 ;
    float2 uv1 : TEXCOORD1 ;
}
;
void MainVS ( in Inputs inputs , out Outputs outputs )
{
    outputs . uv0 . xy = inputs . uv0 . xy * foregroundInverseSize ;
    outputs . uv1 . xy = inputs . uv1 . xy * videoInverseSize ;
    outputs . positiont = inputs . position ;
}
[ShaderDX] name=TestShaderPS version=1000
#line 80
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float2 foregroundInverseSize ;
    float2 videoInverseSize ;
}
uniform SamplerState colorTextureSampler ;
uniform Texture2D colorTexture ;
uniform SamplerState volumeTextureSampler ;
uniform Texture3D volumeTexture ;
struct Inputs
{
    float4 positiont : SV_Position ;
    float2 uv0 : TEXCOORD0 ;
    float2 uv1 : TEXCOORD1 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
float4 DoFunction1 ( in float4 val )
{
    return val ;
}
void DoFunction2 ( in float4 val , out float4 newval )
{
    newval = val ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = DoFunction1 ( float4 ( inputs . uv0 . xy , 0 , 1 ) ) ;
}
[ShaderDX] name=DrawQuadVS version=1000
#line 116
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float4x4 worldToClipTfm ;
}
struct Inputs
{
    float3 position : POSITION ;
    float2 uv0 : TEXCOORD0 ;
    float4 color : COLOR ;
}
;
struct Outputs
{
    float4 positiont : SV_Position ;
    float2 uv0 : TEXCOORD0 ;
    float4 color : COLOR ;
}
;
void MainVS ( in Inputs inputs , out Outputs outputs )
{
    outputs . uv0 = inputs . uv0 ;
    outputs . color = inputs . color ;
    outputs . positiont = float4 ( inputs . position , 1.0 ) ;
    outputs . positiont = mul ( outputs . positiont , worldToClipTfm ) ;
}
[ShaderDX] name=DrawQuadPS version=1000
#line 144
    #pragma pack_matrix ( row_major )
struct Inputs
{
    float4 positiont : SV_Position ;
    float2 uv0 : TEXCOORD0 ;
    float4 color : COLOR ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = inputs . color ;
}
[ShaderDX] name=DrawTexQuadPS version=1000
#line 163
    #pragma pack_matrix ( row_major )
uniform SamplerState colorTextureSampler ;
uniform Texture2D colorTexture ;
struct Inputs
{
    float4 positiont : SV_Position ;
    float2 uv0 : TEXCOORD0 ;
    float4 color : COLOR ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = inputs . color * colorTexture . Sample ( colorTextureSampler , inputs . uv0 ) ;
}
[ShaderDX] name=DrawTextPS version=1000
#line 184
    #pragma pack_matrix ( row_major )
uniform SamplerState colorTextureSampler ;
uniform Texture2D colorTexture ;
struct Inputs
{
    float4 positiont : SV_Position ;
    float2 uv0 : TEXCOORD0 ;
    float4 color : COLOR ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    float4 c = colorTexture . Sample ( colorTextureSampler , inputs . uv0 . xy ) ;
    c = c . rrrr ;
    clip ( c . a * inputs . color . a - 40.5 / 255.0 ) ;
    c *= inputs . color ;
    outputs . target0 = c ;
}
[ShaderDX] name=FallbackWorldVS version=1000
#line 209
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float4x4 worldToClipTfm ;
    float debugCycles ;
}
struct Inputs
{
    float3 position : POSITION ;
}
;
struct Outputs
{
    float4 positiont : SV_Position ;
    float4 color : COLOR ;
}
;
void MainVS ( in Inputs inputs , out Outputs outputs )
{
    outputs . positiont = float4 ( inputs . position . xyz , 1.0 ) ;
    outputs . positiont = mul ( outputs . positiont , worldToClipTfm ) ;
    outputs . color = float4 ( debugCycles + 0.2 , 0.0 , 0.0 , 1.0 ) ;
}
[ShaderDX] name=FallbackModelVS version=1000
#line 234
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float4x4 modelToClipTfm ;
    float debugCycles ;
}
struct Inputs
{
    float3 position : POSITION ;
}
;
struct Outputs
{
    float4 positiont : SV_Position ;
    float4 color : COLOR ;
}
;
void MainVS ( in Inputs inputs , out Outputs outputs )
{
    outputs . positiont = float4 ( inputs . position . xyz , 1.0 ) ;
    outputs . positiont = mul ( outputs . positiont , modelToClipTfm ) ;
    outputs . color = float4 ( debugCycles + 0.2 , 0.0 , 0.0 , 1.0 ) ;
}
[ShaderDX] name=FallbackPS version=1000
#line 259
    #pragma pack_matrix ( row_major )
struct Inputs
{
    float4 positiont : SV_Position ;
    float4 color : COLOR ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = inputs . color ;
}
[ShaderDX] name=DynDrawVDVS version=1000
#line 277
    #pragma pack_matrix ( row_major )
struct Inputs
{
    float3 position : POSITION ;
    float2 uv0 : TEXCOORD0 ;
    float4 color : COLOR ;
}
;
struct Outputs
{
    float4 positiont : SV_Position ;
}
;
void MainVS ( in Inputs inputs , out Outputs outputs )
{
    outputs . positiont = float4 ( inputs . position . xyz , 1.0 ) ;
}
[ShaderDX] name=AceVS version=1000
#line 296
    #pragma pack_matrix ( row_major )
struct Inputs
{
    float4 position : POSITION ;
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 positiont : SV_Position ;
    float2 uv0 : TEXCOORD0 ;
}
;
void MainVS ( in Inputs inputs , out Outputs outputs )
{
    outputs . positiont = inputs . position ;
    outputs . uv0 = inputs . uv0 ;
}
[ShaderDX] name=ACENOPTransformPS version=1000
#line 316
    #pragma pack_matrix ( row_major )
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
}
[ShaderDX] name=ACEOptimizedCMYKtoCMYKPS version=1000
#line 335
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptCMYKtoCMYK12TableSampler ;
uniform Texture3D ACEOptCMYKtoCMYK12Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptCMYKtoCMYK12 ( inout float4 color )
{
    float val = color . a * 15.0 ;
    float idx = floor ( val ) ;
    float frc = val - idx ;
    float2 rg = ( float2 ( 0.5 , 0.5 ) + 15.0 * color . rg ) / 16.0 ;
    float b0 = ( 0.5 + 15.0 * color . b + 16.0 * idx ) / 256.0 ;
    float b1 = b0 + 16.0 / 256.0 ;
    float4 k0 = float4 ( rg , b0 , 0.0 ) ;
    float4 k1 = float4 ( rg , b1 , 0.0 ) ;
    k0 = ACEOptCMYKtoCMYK12Table . Sample ( ACEOptCMYKtoCMYK12TableSampler , k0 . rgb ) ;
    k1 = ACEOptCMYKtoCMYK12Table . Sample ( ACEOptCMYKtoCMYK12TableSampler , k1 . rgb ) ;
    color = lerp ( k0 , k1 , frc ) ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptCMYKtoCMYK12 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedCMYKtoRGBPS version=1000
#line 371
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptCMYKtoRGB10TableSampler ;
uniform Texture3D ACEOptCMYKtoRGB10Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptCMYKtoRGB10 ( inout float4 color )
{
    float val = color . a * 8.0 ;
    float idx = floor ( val ) ;
    float frc = val - idx ;
    float2 rg = ( float2 ( 0.5 , 0.5 ) + 8.0 * color . rg ) / 16.0 ;
    float b0 = ( 0.5 + 8.0 * color . b + 9.0 * idx ) / 128.0 ;
    float b1 = b0 + 9.0 / 128.0 ;
    float3 k0 = float3 ( rg , b0 ) ;
    float3 k1 = float3 ( rg , b1 ) ;
    k0 = ACEOptCMYKtoRGB10Table . Sample ( ACEOptCMYKtoRGB10TableSampler , k0 ) . rgb ;
    k1 = ACEOptCMYKtoRGB10Table . Sample ( ACEOptCMYKtoRGB10TableSampler , k1 ) . rgb ;
    color . rgb = lerp ( k0 , k1 , frc ) ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptCMYKtoRGB10 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedCMYKtoXYZPS version=1000
#line 407
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptCMYKtoXYZ8CurvesSampler ;
uniform Texture2D ACEOptCMYKtoXYZ8Curves ;
uniform SamplerState ACEOptCMYKtoXYZ8TableSampler ;
uniform Texture3D ACEOptCMYKtoXYZ8Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptCMYKtoXYZ8 ( inout float4 color )
{
    color = ( float4 ( 0.5 , 0.5 , 0.5 , 0.5 ) + 255.0 * color ) / 256.0 ;
    color . r = ACEOptCMYKtoXYZ8Curves . Sample ( ACEOptCMYKtoXYZ8CurvesSampler , float2 ( color . r , 0.0 ) ) . r ;
    color . g = ACEOptCMYKtoXYZ8Curves . Sample ( ACEOptCMYKtoXYZ8CurvesSampler , float2 ( color . g , 0.0 ) ) . g ;
    color . b = ACEOptCMYKtoXYZ8Curves . Sample ( ACEOptCMYKtoXYZ8CurvesSampler , float2 ( color . b , 0.0 ) ) . b ;
    color . a = ACEOptCMYKtoXYZ8Curves . Sample ( ACEOptCMYKtoXYZ8CurvesSampler , float2 ( color . a , 0.0 ) ) . a ;
    float val = color . a * 8.0 ;
    float idx = floor ( val ) ;
    float frc = val - idx ;
    float2 rg = ( float2 ( 0.5 , 0.5 ) + 8.0 * color . rg ) / 16.0 ;
    float b0 = ( 0.5 + 8.0 * color . b + 9.0 * idx ) / 128.0 ;
    float b1 = b0 + 9.0 / 128.0 ;
    float3 k0 = float3 ( rg , b0 ) ;
    float3 k1 = float3 ( rg , b1 ) ;
    k0 = ACEOptCMYKtoXYZ8Table . Sample ( ACEOptCMYKtoXYZ8TableSampler , k0 . rgb ) . rgb ;
    k1 = ACEOptCMYKtoXYZ8Table . Sample ( ACEOptCMYKtoXYZ8TableSampler , k1 . rgb ) . rgb ;
    color . rgb = lerp ( k0 , k1 , frc ) ;
    color . r = ( color . r * 100.0 + 16.0 ) / 116.0 ;
    color . g = ( 255.0 * color . g - 128.0 ) / 500.0 ;
    color . b = ( 255.0 * color . b - 128.0 ) / 200.0 ;
    float3 xyz ;
    xyz . r = color . r + color . g ;
    xyz . g = color . r ;
    xyz . b = color . r - color . b ;
    float brk = 6.0 / 29.0 ;
    float scale = 108.0 / 841.0 ;
    float offset = 4.0 / 29.0 ;
    if ( xyz . r >= brk ) color . r = pow ( xyz . r , 3.0 ) ;
    else color . r = scale * ( xyz . r - offset ) ;
    if ( xyz . g >= brk ) color . g = pow ( xyz . g , 3.0 ) ;
    else color . g = scale * ( xyz . g - offset ) ;
    if ( xyz . b >= brk ) color . b = pow ( xyz . b , 3.0 ) ;
    else color . b = scale * ( xyz . b - offset ) ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptCMYKtoXYZ8 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedGamutCMYKPS version=1000
#line 466
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptCMYKGamut22TableSampler ;
uniform Texture3D ACEOptCMYKGamut22Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptCMYKGamut22 ( inout float4 color )
{
    float val = color . a * 15.0 ;
    float idx = floor ( val ) ;
    float frc = val - idx ;
    float2 rg = ( float2 ( 0.5 , 0.5 ) + 15.0 * color . rg ) / 16.0 ;
    float b0 = ( 0.5 + 15.0 * color . b + 16.0 * idx ) / 256.0 ;
    float b1 = b0 + 16.0 / 256.0 ;
    float4 k0 = float4 ( rg , b0 , 0.0 ) ;
    float4 k1 = float4 ( rg , b1 , 0.0 ) ;
    k0 = ACEOptCMYKGamut22Table . Sample ( ACEOptCMYKGamut22TableSampler , k0 . rgb ) ;
    k1 = ACEOptCMYKGamut22Table . Sample ( ACEOptCMYKGamut22TableSampler , k1 . rgb ) ;
    color = lerp ( k0 , k1 , frc ) ;
    color = ( color . r >= 128.0 / 255.0 ) ? float4 ( 1 , 1 , 1 , 1 ) : float4 ( 0 , 0 , 0 , 0 ) ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptCMYKGamut22 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedGamutRGBPS version=1000
#line 503
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptRGBGamut21TableSampler ;
uniform Texture3D ACEOptRGBGamut21Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptRGBGamut21 ( inout float4 color )
{
    color . rgb = saturate ( color . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 31.0 * color . rgb ) / 32.0 ;
    color . rgb = ACEOptRGBGamut21Table . Sample ( ACEOptRGBGamut21TableSampler , color . rgb ) . rgb ;
    color . rgb = ( color . r >= 128.0 / 255.0 ) ? float3 ( 1 , 1 , 1 ) : float3 ( 0 , 0 , 0 ) ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptRGBGamut21 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedGrayTo32PS version=1000
#line 532
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptGrayToRGB19CurvesSampler ;
uniform Texture2D ACEOptGrayToRGB19Curves ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptGrayToRGB19 ( inout float4 color )
{
    color . rgb = saturate ( color . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 255.0 * color . rgb ) / 256.0 ;
    color . rgb = ACEOptGrayToRGB19Curves . Sample ( ACEOptGrayToRGB19CurvesSampler , float2 ( color . r , 0.0 ) ) . rgb ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptGrayToRGB19 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedGrayToXYPS version=1000
#line 560
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptGrayToXYZ13CurvesSampler ;
uniform Texture2D ACEOptGrayToXYZ13Curves ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptGrayToXYZ13 ( inout float4 color )
{
    color . rgb = saturate ( color . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 255.0 * color . rgb ) / 256.0 ;
    color . rgb = ACEOptGrayToXYZ13Curves . Sample ( ACEOptGrayToXYZ13CurvesSampler , float2 ( color . r , 0.0 ) ) . rgb ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptGrayToXYZ13 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedLabGamutPS version=1000
#line 588
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptLabGamut20TableSampler ;
uniform Texture3D ACEOptLabGamut20Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptLabGamut20 ( inout float4 color )
{
    color . rgb = saturate ( color . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 32.0 * color . rgb ) / 64.0 ;
    color . rgb = ACEOptLabGamut20Table . Sample ( ACEOptLabGamut20TableSampler , color . rgb ) . rgb ;
    color . rgb = ( color . r >= 128.0 / 255.0 ) ? float3 ( 1 , 1 , 1 ) : float3 ( 0 , 0 , 0 ) ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptLabGamut20 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedLabToRGBPS version=1000
#line 617
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptLabToRGB2TableSampler ;
uniform Texture3D ACEOptLabToRGB2Table ;
uniform SamplerState ACEOptLabToRGB2CurvesSampler ;
uniform Texture2D ACEOptLabToRGB2Curves ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptLabToRGB2 ( inout float4 color )
{
    color . rgb = saturate ( color . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 24.0 * color . rgb ) / 32.0 ;
    color . rgb = ACEOptLabToRGB2Table . Sample ( ACEOptLabToRGB2TableSampler , color . rgb ) . rgb ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 255.0 * color . rgb ) / 256.0 ;
    color . r = ACEOptLabToRGB2Curves . Sample ( ACEOptLabToRGB2CurvesSampler , float2 ( color . r , 0.0 ) ) . r ;
    color . g = ACEOptLabToRGB2Curves . Sample ( ACEOptLabToRGB2CurvesSampler , float2 ( color . g , 0.0 ) ) . g ;
    color . b = ACEOptLabToRGB2Curves . Sample ( ACEOptLabToRGB2CurvesSampler , float2 ( color . b , 0.0 ) ) . b ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptLabToRGB2 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedMatrixRGBToRGB_1DPS version=1000
#line 651
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float3x3 ACEOptMatrixRGBtoRGB17Matrix ;
}
uniform SamplerState ACEOptMatrixRGBtoRGB17InCurvesSampler ;
uniform Texture2D ACEOptMatrixRGBtoRGB17InCurves ;
uniform SamplerState ACEOptMatrixRGBtoRGB17OutCurvesSampler ;
uniform Texture2D ACEOptMatrixRGBtoRGB17OutCurves ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptMatrixRGBtoRGB17 ( inout float4 color )
{
    color . rgb = saturate ( color . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 255.0 * color . rgb ) / 256.0 ;
    color . r = ACEOptMatrixRGBtoRGB17InCurves . Sample ( ACEOptMatrixRGBtoRGB17InCurvesSampler , float2 ( color . r , 0.0 ) ) . r ;
    color . g = ACEOptMatrixRGBtoRGB17InCurves . Sample ( ACEOptMatrixRGBtoRGB17InCurvesSampler , float2 ( color . g , 0.0 ) ) . g ;
    color . b = ACEOptMatrixRGBtoRGB17InCurves . Sample ( ACEOptMatrixRGBtoRGB17InCurvesSampler , float2 ( color . b , 0.0 ) ) . b ;
    color . rgb = mul ( ACEOptMatrixRGBtoRGB17Matrix , color . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 2047.0 * color . rgb ) / 2048.0 ;
    color . r = ACEOptMatrixRGBtoRGB17OutCurves . Sample ( ACEOptMatrixRGBtoRGB17OutCurvesSampler , float2 ( color . r , 0.0 ) ) . r ;
    color . g = ACEOptMatrixRGBtoRGB17OutCurves . Sample ( ACEOptMatrixRGBtoRGB17OutCurvesSampler , float2 ( color . g , 0.0 ) ) . g ;
    color . b = ACEOptMatrixRGBtoRGB17OutCurves . Sample ( ACEOptMatrixRGBtoRGB17OutCurvesSampler , float2 ( color . b , 0.0 ) ) . b ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptMatrixRGBtoRGB17 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedMatrixRGBToRGB_2DPS version=1000
#line 692
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float3x3 ACEOptMatrixRGBtoRGB17Matrix ;
}
uniform SamplerState ACEOptMatrixRGBtoRGB17InCurvesSampler ;
uniform Texture2D ACEOptMatrixRGBtoRGB17InCurves ;
uniform SamplerState ACEOptMatrixRGBtoRGB17OutCurvesSampler ;
uniform Texture2D ACEOptMatrixRGBtoRGB17OutCurves ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptMatrixRGBtoRGB17 ( inout float4 color )
{
    color . rgb = saturate ( color . rgb ) ;
    float texture_width = 16.0 , texture_height = 16.0 ;
    float3 y1 = floor ( ( 255.0 * color . rgb ) / texture_width ) ;
    float3 x1 = floor ( 255.0 * color . rgb ) - y1 * texture_width ;
    y1 = ( float3 ( 0.5 , 0.5 , 0.5 ) + y1 ) / texture_height ;
    x1 = ( float3 ( 0.5 , 0.5 , 0.5 ) + x1 ) / texture_width ;
    color . r = ACEOptMatrixRGBtoRGB17InCurves . Sample ( ACEOptMatrixRGBtoRGB17InCurvesSampler , float2 ( x1 . r , y1 . r ) ) . r ;
    color . g = ACEOptMatrixRGBtoRGB17InCurves . Sample ( ACEOptMatrixRGBtoRGB17InCurvesSampler , float2 ( x1 . g , y1 . g ) ) . g ;
    color . b = ACEOptMatrixRGBtoRGB17InCurves . Sample ( ACEOptMatrixRGBtoRGB17InCurvesSampler , float2 ( x1 . b , y1 . b ) ) . b ;
    color . rgb = mul ( ACEOptMatrixRGBtoRGB17Matrix , color . rgb ) ;
    texture_width = 64.0 ;
    texture_height = 32.0 ;
    float3 y2 = floor ( ( 2047.0 * color . rgb ) / texture_width ) ;
    float3 x2 = floor ( 2047.0 * color . rgb ) - y2 * texture_width ;
    y2 = ( float3 ( 0.5 , 0.5 , 0.5 ) + y2 ) / texture_height ;
    x2 = ( float3 ( 0.5 , 0.5 , 0.5 ) + x2 ) / texture_width ;
    color . r = ACEOptMatrixRGBtoRGB17OutCurves . Sample ( ACEOptMatrixRGBtoRGB17OutCurvesSampler , float2 ( x2 . r , y2 . r ) ) . r ;
    color . g = ACEOptMatrixRGBtoRGB17OutCurves . Sample ( ACEOptMatrixRGBtoRGB17OutCurvesSampler , float2 ( x2 . g , y2 . g ) ) . g ;
    color . b = ACEOptMatrixRGBtoRGB17OutCurves . Sample ( ACEOptMatrixRGBtoRGB17OutCurvesSampler , float2 ( x2 . b , y2 . b ) ) . b ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptMatrixRGBtoRGB17 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedMatrixRGBToXYZ_1DPS version=1000
#line 742
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float3x3 ACEOptMatrixRGBtoXYZ16Matrix ;
}
uniform SamplerState ACEOptMatrixRGBtoXYZ16CurvesSampler ;
uniform Texture2D ACEOptMatrixRGBtoXYZ16Curves ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptMatrixRGBtoXYZ16 ( inout float4 color )
{
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 255.0 * color . rgb ) / 256.0 ;
    color . r = ACEOptMatrixRGBtoXYZ16Curves . Sample ( ACEOptMatrixRGBtoXYZ16CurvesSampler , float2 ( color . r , 0.0 ) ) . r ;
    color . g = ACEOptMatrixRGBtoXYZ16Curves . Sample ( ACEOptMatrixRGBtoXYZ16CurvesSampler , float2 ( color . g , 0.0 ) ) . g ;
    color . b = ACEOptMatrixRGBtoXYZ16Curves . Sample ( ACEOptMatrixRGBtoXYZ16CurvesSampler , float2 ( color . b , 0.0 ) ) . b ;
    color . rgb = mul ( ACEOptMatrixRGBtoXYZ16Matrix , color . rgb ) ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptMatrixRGBtoXYZ16 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedMatrixRGBToXYZ_2DPS version=1000
#line 776
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float3x3 ACEOptMatrixRGBtoXYZ16Matrix ;
}
uniform SamplerState ACEOptMatrixRGBtoXYZ16CurvesSampler ;
uniform Texture2D ACEOptMatrixRGBtoXYZ16Curves ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptMatrixRGBtoXYZ16 ( inout float4 color )
{
    float texture_width = 16.0 , texture_height = 16.0 ;
    float3 y = floor ( ( 255.0 * color . rgb ) / texture_width ) ;
    float3 x = floor ( 255.0 * color . rgb ) - y * texture_width ;
    y = ( float3 ( 0.5 , 0.5 , 0.5 ) + y ) / texture_height ;
    x = ( float3 ( 0.5 , 0.5 , 0.5 ) + x ) / texture_width ;
    color . r = ACEOptMatrixRGBtoXYZ16Curves . Sample ( ACEOptMatrixRGBtoXYZ16CurvesSampler , float2 ( x . r , y . r ) ) . r ;
    color . g = ACEOptMatrixRGBtoXYZ16Curves . Sample ( ACEOptMatrixRGBtoXYZ16CurvesSampler , float2 ( x . g , y . g ) ) . g ;
    color . b = ACEOptMatrixRGBtoXYZ16Curves . Sample ( ACEOptMatrixRGBtoXYZ16CurvesSampler , float2 ( x . b , y . b ) ) . b ;
    color . rgb = mul ( ACEOptMatrixRGBtoXYZ16Matrix , color . rgb ) ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptMatrixRGBtoXYZ16 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedRGBToCMYKPS version=1000
#line 814
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptRGBtoCMYK11TableSampler ;
uniform Texture3D ACEOptRGBtoCMYK11Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptRGBtoCMYK11 ( inout float4 color )
{
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 24.0 * color . rgb ) / 32.0 ;
    color = ACEOptRGBtoCMYK11Table . Sample ( ACEOptRGBtoCMYK11TableSampler , color . rgb ) ;
    color = 1.0 - color ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptRGBtoCMYK11 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedRGBToGrayPS version=1000
#line 842
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptRGBtoGray6CurvesSampler ;
uniform Texture2D ACEOptRGBtoGray6Curves ;
uniform SamplerState ACEOptRGBtoGray6TableSampler ;
uniform Texture3D ACEOptRGBtoGray6Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptRGBtoGray6 ( inout float4 color )
{
    color . rgb = saturate ( color . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 255.0 * color . rgb ) / 256.0 ;
    color . r = ACEOptRGBtoGray6Curves . Sample ( ACEOptRGBtoGray6CurvesSampler , float2 ( color . r , 0.0 ) ) . r ;
    color . g = ACEOptRGBtoGray6Curves . Sample ( ACEOptRGBtoGray6CurvesSampler , float2 ( color . g , 0.0 ) ) . g ;
    color . b = ACEOptRGBtoGray6Curves . Sample ( ACEOptRGBtoGray6CurvesSampler , float2 ( color . b , 0.0 ) ) . b ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 15.0 * color . rgb ) / 16.0 ;
    color . rgb = ACEOptRGBtoGray6Table . Sample ( ACEOptRGBtoGray6TableSampler , color . rgb ) . rgb ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptRGBtoGray6 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedRGBToLabPS version=1000
#line 876
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptRGBtoLab3CurvesSampler ;
uniform Texture2D ACEOptRGBtoLab3Curves ;
uniform SamplerState ACEOptRGBtoLab3TableSampler ;
uniform Texture3D ACEOptRGBtoLab3Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptRGBtoLab3 ( inout float4 color )
{
    color . rgb = saturate ( color . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 255.0 * color . rgb ) / 256.0 ;
    color . r = ACEOptRGBtoLab3Curves . Sample ( ACEOptRGBtoLab3CurvesSampler , float2 ( color . r , 0.0 ) ) . r ;
    color . g = ACEOptRGBtoLab3Curves . Sample ( ACEOptRGBtoLab3CurvesSampler , float2 ( color . g , 0.0 ) ) . g ;
    color . b = ACEOptRGBtoLab3Curves . Sample ( ACEOptRGBtoLab3CurvesSampler , float2 ( color . b , 0.0 ) ) . b ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 24.0 * color . rgb ) / 32.0 ;
    color . rgb = ACEOptRGBtoLab3Table . Sample ( ACEOptRGBtoLab3TableSampler , color . rgb ) . rgb ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptRGBtoLab3 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedRGBToRGBPS version=1000
#line 910
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptRGBtoRGB1CurvesSampler ;
uniform Texture2D ACEOptRGBtoRGB1Curves ;
uniform SamplerState ACEOptRGBtoRGB1TableSampler ;
uniform Texture3D ACEOptRGBtoRGB1Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptRGBtoRGB1 ( inout float4 color )
{
    color . rgb = saturate ( color . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 255.0 * color . rgb ) / 256.0 ;
    color . r = ACEOptRGBtoRGB1Curves . Sample ( ACEOptRGBtoRGB1CurvesSampler , float2 ( color . r , 0.0 ) ) . r ;
    color . g = ACEOptRGBtoRGB1Curves . Sample ( ACEOptRGBtoRGB1CurvesSampler , float2 ( color . g , 0.0 ) ) . g ;
    color . b = ACEOptRGBtoRGB1Curves . Sample ( ACEOptRGBtoRGB1CurvesSampler , float2 ( color . b , 0.0 ) ) . b ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 15.0 * color . rgb ) / 16.0 ;
    color . rgb = ACEOptRGBtoRGB1Table . Sample ( ACEOptRGBtoRGB1TableSampler , color . rgb ) . rgb ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptRGBtoRGB1 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedRGBToRGBOverRangePS version=1000
#line 944
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptMatrixRGBtoRGBOverRange14CurvesSampler ;
uniform Texture2D ACEOptMatrixRGBtoRGBOverRange14Curves ;
uniform SamplerState ACEOptMatrixRGBtoRGBOverRange14TableSampler ;
uniform Texture3D ACEOptMatrixRGBtoRGBOverRange14Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptMatrixRGBtoRGBOverRange14 ( inout float4 color )
{
    color . rgb = clamp ( color . rgb , float3 ( - 1.0 , - 1.0 , - 1.0 ) , float3 ( 6.0 , 6.0 , 6.0 ) ) + 1.0 ;
    float4 idx = floor ( color ) ;
    float4 frc = color - idx ;
    float4 color0 = ( idx + 0.5 ) / 8.0 ;
    float4 color1 = ( idx + 1.5 ) / 8.0 ;
    color0 . r = ACEOptMatrixRGBtoRGBOverRange14Curves . Sample ( ACEOptMatrixRGBtoRGBOverRange14CurvesSampler , float2 ( color0 . r , 0.0 ) ) . r ;
    color0 . g = ACEOptMatrixRGBtoRGBOverRange14Curves . Sample ( ACEOptMatrixRGBtoRGBOverRange14CurvesSampler , float2 ( color0 . g , 0.0 ) ) . g ;
    color0 . b = ACEOptMatrixRGBtoRGBOverRange14Curves . Sample ( ACEOptMatrixRGBtoRGBOverRange14CurvesSampler , float2 ( color0 . b , 0.0 ) ) . b ;
    color1 . r = ACEOptMatrixRGBtoRGBOverRange14Curves . Sample ( ACEOptMatrixRGBtoRGBOverRange14CurvesSampler , float2 ( color1 . r , 0.0 ) ) . r ;
    color1 . g = ACEOptMatrixRGBtoRGBOverRange14Curves . Sample ( ACEOptMatrixRGBtoRGBOverRange14CurvesSampler , float2 ( color1 . g , 0.0 ) ) . g ;
    color1 . b = ACEOptMatrixRGBtoRGBOverRange14Curves . Sample ( ACEOptMatrixRGBtoRGBOverRange14CurvesSampler , float2 ( color1 . b , 0.0 ) ) . b ;
    color . rgb = lerp ( color0 . rgb , color1 . rgb , frc . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 31.0 * color . rgb ) / 32.0 ;
    color . rgb = ACEOptMatrixRGBtoRGBOverRange14Table . Sample ( ACEOptMatrixRGBtoRGBOverRange14TableSampler , color . rgb ) . rgb ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptMatrixRGBtoRGBOverRange14 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedRGBToXYZPS version=1000
#line 985
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptMatrixRGBtoXYZ5CurvesSampler ;
uniform Texture2D ACEOptMatrixRGBtoXYZ5Curves ;
uniform SamplerState ACEOptMatrixRGBtoXYZ5TableSampler ;
uniform Texture3D ACEOptMatrixRGBtoXYZ5Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptMatrixRGBtoXYZ5 ( inout float4 color )
{
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 255.0 * color . rgb ) / 256.0 ;
    color . r = ACEOptMatrixRGBtoXYZ5Curves . Sample ( ACEOptMatrixRGBtoXYZ5CurvesSampler , float2 ( color . r , 0.0 ) ) . r ;
    color . g = ACEOptMatrixRGBtoXYZ5Curves . Sample ( ACEOptMatrixRGBtoXYZ5CurvesSampler , float2 ( color . g , 0.0 ) ) . g ;
    color . b = ACEOptMatrixRGBtoXYZ5Curves . Sample ( ACEOptMatrixRGBtoXYZ5CurvesSampler , float2 ( color . b , 0.0 ) ) . b ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 24.0 * color . rgb ) / 32.0 ;
    color . rgb = ACEOptMatrixRGBtoXYZ5Table . Sample ( ACEOptMatrixRGBtoXYZ5TableSampler , color . rgb ) . rgb ;
    color . r = ( color . r * 100.0 + 16.0 ) / 116.0 ;
    color . g = ( 255.0 * color . g - 128.0 ) / 500.0 ;
    color . b = ( 255.0 * color . b - 128.0 ) / 200.0 ;
    float3 xyz ;
    xyz . r = color . r + color . g ;
    xyz . g = color . r ;
    xyz . b = color . r - color . b ;
    float brk = 6.0 / 29.0 ;
    float scale = 108.0 / 841.0 ;
    float offset = 4.0 / 29.0 ;
    if ( xyz . r >= brk ) color . r = pow ( xyz . r , 3.0 ) ;
    else color . r = scale * ( xyz . r - offset ) ;
    if ( xyz . g >= brk ) color . g = pow ( xyz . g , 3.0 ) ;
    else color . g = scale * ( xyz . g - offset ) ;
    if ( xyz . b >= brk ) color . b = pow ( xyz . b , 3.0 ) ;
    else color . b = scale * ( xyz . b - offset ) ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptMatrixRGBtoXYZ5 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedXYZToCMYKPS version=1000
#line 1034
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptXYZtoCMYK9TableSampler ;
uniform Texture3D ACEOptXYZtoCMYK9Table ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptXYZtoCMYK9 ( inout float4 color )
{
    float brk = 216.0 / 24389.0 ;
    float scale = 24389.0 / 27.0 ;
    if ( color . r >= brk ) color . r = 116.0 * pow ( color . r , 1.0 / 3.0 ) - 16.0 ;
    else color . r = color . r * scale ;
    if ( color . g >= brk ) color . g = 116.0 * pow ( color . g , 1.0 / 3.0 ) - 16.0 ;
    else color . g = color . g * scale ;
    if ( color . b >= brk ) color . b = 116.0 * pow ( color . b , 1.0 / 3.0 ) - 16.0 ;
    else color . b = color . b * scale ;
    float3 xyz ;
    xyz . r = color . g / 100.0 ;
    xyz . g = ( ( 500.0 / 116.0 ) * ( color . r - color . g ) + 128.0 ) / 255.0 ;
    xyz . b = ( ( 200.0 / 116.0 ) * ( color . g - color . b ) + 128.0 ) / 255.0 ;
    color . rgb = saturate ( xyz . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 24.0 * color . rgb ) / 32.0 ;
    color = ACEOptXYZtoCMYK9Table . Sample ( ACEOptXYZtoCMYK9TableSampler , color . rgb ) ;
    color = 1.0 - color ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptXYZtoCMYK9 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedXYZToMatrixRGB_1DPS version=1000
#line 1075
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float3x3 ACEOptXYZtoMatrixRGB15Matrix ;
}
uniform SamplerState ACEOptXYZtoMatrixRGB15CurvesSampler ;
uniform Texture2D ACEOptXYZtoMatrixRGB15Curves ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptXYZtoMatrixRGB15 ( inout float4 color )
{
    color . rgb = mul ( ACEOptXYZtoMatrixRGB15Matrix , color . rgb ) ;
    color . rgb = saturate ( color . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 2047.0 * color . rgb ) / 2048.0 ;
    color . r = ACEOptXYZtoMatrixRGB15Curves . Sample ( ACEOptXYZtoMatrixRGB15CurvesSampler , float2 ( color . r , 0.0 ) ) . r ;
    color . g = ACEOptXYZtoMatrixRGB15Curves . Sample ( ACEOptXYZtoMatrixRGB15CurvesSampler , float2 ( color . g , 0.0 ) ) . g ;
    color . b = ACEOptXYZtoMatrixRGB15Curves . Sample ( ACEOptXYZtoMatrixRGB15CurvesSampler , float2 ( color . b , 0.0 ) ) . b ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptXYZtoMatrixRGB15 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedXYZToMatrixRGB_2DPS version=1000
#line 1110
    #pragma pack_matrix ( row_major )
cbuffer params
{
    float3x3 ACEOptXYZtoMatrixRGB15Matrix ;
}
uniform SamplerState ACEOptXYZtoMatrixRGB15CurvesSampler ;
uniform Texture2D ACEOptXYZtoMatrixRGB15Curves ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptXYZtoMatrixRGB15 ( inout float4 color )
{
    color . rgb = mul ( ACEOptXYZtoMatrixRGB15Matrix , color . rgb ) ;
    color . rgb = saturate ( color . rgb ) ;
    float texture_width = 64.0 , texture_height = 32.0 ;
    float3 y = floor ( ( 2047.0 * color . rgb ) / texture_width ) ;
    float3 x = floor ( 2047.0 * color . rgb ) - y * texture_width ;
    y = ( float3 ( 0.5 , 0.5 , 0.5 ) + y ) / texture_height ;
    x = ( float3 ( 0.5 , 0.5 , 0.5 ) + x ) / texture_width ;
    color . r = ACEOptXYZtoMatrixRGB15Curves . Sample ( ACEOptXYZtoMatrixRGB15CurvesSampler , float2 ( x . r , y . r ) ) . r ;
    color . g = ACEOptXYZtoMatrixRGB15Curves . Sample ( ACEOptXYZtoMatrixRGB15CurvesSampler , float2 ( x . g , y . g ) ) . g ;
    color . b = ACEOptXYZtoMatrixRGB15Curves . Sample ( ACEOptXYZtoMatrixRGB15CurvesSampler , float2 ( x . b , y . b ) ) . b ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptXYZtoMatrixRGB15 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEOptimizedXYZToRGBPS version=1000
#line 1149
    #pragma pack_matrix ( row_major )
uniform SamplerState ACEOptXYZtoMatrixRGB4TableSampler ;
uniform Texture3D ACEOptXYZtoMatrixRGB4Table ;
uniform SamplerState ACEOptXYZtoMatrixRGB4CurvesSampler ;
uniform Texture2D ACEOptXYZtoMatrixRGB4Curves ;
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEOptXYZtoMatrixRGB4 ( inout float4 color )
{
    float brk = 216.0 / 24389.0 ;
    float scale = 24389.0 / 27.0 ;
    if ( color . r >= brk ) color . r = 116.0 * pow ( color . r , 1.0 / 3.0 ) - 16.0 ;
    else color . r = color . r * scale ;
    if ( color . g >= brk ) color . g = 116.0 * pow ( color . g , 1.0 / 3.0 ) - 16.0 ;
    else color . g = color . g * scale ;
    if ( color . b >= brk ) color . b = 116.0 * pow ( color . b , 1.0 / 3.0 ) - 16.0 ;
    else color . b = color . b * scale ;
    float3 xyz ;
    xyz . r = color . g / 100.0 ;
    xyz . g = ( ( 500.0 / 116.0 ) * ( color . r - color . g ) + 128.0 ) / 255.0 ;
    xyz . b = ( ( 200.0 / 116.0 ) * ( color . g - color . b ) + 128.0 ) / 255.0 ;
    color . rgb = saturate ( xyz . rgb ) ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 24.0 * color . rgb ) / 32.0 ;
    color . rgb = ACEOptXYZtoMatrixRGB4Table . Sample ( ACEOptXYZtoMatrixRGB4TableSampler , color . rgb ) . rgb ;
    color . rgb = ( float3 ( 0.5 , 0.5 , 0.5 ) + 255.0 * color . rgb ) / 256.0 ;
    color . r = ACEOptXYZtoMatrixRGB4Curves . Sample ( ACEOptXYZtoMatrixRGB4CurvesSampler , float2 ( color . r , 0.0 ) ) . r ;
    color . g = ACEOptXYZtoMatrixRGB4Curves . Sample ( ACEOptXYZtoMatrixRGB4CurvesSampler , float2 ( color . g , 0.0 ) ) . g ;
    color . b = ACEOptXYZtoMatrixRGB4Curves . Sample ( ACEOptXYZtoMatrixRGB4CurvesSampler , float2 ( color . b , 0.0 ) ) . b ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEOptXYZtoMatrixRGB4 ( outputs . target0 ) ;
}
[ShaderDX] name=ACEStandardLabToXYZPS version=1000
#line 1195
    #pragma pack_matrix ( row_major )
uniform SamplerState myTextureSampler ;
uniform Texture2D myTexture ;
struct Inputs
{
    float2 uv0 : TEXCOORD0 ;
}
;
struct Outputs
{
    float4 target0 : SV_Target0 ;
}
;
void ACEStdLabToXYZ7 ( inout float4 color )
{
    color . rgb = saturate ( color . rgb ) ;
    color . r = ( color . r * 100.0 + 16.0 ) / 116.0 ;
    color . g = ( 255.0 * color . g - 128.0 ) / 500.0 ;
    color . b = ( 255.0 * color . b - 128.0 ) / 200.0 ;
    float3 xyz ;
    xyz . r = color . r + color . g ;
    xyz . g = color . r ;
    xyz . b = color . r - color . b ;
    float brk = 6.0 / 29.0 ;
    float scale = 108.0 / 841.0 ;
    float offset = 4.0 / 29.0 ;
    if ( xyz . r >= brk ) color . r = pow ( xyz . r , 3.0 ) ;
    else color . r = scale * ( xyz . r - offset ) ;
    if ( xyz . g >= brk ) color . g = pow ( xyz . g , 3.0 ) ;
    else color . g = scale * ( xyz . g - offset ) ;
    if ( xyz . b >= brk ) color . b = pow ( xyz . b , 3.0 ) ;
    else color . b = scale * ( xyz . b - offset ) ;
}
void MainPS ( in Inputs inputs , out Outputs outputs )
{
    outputs . target0 = myTexture . Sample ( myTextureSampler , inputs . uv0 ) ;
    ACEStdLabToXYZ7 ( outputs . target0 ) ;
}
[Material]  name=Shared vs=SharedVS ps=SharedPS
[Material]  name=TestShader vs=TestShaderVS ps=TestShaderPS
[Material] name=DrawQuad vs=DrawQuadVS ps=DrawQuadPS
[Material] name=DrawTexQuad vs=DrawQuadVS ps=DrawTexQuadPS
[Material] name=DrawText vs=DrawQuadVS ps=DrawTextPS
[Material] name=FallbackWorld vs=FallbackWorldVS ps=FallbackPS
[Material] name=FallbackModel vs=FallbackModelVS ps=FallbackPS
[Material]  name=ACENOPTransform vs=AceVS ps=ACENOPTransformPS
[Material]  name=ACEOptimizedCMYKtoCMYK vs=AceVS ps=ACEOptimizedCMYKtoCMYKPS
[Material]  name=ACEOptimizedCMYKtoRGB vs=AceVS ps=ACEOptimizedCMYKtoRGBPS
[Material]  name=ACEOptimizedCMYKtoXYZ vs=AceVS ps=ACEOptimizedCMYKtoXYZPS
[Material]  name=ACEOptimizedGamutCMYK vs=AceVS ps=ACEOptimizedGamutCMYKPS
[Material]  name=ACEOptimizedGamutRGB vs=AceVS ps=ACEOptimizedGamutRGBPS
[Material]  name=ACEOptimizedGrayTo32 vs=AceVS ps=ACEOptimizedGrayTo32PS
[Material]  name=ACEOptimizedGrayToXY vs=AceVS ps=ACEOptimizedGrayToXYPS
[Material]  name=ACEOptimizedLabGamut vs=AceVS ps=ACEOptimizedLabGamutPS
[Material]  name=ACEOptimizedLabToRGB vs=AceVS ps=ACEOptimizedLabToRGBPS
[Material]  name=ACEOptimizedMatrixRGBToRGB_1D vs=AceVS ps=ACEOptimizedMatrixRGBToRGB_1DPS
[Material]  name=ACEOptimizedMatrixRGBToRGB_2D vs=AceVS ps=ACEOptimizedMatrixRGBToRGB_2DPS
[Material]  name=ACEOptimizedMatrixRGBToXYZ_1D vs=AceVS ps=ACEOptimizedMatrixRGBToXYZ_1DPS
[Material]  name=ACEOptimizedMatrixRGBToXYZ_2D vs=AceVS ps=ACEOptimizedMatrixRGBToXYZ_2DPS
[Material]  name=ACEOptimizedRGBToCMYK vs=AceVS ps=ACEOptimizedRGBToCMYKPS
[Material]  name=ACEOptimizedRGBToGray vs=AceVS ps=ACEOptimizedRGBToGrayPS
[Material]  name=ACEOptimizedRGBToLab vs=AceVS ps=ACEOptimizedRGBToLabPS
[Material]  name=ACEOptimizedRGBToRGB vs=AceVS ps=ACEOptimizedRGBToRGBPS
[Material]  name=ACEOptimizedRGBToRGBOverRange vs=AceVS ps=ACEOptimizedRGBToRGBOverRangePS
[Material]  name=ACEOptimizedRGBToXYZ vs=AceVS ps=ACEOptimizedRGBToXYZPS
[Material]  name=ACEOptimizedXYZToCMYK vs=AceVS ps=ACEOptimizedXYZToCMYKPS
[Material]  name=ACEOptimizedXYZToMatrixRGB_1D vs=AceVS ps=ACEOptimizedXYZToMatrixRGB_1DPS
[Material]  name=ACEOptimizedXYZToMatrixRGB_2D vs=AceVS ps=ACEOptimizedXYZToMatrixRGB_2DPS
[Material]  name=ACEOptimizedXYZToRGB vs=AceVS ps=ACEOptimizedXYZToRGBPS
[Material]  name=ACEStandardLabToXYZ vs=AceVS ps=ACEStandardLabToXYZPS
