// ----------------------------------------------------------------------------
//
// Copyright (C) 2012 Adobe Systems Incorporated
// All Rights Reserved.
//
// NOTICE:  All information contained  herein is,  and remains the property of
// Adobe Systems Incorporated and its suppliers, if any.  The intellectual and
// technical  concepts  contained  herein  are  proprietary  to  Adobe Systems
// Incorporated  and  its suppliers  and may  be covered  by U.S.  and Foreign
// Patents, patents in process, and are protected by trade secret or copyright
// law.  Dissemination of this information or reproduction of this material is
// strictly forbidden  unless prior  written permission is obtained from Adobe
// Systems Incorporated.
//
// ----------------------------------------------------------------------------

/////////////////////////////////////////////

// TODO: this needs to be in sep file (needed for every shader)
[Fragment] name=Helper

[Header]
    #ifndef qOpenGLES
    #define qOpenGLES 0
    #endif

    #ifndef qOpenGL
    #define qOpenGL 0
    #endif

    #ifndef qDirectX
    #define qDirectX 0
    #endif

	#if qDirectX
	// dx11 defaults to column-major, want row-major
    #pragma pack_matrix(row_major)
	#endif
	
	// pull this test in from fragment automatically
	//  don't include precision on VS, only on pS
	#if qOpenGLES || (qOpenGL == 210)
       
		#define rsqrt(x)    inversesqrt(x)
		#define saturate(x) clamp((x), 0.0, 1.0)
		#define clip(x)     if ((x) < 0.0) discard // ps
		#define lerp(x,y,t) mix((x),(y),(t))
        #define frac(x)     fract(x)
        #define mul(x, y)   ((x) * (y))

        // ES 2.0 has 3d texturing optional, emulate with 2D tex and Width added by splicer
        //#define texture3D(x, uv) texture2D((x), vec2(((uv).x + (uv).z) * x##Width, (uv).y)

        // just get it to compile under ES 2.0
		#define texture3D(t, uv) texture2D(t, (uv).xy)
		
		// this may need to be replaced by gpu_gen ?
		//  like sampler1D
        #define sampler3D sampler2D

        #define cbuffer uniform

	#elif (qOpenGL >= 320)
        
		// vs inout/ps input
		#define POSITION  0
		#define TEXCOORD0 1
		#define TEXCOORD1 2
		#define TEXCOORD2 3
		#define TEXCOORD3 4
		#define TEXCOORD4 5
		#define TEXCOORD5 6
		#define TEXCOORD6 7
		#define TANGENT   5
		#define NORMAL    6
		#define COLOR     7
		
		// consider stripping this from vs shaders
		// ps output
		#define TARGET0   0
		#define TARGET1   1
		#define TARGET2   2
		#define TARGET3   3
		#define TARGET4   4
		#define TARGET5   5
		#define TARGET6   6
		#define TARGET7   7
		
		#define cbuffer uniform
	
		// dx -> gl
		#define rsqrt(x)    inversesqrt(x)
		#define saturate(x) clamp((x), 0.0, 1.0)
		#define clip(x)     if ((x) < 0.0) discard // ps
		#define lerp(x,y,t) mix((x),(y),(t))
        #define frac(x)     fract(x)
        #define mul(x, y)   ((x) * (y))

		// ps
		#define texture2D(t, uv)   texture(t, uv)
		#define texture3D(t, uv)   texture(t, uv)
		#define textureCube(t, uv) texture(t, uv)
		
	#elif qDirectX
		#define mat4 float4x4
        #define mat3 float3x3
        #define mat2 float2x2

		#define vec4 float4
		#define vec3 float3 
		#define vec2 float2 
		
		// gl -> dx
		#define mix(x,y,t) lerp(x,y,t)
        #define fract(x)   frac(x)

		// ps
		#define texture2D(t, uv)   t.Sample(t##Sampler, uv)
		#define texture3D(t, uv)   t.Sample(t##Sampler, uv)
		#define textureCube(t, uv) t.Sample(t##Sampler, uv)

	#endif
	
	// gpu_sdk remaps 1D -> 2D textures, 3D -> 2D for some platforms
	#define texture1D(t, uv) texture2D(t, vec2(uv, 0.0))
	
/////////////////////////////////////////////

[Source] name=SharedVS

[Param]
    in vec4 position;
    out vec4 positiont;

    uniform mat4  modelToWorldTfm;
    uniform mat4  worldToViewTfm;
    uniform mat4  viewToClipTfm;
    uniform mat4  worldToClipTfm;
    uniform mat4  modelToClipTfm;
    uniform float debugCycles;

    //uniform bool  testB;
    //uniform float testF;
    //uniform int   testI;
    //uniform vec2  packTestV2;
    //uniform float packTestF;
    //uniform mat2  packTestM22;
	//uniform vec2  packTestM22V2;
	//uniform mat3  packTestM33;
	//uniform float packTestM33F;
[Code]
    vec4 pos = inputs.position;

    // keep the vars from being stripped
    pos = mul(pos, modelToWorldTfm);
    pos = mul(pos, worldToViewTfm);
    pos = mul(pos, viewToClipTfm);
    pos = mul(pos, worldToClipTfm);
    pos = mul(pos, modelToClipTfm);

    pos.x *= debugCycles;
    
    /*
    // test code ----------
    // gl and es just don't appear to like casts
    //pos.x *= (float)testB;
    //pos.x *= (float)testI;

    if (testB)
        pos.x *= 2.0;
    pos.x *= testF;

    if (testI > 0)
        pos.x *= 2.0;

    pos.xy *= packTestV2;
    pos.x *= packTestF;
	
    pos.xy = mul(pos.xy, packTestM22);
	pos.xy *= packTestM22V2;
	pos.xyz = mul(pos.xyz, packTestM33);
	pos.x *= packTestM33F;
	*/
    
    // does this need to use vars, so they're not stripped
    outputs.positiont = pos;


/////////////////////////////////////////////

[Source] name=SharedPS

[Param]
    in vec4 positiont; // not used, but needed to prevent empty inputs struct

    out vec4 target0;

[Code]

    outputs.target0 = vec4(0,0,0,0);

////////////////////////////////////////////////

[Material]  name=Shared vs=SharedVS ps=SharedPS

/////////////////////////////////////////////

[Fragment] name=TestFragment

[Header]

[Param]

[Function]

    vec4 DoFunction1(in vec4 val)
    {
        return val;
    }

    void DoFunction2(in vec4 val, out vec4 newval)
    {
        newval = val;
    }

[Code]


///////////////////////////////////////////

[Source] name=TestShaderVS

[Param]

    in vec4 position;  
    in vec2 uv0; 
    in vec2 uv1; 

    out vec4 positiont;
    out vec2 uv0;
    out vec2 uv1;

    uniform vec2 foregroundInverseSize;
    uniform vec2 videoInverseSize;
   
[Code]

	outputs.uv0.xy    = inputs.uv0.xy * foregroundInverseSize;
	outputs.uv1.xy    = inputs.uv1.xy * videoInverseSize;
	outputs.positiont = inputs.position; 

///////////////////////////////////////////

[Source] name=TestShaderPS

[Param]

    in vec4 positiont;
    in vec2 uv0;
    in vec2 uv1;

    out vec4 target0;

    uniform vec2 foregroundInverseSize;
    uniform vec2 videoInverseSize;

    uniform sampler2D colorTexture;
    uniform sampler3D volumeTexture;

[Ref] name=TestFragment

[Code]

	outputs.target0 = DoFunction1(vec4(inputs.uv0.xy,0,1));

////////////////////////////////////////////////

[Material]  name=TestShader vs=TestShaderVS ps=TestShaderPS


////////////////////////////////////////////////

[Source] name=DrawQuadVS

[Param]

    in vec3 position;  
    in vec2 uv0; 
    in vec4 color;

    out vec4 positiont; 
    out vec2 uv0;
    out vec4 color;

    uniform mat4 worldToClipTfm;

[Code]

	outputs.uv0       = inputs.uv0;
	outputs.color     = inputs.color;
	outputs.positiont = vec4(inputs.position, 1.0);
    outputs.positiont = mul(outputs.positiont, worldToClipTfm);

///////////////////////////////////////////

[Source] name=DrawQuadPS

[Param]

    in vec4 positiont;
    in vec2 uv0;
    in vec4 color;

    out vec4 target0;

[Code]

    outputs.target0 = inputs.color;

///////////////////////////////////////////

[Source] name=DrawTexQuadPS

[Param]

    in vec4 positiont;
    in vec2 uv0;
    in vec4 color;

    out vec4 target0;

    uniform sampler2D colorTexture;
    
[Code]

    outputs.target0 = inputs.color * texture2D(colorTexture, inputs.uv0);

///////////////////////////////////////////

// same as MyDrawTexQuad, but repeats red to all channels and does alpha test
//  used for debug text draw.  TODO: move to TestShaders.
[Source] name=DrawTextPS

[Param]

    in vec4 positiont;
    in vec2 uv0;
    in vec4 color;

    out vec4 target0;

	uniform sampler2D colorTexture;
	
[Code]
    vec4 c = texture2D(colorTexture, inputs.uv0.xy);
    c = c.rrrr; // TODO: handle rgba, rg (ra), and r
    
    clip(c.a * inputs.color.a - 40.5/255.0); // alpha test, must be negative to discard
    
    //c.rgb = 1.0;
    //c.a = 1.0;
    c *= inputs.color;
    
    outputs.target0 = c;

////////////////////////////////////////////////

[Material] name=DrawQuad vs=DrawQuadVS ps=DrawQuadPS

[Material] name=DrawTexQuad vs=DrawQuadVS ps=DrawTexQuadPS

[Material] name=DrawText vs=DrawQuadVS ps=DrawTextPS

////////////////////////////////////////////////

// The fallback material is used when a shader fails
//  This is a fallback using the worldToClip
[Source] name=FallbackWorldVS

[Param]

    in vec3 position;

    out vec4 positiont;
    out vec4 color;

    uniform mat4  worldToClipTfm;
    uniform float debugCycles;

[Code]

    outputs.positiont = vec4(inputs.position.xyz, 1.0);
    outputs.positiont = mul(outputs.positiont, worldToClipTfm);
    
    outputs.color = vec4(debugCycles + 0.2, 0.0, 0.0, 1.0);

////////////////////////////////////////////////

// The fallback material is used when a shader fails
//  This is a fallback using the modelToClip
[Source] name=FallbackModelVS

[Param]

    in vec3 position;
    
    out vec4 positiont;
    out vec4 color;
    
    uniform mat4  modelToClipTfm;
    uniform float debugCycles;

[Code]

    outputs.positiont = vec4(inputs.position.xyz, 1.0);
    outputs.positiont = mul(outputs.positiont, modelToClipTfm);
    
    outputs.color = vec4(debugCycles + 0.2, 0.0, 0.0, 1.0);

////////////////////////////////////////////////

[Source] name=FallbackPS

[Param]

    in vec4 positiont;
    in vec4 color;
    
    out vec4 target0;
     
[Code]
    outputs.target0 = inputs.color;

////////////////////////////////////////////////

[Material] name=FallbackWorld vs=FallbackWorldVS ps=FallbackPS

[Material] name=FallbackModel vs=FallbackModelVS ps=FallbackPS

////////////////////////////////////////////////

// This is only for DX decl support on DynDrawVD/gpu_vertex
[Source] name=DynDrawVDVS

[Param]

    in vec3 position;
    in vec2 uv0;
    in vec4 color;

    out vec4 positiont;

[Code]

	outputs.positiont = vec4(inputs.position.xyz, 1.0);

// No program or material for the DX decls
